/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.date;

import com.alee.extended.layout.ToolbarLayout;
import com.alee.laf.StyleConstants;
import com.alee.laf.button.WebButton;
import com.alee.laf.text.WebTextField;
import com.alee.managers.focus.FocusManager;
import com.alee.managers.focus.FocusTracker;
import com.alee.utils.LafUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.geom.Rectangle2D;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.List;

/**
 * User: mgarin Date: 03.11.11 Time: 10:24
 */

public class WebDateField extends JComponent
{
    public static final ImageIcon selectDateIcon =
            new ImageIcon ( WebDateField.class.getResource ( "icons/date.png" ) );

    private java.util.List<DateSelectionListener> dateSelectionListeners =
            new ArrayList<DateSelectionListener> ();

    private SimpleDateFormat dateFormat = new SimpleDateFormat ( "dd.MM.yyyy" );
    private Date date = null;

    private boolean focusOwner = false;

    private WebTextField dateField;
    private WebButton selectDate;

    public WebDateField ()
    {
        this ( new Date ( System.currentTimeMillis () ) );
    }

    public WebDateField ( Date initialDate )
    {
        super ();

        this.date = initialDate;

        setLayout ( new ToolbarLayout ( 0, 0, ToolbarLayout.HORIZONTAL ) );
        setOpaque ( false );
        setBackground ( Color.WHITE );
        setBorder ( BorderFactory
                .createEmptyBorder ( StyleConstants.shadeWidth + 2, StyleConstants.shadeWidth + 2,
                        StyleConstants.shadeWidth + 2, StyleConstants.shadeWidth + 2 ) );

        dateField = WebTextField.createWebTextField ( false );
        dateField.setFocusable ( true );
        dateField.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                setDateFromField ();
            }
        } );
        add ( dateField, ToolbarLayout.FILL );

        selectDate = WebButton.createIconWebButton ( selectDateIcon, StyleConstants.smallRound );
        selectDate.setDrawFocus ( false );
        selectDate.setDrawLeft ( false );
        selectDate.setDrawLeftLine ( true );
        selectDate.setRolloverDarkBorderOnly ( false );
        selectDate.putClientProperty ( ToolbarLayout.IGNORE_BORDER, true );
        selectDate.addActionListener ( new ActionListener ()
        {
            private JWindow popup;
            private WebCalendar calendar;

            public void actionPerformed ( ActionEvent e )
            {
                if ( popup == null || calendar == null )
                {
                    Window ancestor = SwingUtilities.getWindowAncestor ( WebDateField.this );

                    // Calendar
                    calendar = new WebCalendar ( date );
                    calendar.setRound ( StyleConstants.smallRound );
                    calendar.setShadeWidth ( 0 );

                    // Popup window
                    popup = new JWindow ( ancestor );
                    popup.getContentPane ().setLayout ( new BorderLayout () );
                    popup.getContentPane ().add ( calendar );

                    // Correct popup size
                    popup.pack ();

                    // Correct popup positioning
                    updatePopupLocation ();
                    ancestor.addComponentListener ( new ComponentAdapter ()
                    {
                        public void componentMoved ( ComponentEvent e )
                        {
                            if ( popup.isVisible () )
                            {
                                updatePopupLocation ();
                            }
                        }

                        public void componentResized ( ComponentEvent e )
                        {
                            if ( popup.isVisible () )
                            {
                                updatePopupLocation ();
                            }
                        }
                    } );

                    // Focus loss listener
                    FocusManager.registerFocusTracker ( new FocusTracker ()
                    {
                        private boolean focusOwner = false;

                        public Component getComponent ()
                        {
                            return calendar;
                        }

                        public boolean countChilds ()
                        {
                            return true;
                        }

                        public boolean isFocusOwner ()
                        {
                            return focusOwner;
                        }

                        public void focusChanged ( boolean focused )
                        {
                            focusOwner = focused;
                            if ( !focused )
                            {
                                popup.setVisible ( false );
                            }
                        }
                    } );

                    // Selection listener
                    calendar.addDateSelectionListener ( new DateSelectionListener ()
                    {
                        public void dateSelected ( Date date )
                        {
                            popup.setVisible ( false );
                            WebDateField.this.setDate ( date );
                        }
                    } );
                }
                else
                {
                    // Updating window location
                    updatePopupLocation ();

                    // Updating date
                    calendar.setDate ( date );
                }

                // Showing popup and changing focus
                popup.setVisible ( true );
                calendar.requestFocusInWindow ();
            }

            private void updatePopupLocation ()
            {
                Point los = WebDateField.this.getLocationOnScreen ();
                Rectangle gb = popup.getGraphicsConfiguration ().getBounds ();
                int x;
                if ( los.x + StyleConstants.shadeWidth + popup.getWidth () <= gb.x + gb.width )
                {
                    x = los.x + StyleConstants.shadeWidth;
                }
                else
                {
                    x = los.x + WebDateField.this.getWidth () - StyleConstants.shadeWidth -
                            popup.getWidth ();
                }
                int y;
                if ( los.y + WebDateField.this.getHeight () + popup.getHeight () <=
                        gb.y + gb.height )
                {
                    y = los.y + WebDateField.this.getHeight ();
                }
                else
                {
                    y = los.y - popup.getHeight ();
                }
                popup.setLocation ( x, y );
            }
        } );
        add ( selectDate, ToolbarLayout.END );

        // Adding focus listener
        FocusManager.registerFocusTracker ( new FocusTracker ()
        {
            public Component getComponent ()
            {
                return WebDateField.this;
            }

            public boolean countChilds ()
            {
                return true;
            }

            public boolean isFocusOwner ()
            {
                return focusOwner;
            }

            public void focusChanged ( boolean focused )
            {
                focusOwner = focused;
                if ( !focusOwner )
                {
                    setDateFromField ();
                }
                WebDateField.this.repaint ();
            }
        } );

        updateDateText ();
    }

    private void setDateFromField ()
    {
        try
        {
            Date newDate = dateFormat.parse ( dateField.getText () );
            setDate ( newDate );
        }
        catch ( Throwable ex )
        {
            setDate ( date );
        }
    }

    private void updateDateText ()
    {
        dateField.setText ( dateFormat.format ( date ) );
    }

    public Date getDate ()
    {
        return date;
    }

    public void setDate ( Date date )
    {
        if ( this.date.getTime () != date.getTime () )
        {
            this.date = date;
            updateDateText ();
            fireDateSelected ( date );
        }
    }

    public SimpleDateFormat getDateFormat ()
    {
        return dateFormat;
    }

    public void setDateFormat ( SimpleDateFormat dateFormat )
    {
        this.dateFormat = dateFormat;
        updateDateText ();
    }

    protected void paintComponent ( Graphics g )
    {
        super.paintComponent ( g );

        // Styled border
        LafUtils.drawWebBorder ( ( Graphics2D ) g, this, StyleConstants.shadeColor,
                StyleConstants.shadeWidth, StyleConstants.smallRound );
    }

    protected void paintChildren ( Graphics g )
    {
        // Fixed childs clip
        Shape old = LafUtils.intersectClip ( g,
                new Rectangle2D.Double ( StyleConstants.shadeWidth + 1, 0,
                        getWidth () - StyleConstants.shadeWidth + 1, getHeight () ) );
        super.paintChildren ( g );
        LafUtils.restoreClip ( g, old );

        // Focus over the childs
        LafUtils.drawWebFocus ( ( Graphics2D ) g, this, StyleConstants.focusType,
                StyleConstants.shadeWidth, StyleConstants.smallRound, null, focusOwner );
    }

    private void fireDateSelected ( Date date )
    {
        List<DateSelectionListener> listeners = new ArrayList<DateSelectionListener> ();
        listeners.addAll ( dateSelectionListeners );
        for ( DateSelectionListener listener : listeners )
        {
            listener.dateSelected ( date );
        }
        listeners.clear ();
    }
}
