/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.filechooser;

import com.alee.extended.filefilter.DefaultFileFilter;
import com.alee.extended.layout.ToolbarLayout;
import com.alee.extended.panel.BackgroundPanel;
import com.alee.laf.GlobalConstants;
import com.alee.laf.StyleConstants;
import com.alee.laf.button.WebButton;
import com.alee.laf.optionpane.WebOptionPane;
import com.alee.laf.scroll.WebScrollPane;
import com.alee.laf.separator.WebSeparator;
import com.alee.laf.toolbar.ToolbarStyle;
import com.alee.laf.toolbar.WebToolBar;
import com.alee.managers.hotkey.Hotkey;
import com.alee.managers.hotkey.HotkeyManager;
import com.alee.managers.hotkey.HotkeyRunnable;
import com.alee.managers.tooltip.TooltipManager;
import com.alee.managers.tooltip.TooltipWay;
import com.alee.utils.FileUtils;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeSelectionModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * User: mgarin Date: 13.10.11 Time: 4:08
 */

public class WebDirectoryChooserPanel extends BackgroundPanel
{
    private static final ImageIcon FOLDER_UP_ICON =
            new ImageIcon ( WebDirectoryChooserPanel.class.getResource ( "icons/folder_up.png" ) );
    private static final ImageIcon FOLDER_HOME_ICON = new ImageIcon (
            WebDirectoryChooserPanel.class.getResource ( "icons/folder_home.png" ) );
    private static final ImageIcon FOLDER_NEW_ICON =
            new ImageIcon ( WebDirectoryChooserPanel.class.getResource ( "icons/folder_new.png" ) );
    private static final ImageIcon REFRESH_ICON =
            new ImageIcon ( WebDirectoryChooserPanel.class.getResource ( "icons/refresh.png" ) );
    private static final ImageIcon REMOVE_ICON =
            new ImageIcon ( WebDirectoryChooserPanel.class.getResource ( "icons/remove.png" ) );

    public static DefaultFileFilter defaultFileFilter = GlobalConstants.DIRECTORIES_FILTER;

    private List<WebDirectoryChooserListener> listeners =
            new ArrayList<WebDirectoryChooserListener> ();

    private File selectedFolder = FileUtils.getRoots ()[ 0 ];

    private WebPathField webPathField;

    private WebFileTree fileTree;
    private TreeSelectionListener fileTreeListener;

    private WebButton folderUp;
    private WebButton folderHome;
    private WebButton folderNew;
    private WebButton refresh;
    private WebButton remove;

    public WebDirectoryChooserPanel ()
    {
        super ();

        // Panel settings
        setOpaque ( true );

        // Controls pane
        WebToolBar contolsToolbar = new WebToolBar ( WebToolBar.HORIZONTAL );
        contolsToolbar.setToolbarStyle ( ToolbarStyle.attached );
        contolsToolbar.setFloatable ( false );

        folderUp = WebButton.createIconWebButton ( FOLDER_UP_ICON, StyleConstants.smallRound,
                StyleConstants.shadeWidth, StyleConstants.innerShadeWidth, true );
        HotkeyManager.registerHotkey ( WebDirectoryChooserPanel.this, folderUp, Hotkey.ALT_UP,
                new HotkeyRunnable ()
                {
                    public void run ( KeyEvent e )
                    {
                        if ( allowHotkeys () )
                        {
                            folderUp.doClick ();
                        }
                    }
                } );
        TooltipManager.setTooltip ( folderUp, UIManager.getString ( "WebFileChooser.folderup" ) );
        folderUp.setFocusable ( false );
        folderUp.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                if ( selectedFolder != null )
                {
                    updateShownFolder ( selectedFolder.getParentFile (), true, true );
                }
            }
        } );
        contolsToolbar.add ( folderUp );

        contolsToolbar.add ( new WebSeparator ( WebSeparator.VERTICAL ) );

        folderHome = WebButton.createIconWebButton ( FOLDER_HOME_ICON, StyleConstants.smallRound,
                StyleConstants.shadeWidth, StyleConstants.innerShadeWidth, true );
        HotkeyManager.registerHotkey ( WebDirectoryChooserPanel.this, folderHome, Hotkey.ALT_H,
                new HotkeyRunnable ()
                {
                    public void run ( KeyEvent e )
                    {
                        if ( allowHotkeys () )
                        {
                            folderHome.doClick ();
                        }
                    }
                }, TooltipWay.right );
        TooltipManager.setTooltip ( folderHome, UIManager.getString ( "WebFileChooser.home" ) );
        folderHome.setFocusable ( false );
        folderHome.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                updateShownFolder ( FileUtils.getUserHomeFile (), true, true );
            }
        } );
        contolsToolbar.add ( folderHome );

        contolsToolbar.add ( new WebSeparator ( WebSeparator.VERTICAL ) );

        for ( final File file : FileUtils.getRoots () )
        {
            WebButton root = WebButton.createIconWebButton ( FileUtils.getFileIcon ( file ),
                    StyleConstants.smallRound, StyleConstants.shadeWidth,
                    StyleConstants.innerShadeWidth, true );
            TooltipManager.setTooltip ( root, FileUtils.getDisplayFileName ( file ) );
            root.setFocusable ( false );
            root.addActionListener ( new ActionListener ()
            {
                public void actionPerformed ( ActionEvent e )
                {
                    updateShownFolder ( file, true, true );
                }
            } );
            contolsToolbar.add ( root );
        }


        refresh = WebButton.createIconWebButton ( REFRESH_ICON, StyleConstants.smallRound,
                StyleConstants.shadeWidth, StyleConstants.innerShadeWidth, true );
        HotkeyManager.registerHotkey ( WebDirectoryChooserPanel.this, refresh, Hotkey.F5,
                new HotkeyRunnable ()
                {
                    public void run ( KeyEvent e )
                    {
                        if ( allowHotkeys () )
                        {
                            refresh.doClick ();
                        }
                    }
                }, TooltipWay.left );
        TooltipManager.setTooltip ( refresh, UIManager.getString ( "WebFileChooser.refresh" ) );
        refresh.setFocusable ( false );
        refresh.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                fileTree.getModel ().removeFilesCache ( selectedFolder );
                updateShownFolder ( selectedFolder, true, true );
            }
        } );
        contolsToolbar.add ( refresh, ToolbarLayout.END );

        contolsToolbar.add ( new WebSeparator ( WebSeparator.VERTICAL ), ToolbarLayout.END );

        folderNew = WebButton.createIconWebButton ( FOLDER_NEW_ICON, StyleConstants.smallRound,
                StyleConstants.shadeWidth, StyleConstants.innerShadeWidth, true );
        HotkeyManager.registerHotkey ( WebDirectoryChooserPanel.this, folderNew, Hotkey.CTRL_N,
                new HotkeyRunnable ()
                {
                    public void run ( KeyEvent e )
                    {
                        if ( allowHotkeys () )
                        {
                            folderNew.doClick ();
                        }
                    }
                } );
        TooltipManager.setTooltip ( folderNew, UIManager.getString ( "WebFileChooser.newfolder" ) );
        folderNew.setFocusable ( false );
        folderNew.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                if ( selectedFolder != null )
                {
                    File file = new File ( selectedFolder, FileUtils.getFreeName ( selectedFolder,
                            UIManager.getString ( "WebFileChooser.newfolder.name" ) ) );
                    if ( file.mkdir () )
                    {
                        // Updating filestree
                        fileTree.getModel ().addFile ( selectedFolder, file );
                        fileTree.updateUI ();

                        // Selecting added folder
                        fileTree.setSelectedFile ( file );

                        // Starting folder name editing
                        fileTree.startEditingAtPath ( fileTree.getSelectionPath () );
                    }
                    else
                    {
                        WebOptionPane.showMessageDialog ( WebDirectoryChooserPanel.this,
                                UIManager.getString ( "WebFileChooser.newfolder.error.text" ),
                                UIManager.getString ( "WebFileChooser.newfolder.error.title" ),
                                WebOptionPane.ERROR_MESSAGE );
                    }
                }
            }
        } );
        contolsToolbar.add ( folderNew, ToolbarLayout.END );

        remove = WebButton.createIconWebButton ( REMOVE_ICON, StyleConstants.smallRound,
                StyleConstants.shadeWidth, StyleConstants.innerShadeWidth, true );
        HotkeyManager.registerHotkey ( WebDirectoryChooserPanel.this, remove, Hotkey.DELETE,
                new HotkeyRunnable ()
                {
                    public void run ( KeyEvent e )
                    {
                        if ( allowHotkeys () )
                        {
                            remove.doClick ();
                        }
                    }
                } );
        TooltipManager.setTooltip ( remove, UIManager.getString ( "WebFileChooser.delete" ) );
        remove.setFocusable ( false );
        remove.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                File file = fileTree.getSelectedFile ();
                if ( file == null )
                {
                    return;
                }

                int confirm = WebOptionPane.showConfirmDialog ( WebDirectoryChooserPanel.this,
                        "<html>" + UIManager
                                .getString ( "WebDirectoryChooser.delete.confirm.singledir" ) +
                                " <b>" + file.getName () + "</b>?</html>",
                        UIManager.getString ( "WebFileChooser.delete.confirm.title" ),
                        WebOptionPane.YES_NO_OPTION, WebOptionPane.QUESTION_MESSAGE );

                if ( confirm == WebOptionPane.YES_OPTION )
                {
                    FileUtils.removeFile ( file );

                    fileTree.getModel ().removeFilesCache ( file );
                    fileTree.updateUI ();

                    updateShownFolder ( selectedFolder, true, true );
                }
            }
        } );
        contolsToolbar.add ( remove, ToolbarLayout.END );


        // Path field
        webPathField = new WebPathField ( selectedFolder );
        webPathField.setFileFilter ( defaultFileFilter );
        webPathField.addWebPathFieldListener ( new WebPathFieldListener ()
        {
            public void directoryChanged ( File newDirectory )
            {
                updateShownFolder ( webPathField.getSelectedPath (), false, true );
            }
        } );

        // Files tree
        fileTree = new WebFileTree ();
        fileTree.setVisibleRowCount ( 15 );
        fileTree.setFileFilter ( defaultFileFilter );
        fileTree.setSelectedFile ( selectedFolder, true );
        fileTree.setSelectionMode ( TreeSelectionModel.SINGLE_TREE_SELECTION );
        fileTreeListener = new TreeSelectionListener ()
        {
            public void valueChanged ( TreeSelectionEvent e )
            {
                updateShownFolder ( fileTree.getSelectedFile (), true, false );
            }
        };
        fileTree.addTreeSelectionListener ( fileTreeListener );
        WebScrollPane treeScroll = new WebScrollPane ( fileTree )
        {
            public Dimension getPreferredSize ()
            {
                Dimension ps = super.getPreferredSize ();
                ps.width = 400;
                return ps;
            }
        };

        // For toolbar updates
        fileTree.addTreeSelectionListener ( new TreeSelectionListener ()
        {
            public void valueChanged ( TreeSelectionEvent e )
            {
                updateToolbar ();
            }
        } );

        // Drag and drop tip
        //        WebTipLabel dndTip = new WebTipLabel ( "WebDirectoryChooser.drag",
        //                UIManager.getString ( "WebDirectoryChooser.drag" ), WebTipLabel.CENTER );

        // Updating toolbar
        updateToolbar ();

        // Panel content
        setLayout ( new BorderLayout ( 0, 3 ) );

        add ( contolsToolbar, BorderLayout.NORTH );

        JPanel panel = new JPanel ( new BorderLayout ( 0, 1 ) );
        panel.setBorder ( BorderFactory.createEmptyBorder ( 0, 3, 2, 3 ) );
        panel.add ( webPathField, BorderLayout.NORTH );
        panel.add ( treeScroll, BorderLayout.CENTER );
        //        panel.add ( dndTip, BorderLayout.SOUTH );
        add ( panel, BorderLayout.CENTER );
    }

    private void updateToolbar ()
    {
        File selectedFile = fileTree.getSelectedFile ();

        folderUp.setEnabled ( selectedFile != null && selectedFile.getParentFile () != null );

        folderNew.setEnabled ( selectedFile != null && selectedFile.canWrite () );
        remove.setEnabled ( selectedFile != null && selectedFile.getParentFile () != null &&
                selectedFile.canWrite () );
    }

    public boolean allowHotkeys ()
    {
        return !fileTree.isEditing () && !webPathField.isEditing () /*&& !fileList.isEditing()*/;
    }

    private void updateShownFolder ( File folder, boolean updatePath, boolean updateTree )
    {
        this.selectedFolder = folder;

        if ( updatePath )
        {
            webPathField.setSelectedPath ( folder );
        }

        if ( updateTree )
        {
            fileTree.removeTreeSelectionListener ( fileTreeListener );
            fileTree.setSelectedFile ( folder, true );
            fileTree.addTreeSelectionListener ( fileTreeListener );
        }

        fireSelectionChanged ( this.selectedFolder );
    }

    public File getSelectedFolder ()
    {
        return selectedFolder;
    }

    public void setSelectedFolder ( File selectedFolder )
    {
        updateShownFolder ( selectedFolder, true, true );
    }

    public void addWebDirectoryChooserListener ( WebDirectoryChooserListener listener )
    {
        listeners.add ( listener );
    }

    public void removeWebDirectoryChooserListener ( WebDirectoryChooserListener listener )
    {
        listeners.remove ( listener );
    }

    private void fireSelectionChanged ( File file )
    {
        for ( WebDirectoryChooserListener listener : listeners )
        {
            listener.selectionChanged ( file );
        }
    }
}
