/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.filechooser;

import com.alee.extended.drag.FileDropHandler;
import com.alee.extended.filefilter.DefaultFileFilter;
import com.alee.extended.layout.HorizontalFlowLayout;
import com.alee.laf.StyleConstants;
import com.alee.laf.button.WebButton;
import com.alee.laf.scroll.WebScrollPane;
import com.alee.managers.focus.FocusManager;
import com.alee.managers.focus.FocusTracker;
import com.alee.utils.FileUtils;
import com.alee.utils.LafUtils;

import javax.swing.*;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * User: mgarin Date: 07.08.11 Time: 15:44
 */

public class WebFileChooserField extends JComponent
{
    public static final ImageIcon CROSS_ICON =
            new ImageIcon ( WebFileChooserField.class.getResource ( "icons/cross.png" ) );

    private List<FilesSelectionListener> listeners = new ArrayList<FilesSelectionListener> ();

    private boolean focusOwner = false;

    private List<File> selectedFiles = new ArrayList<File> ();

    private SelectionMode selectionMode = SelectionMode.MULTIPLE_SELECTION;
    private int preferredWidth = -1;
    private boolean showFileShortName = true;
    private boolean showFileIcon = true;
    private boolean showRemoveButton = true;
    private boolean showFileExtensions = false;
    private boolean showChooseButton = true;
    private boolean filesDropEnabled = true;

    private WebFileChooser webFileChooser = null;

    private JPanel contentPanel;
    private WebScrollPane scroll;
    private WebButton chooseButton;

    public WebFileChooserField ()
    {
        this ( null );
    }

    public WebFileChooserField ( Window parent )
    {
        this ( parent, true );
    }

    public WebFileChooserField ( boolean showChooseButton )
    {
        this ( null, showChooseButton );
    }

    public WebFileChooserField ( Window owner, boolean showChooseButton )
    {
        super ();

        this.showChooseButton = showChooseButton;

        setLayout ( new BorderLayout ( 0, 0 ) );
        //        setBorder ( BorderFactory.createEmptyBorder ( 2, 2, 2, 2 ) );
        setOpaque ( false );
        setBackground ( Color.WHITE );

        contentPanel = new JPanel ();
        contentPanel.setOpaque ( false );
        //        contentPanel.setBorder ( BorderFactory.createEmptyBorder ( 2, 2, 2, 1 ) );

        // Files TransferHandler
        setTransferHandler ( new FileDropHandler ()
        {
            protected boolean isDropEnabled ()
            {
                return filesDropEnabled;
            }

            protected boolean filesImported ( List<File> files )
            {
                // Setting dragged files
                setSelectedFiles ( files );
                return getSelectedFiles ().size () > 0;
            }
        } );

        // For wide content scrolling
        contentPanel.addMouseWheelListener ( new MouseWheelListener ()
        {
            public void mouseWheelMoved ( MouseWheelEvent e )
            {
                Rectangle vr = contentPanel.getVisibleRect ();
                contentPanel.scrollRectToVisible (
                        new Rectangle ( vr.x + e.getWheelRotation () * 25, vr.y, vr.width,
                                vr.height ) );
            }
        } );

        scroll = new WebScrollPane ( contentPanel, false )
        {
            public Dimension getPreferredSize ()
            {
                Dimension ps = super.getPreferredSize ();
                ps.height = contentPanel.getPreferredSize ().height;
                return ps;
            }
        };
        scroll.setHorizontalScrollBarPolicy ( WebScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        scroll.setVerticalScrollBarPolicy ( WebScrollPane.VERTICAL_SCROLLBAR_NEVER );
        add ( scroll, BorderLayout.CENTER );

        if ( this.showChooseButton )
        {
            webFileChooser =
                    new WebFileChooser ( owner, UIManager.getString ( "WebFileChooser.title" ) );
            webFileChooser.setSelectionMode ( selectionMode );
            webFileChooser.setOkListener ( new ActionListener ()
            {
                public void actionPerformed ( ActionEvent e )
                {
                    // Adding all selected files
                    setSelectedFiles ( webFileChooser.getSelectedFiles () );
                }
            } );

            chooseButton = new WebButton ( "..." );
            //            chooseButton.setShineColor ( new Color ( 0, 0, 255, 48 ) );
            chooseButton.setRound ( StyleConstants.smallRound );
            chooseButton.setShadeWidth ( StyleConstants.shadeWidth );
            chooseButton.setLeftRightSpacing ( StyleConstants.smallLeftRightSpacing );
            chooseButton.setDrawFocus ( false );
            chooseButton.setDrawLeft ( false );
            chooseButton.setDrawLeftLine ( true );
            chooseButton.setRolloverDarkBorderOnly ( false );
            chooseButton.addActionListener ( new ActionListener ()
            {
                public void actionPerformed ( ActionEvent e )
                {
                    // Files selection
                    webFileChooser.setVisible ( true );

                    // Requesting focus back to this component after file chooser close
                    chooseButton.requestFocusInWindow ();
                }
            } );
            contentPanel.addMouseListener ( new MouseAdapter ()
            {
                public void mousePressed ( MouseEvent e )
                {
                    // For easier files choose
                    if ( SwingUtilities.isLeftMouseButton ( e ) )
                    {
                        chooseButton.doClick ( 0 );
                    }
                }
            } );
            add ( chooseButton, BorderLayout.EAST );
        }

        // Updating layout
        updateContentLayout ();

        // Adding focus listener
        FocusManager.registerFocusTracker ( new FocusTracker ()
        {
            public Component getComponent ()
            {
                return WebFileChooserField.this;
            }

            public boolean countChilds ()
            {
                return true;
            }

            public boolean isFocusOwner ()
            {
                return focusOwner;
            }

            public void focusChanged ( boolean focused )
            {
                focusOwner = focused;
                WebFileChooserField.this.repaint ();
            }
        } );
    }

    private void updateContentLayout ()
    {
        contentPanel.setLayout ( new HorizontalFlowLayout ( 0,
                getSelectionMode ().equals ( SelectionMode.SINGLE_SELECTION ) ) );
    }

    public WebFileChooser getWebFileChooser ()
    {
        return webFileChooser;
    }

    public WebButton getChooseButton ()
    {
        return chooseButton;
    }

    public boolean isShowChooseButton ()
    {
        return showChooseButton;
    }

    //    public void setShowChooseButton ( boolean showChooseButton )
    //    {
    //        // Ignore if not changed
    //        if ( this.showChooseButton == showChooseButton )
    //        {
    //            return;
    //        }
    //
    //        // Updaing button existance
    //        this.showChooseButton = showChooseButton;
    //        if ( showChooseButton )
    //        {
    //            WebFileChooserField.this.add ( chooseButton, BorderLayout.EAST );
    //        }
    //        else
    //        {
    //            WebFileChooserField.this.remove ( chooseButton );
    //        }
    //        WebFileChooserField.this.revalidate ();
    //        WebFileChooserField.this.repaint ();
    //    }

    public SelectionMode getSelectionMode ()
    {
        return selectionMode;
    }

    public void setSelectionMode ( SelectionMode selectionMode )
    {
        this.selectionMode = selectionMode;
        if ( webFileChooser != null )
        {
            webFileChooser.setSelectionMode ( selectionMode );
        }
        updateContentLayout ();
    }

    public boolean isFilesDropEnabled ()
    {
        return filesDropEnabled;
    }

    public void setFilesDropEnabled ( boolean filesDropEnabled )
    {
        this.filesDropEnabled = filesDropEnabled;
    }

    public void setCurrentDirectory ( String dir )
    {
        webFileChooser.setCurrentDirectory ( dir );
    }

    public void setCurrentDirectory ( File dir )
    {
        webFileChooser.setCurrentDirectory ( dir );
    }

    public File getCurrentDirectory ()
    {
        return webFileChooser.getCurrentDirectory ();
    }

    public FilesToChoose getFilesToChoose ()
    {
        return webFileChooser.getFilesToChoose ();
    }

    public void setFilesToChoose ( FilesToChoose filesToChoose )
    {
        webFileChooser.setFilesToChoose ( filesToChoose );
    }

    public List<DefaultFileFilter> getAvailableFilters ()
    {
        if ( webFileChooser != null )
        {
            return webFileChooser.getAvailableFilters ();
        }
        else
        {
            return null;
        }
    }

    public void setAvailableFilter ( DefaultFileFilter availableFilter )
    {
        webFileChooser.setAvailableFilter ( availableFilter );
    }

    public void setAvailableFilters ( List<DefaultFileFilter> availableFilters )
    {
        webFileChooser.setAvailableFilters ( availableFilters );
    }

    public boolean isShowFileShortName ()
    {
        return showFileShortName;
    }

    public void setShowFileShortName ( boolean showFileShortName )
    {
        this.showFileShortName = showFileShortName;
        updateSelectedFiles ();
    }

    public boolean isShowFileIcon ()
    {
        return showFileIcon;
    }

    public void setShowFileIcon ( boolean showFileIcon )
    {
        this.showFileIcon = showFileIcon;
        updateSelectedFiles ();
    }

    public boolean isShowRemoveButton ()
    {
        return showRemoveButton;
    }

    public void setShowRemoveButton ( boolean showRemoveButton )
    {
        this.showRemoveButton = showRemoveButton;
        updateSelectedFiles ();
    }

    public boolean isShowFileExtensions ()
    {
        return showFileExtensions;
    }

    public void setShowFileExtensions ( boolean showFileExtensions )
    {
        this.showFileExtensions = showFileExtensions;
        updateSelectedFiles ();
    }

    public List<File> getSelectedFiles ()
    {
        return selectedFiles;
    }

    public void setSelectedFile ( File selectedFile )
    {
        this.selectedFiles.clear ();
        if ( selectedFile != null &&
                FileUtils.isFileAccepted ( selectedFile, getAvailableFilters () ) )
        {
            this.selectedFiles.add ( selectedFile );
        }
        updateSelectedFiles ();

        // Inform that selection changed
        fireSelectionChanged ( this.selectedFiles );
    }

    public void setSelectedFiles ( List<File> selectedFiles )
    {
        this.selectedFiles.clear ();
        if ( selectedFiles != null && selectedFiles.size () > 0 )
        {
            if ( getSelectionMode ().equals ( SelectionMode.SINGLE_SELECTION ) )
            {
                for ( File file : selectedFiles )
                {
                    if ( FileUtils.isFileAccepted ( file, getAvailableFilters () ) )
                    {
                        this.selectedFiles.add ( file );
                        break;
                    }
                }
            }
            else
            {
                for ( File file : selectedFiles )
                {
                    if ( FileUtils.isFileAccepted ( file, getAvailableFilters () ) )
                    {
                        this.selectedFiles.add ( file );
                    }
                }
            }
        }
        updateSelectedFiles ();

        // Inform that selection changed
        fireSelectionChanged ( this.selectedFiles );
    }

    public void clearSelectedFiles ()
    {
        setSelectedFiles ( null );
    }

    private void updateSelectedFiles ()
    {
        contentPanel.removeAll ();
        for ( File file : selectedFiles )
        {
            contentPanel.add ( new FilePlate ( file ) );
        }
        WebFileChooserField.this.revalidate ();
        WebFileChooserField.this.repaint ();
    }

    protected void paintComponent ( Graphics g )
    {
        super.paintComponent ( g );

        // Styled border
        LafUtils.drawWebBorder ( ( Graphics2D ) g, this, StyleConstants.shadeColor,
                StyleConstants.shadeWidth, StyleConstants.smallRound );
    }

    protected void paintChildren ( Graphics g )
    {
        // Fixed childs clip
        Shape old = LafUtils.intersectClip ( g,
                new Rectangle2D.Double ( StyleConstants.shadeWidth + 1, 0, getWidth () -
                        ( showChooseButton ? StyleConstants.shadeWidth + 1 :
                                ( StyleConstants.shadeWidth + 1 ) * 2 ), getHeight () ) );
        super.paintChildren ( g );
        LafUtils.restoreClip ( g, old );

        // Focus over the childs
        LafUtils.drawWebFocus ( ( Graphics2D ) g, this, StyleConstants.focusType,
                StyleConstants.shadeWidth, StyleConstants.smallRound, null, focusOwner );
    }

    public void setPreferredWidth ( int preferredWidth )
    {
        this.preferredWidth = preferredWidth;
    }

    public int getPreferredWidth ()
    {
        return preferredWidth;
    }

    public Dimension getPreferredSize ()
    {
        Dimension ps = super.getPreferredSize ();
        if ( preferredWidth != -1 )
        {
            ps.width = preferredWidth;
        }
//        ps.height = 4 + 2 + 4 + 16;
        return ps;
    }

    public class FilePlate extends JComponent
    {
        private Timer animator = null;
        private float opacity = 0f;

        public FilePlate ( final File file )
        {
            super ();

            setLayout ( new BorderLayout () );
            setBorder ( BorderFactory.createEmptyBorder ( 3, 5, 3, showRemoveButton ? 3 : 5 ) );

            // File name label
            String actualFileName = FileUtils.getDisplayFileName ( file );
            final String displayFileName =
                    showFileExtensions || file.isDirectory () ? actualFileName :
                            FileUtils.getFileNamePart ( actualFileName );
            final String absolutePath = file.getAbsolutePath ();
            final JLabel fileName =
                    new JLabel ( showFileShortName ? displayFileName : absolutePath );
            fileName.setIcon ( showFileIcon ? FileUtils.getFileIcon ( file, false ) : null );
            fileName.setBorder (
                    BorderFactory.createEmptyBorder ( 0, 0, 0, showRemoveButton ? 1 : 0 ) );
            addMouseListener ( new MouseAdapter ()
            {
                private boolean showShortName = showFileShortName;

                public void mousePressed ( MouseEvent e )
                {
                    WebFileChooserField.this.requestFocusInWindow ();
                    if ( SwingUtilities.isRightMouseButton ( e ) )
                    {
                        if ( getSelectionMode ().equals ( SelectionMode.MULTIPLE_SELECTION ) )
                        {
                            showShortName = !showShortName;
                            fileName.setText ( showShortName ? displayFileName : absolutePath );
                        }
                        else
                        {
                            setShowFileShortName ( !isShowFileShortName () );
                        }
                    }
                }
            } );
            add ( fileName, BorderLayout.CENTER );

            // Remove button
            if ( showRemoveButton )
            {
                WebButton remove = WebButton
                        .createIconWebButton ( CROSS_ICON, StyleConstants.smallRound, 3, 2, true,
                                false, false );
                remove.addActionListener ( new ActionListener ()
                {
                    public void actionPerformed ( ActionEvent e )
                    {
                        if ( animator.isRunning () )
                        {
                            animator.stop ();
                        }
                        animator = new Timer ( 1000 / StyleConstants.animationFps,
                                new ActionListener ()
                                {
                                    public void actionPerformed ( ActionEvent e )
                                    {
                                        opacity -= 0.1f;
                                        if ( opacity > 0f )
                                        {
                                            FilePlate.this.repaint ();
                                        }
                                        else
                                        {
                                            // Remove file
                                            selectedFiles.remove ( file );
                                            contentPanel.remove ( FilePlate.this );
                                            WebFileChooserField.this.revalidate ();
                                            WebFileChooserField.this.repaint ();

                                            // Inform that selected files changed
                                            fireSelectionChanged ( selectedFiles );

                                            // Stopping animation
                                            animator.stop ();
                                        }
                                    }
                                } );
                        animator.start ();
                    }
                } );
                add ( remove, BorderLayout.EAST );
            }

            addAncestorListener ( new AncestorListener ()
            {
                public void ancestorAdded ( AncestorEvent event )
                {
                    animator = new Timer ( 1000 / StyleConstants.animationFps, null );
                    animator.addActionListener ( new ActionListener ()
                    {
                        public void actionPerformed ( ActionEvent e )
                        {
                            opacity += 0.1f;
                            if ( opacity < 1f )
                            {
                                FilePlate.this.repaint ();
                            }
                            else
                            {
                                opacity = 1f;
                                FilePlate.this.repaint ();
                                animator.stop ();
                            }
                        }
                    } );
                    animator.start ();
                }

                public void ancestorRemoved ( AncestorEvent event )
                {
                    //
                }

                public void ancestorMoved ( AncestorEvent event )
                {
                    //
                }
            } );
        }

        protected void paintComponent ( Graphics g )
        {
            if ( opacity < 1f )
            {
                Graphics2D g2d = ( Graphics2D ) g;
                g2d.setComposite (
                        AlphaComposite.getInstance ( AlphaComposite.SRC_OVER, opacity ) );
            }

            super.paintComponent ( g );
            LafUtils.drawWebBorder ( ( Graphics2D ) g, this, StyleConstants.shadeColor,
                    StyleConstants.shadeWidth, StyleConstants.smallRound, true, true );
        }
    }

    public void addSelectedFilesListener ( FilesSelectionListener listener )
    {
        listeners.add ( listener );
    }

    public void removeSelectedFilesListener ( FilesSelectionListener listener )
    {
        listeners.remove ( listener );
    }

    private void fireSelectionChanged ( List<File> selectedFiles )
    {
        for ( FilesSelectionListener listener : listeners )
        {
            listener.selectionChanged ( selectedFiles );
        }
    }
}
