/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.filechooser;

import com.alee.extended.drag.FileDropHandler;
import com.alee.extended.filefilter.DefaultFileFilter;
import com.alee.extended.layout.WrapFlowLayout;
import com.alee.laf.StyleConstants;
import com.alee.laf.button.WebButton;
import com.alee.utils.FileUtils;
import com.alee.utils.LafUtils;
import com.alee.utils.SwingUtils;
import info.clearthought.layout.TableLayout;

import javax.swing.*;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * User: mgarin Date: 07.10.11 Time: 14:25
 */

public class WebFileDrop extends JComponent
{
    public static final ImageIcon CROSS_ICON =
            new ImageIcon ( WebFileDrop.class.getResource ( "icons/cross.png" ) );

    private static final BasicStroke dashStroke =
            new BasicStroke ( 4, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 1f,
                    new float[]{ 8f, 8f }, 0f );

    public int dashRound = StyleConstants.smallRound;
    public int dashSideSpacing = 10;

    private Color dropBackground = new Color ( 242, 242, 242 );
    private Color dropBorder = new Color ( 192, 192, 192 );

    private List<FilesSelectionListener> listeners = new ArrayList<FilesSelectionListener> ();

    private boolean showRemoveButton = true;
    private boolean showFileExtensions = false;
    private boolean filesDropEnabled = true;
    private boolean showDropFilesLabel = true;

    private boolean allowSameFiles = false;
    private DefaultFileFilter fileFilter = null;

    private float dropFilesLabelOpacity = 1f;

    private List<File> selectedFiles = new ArrayList<File> ();

    public WebFileDrop ()
    {
        super ();

        setLayout ( new WrapFlowLayout () );
        setBorder ( BorderFactory.createEmptyBorder ( 1, 1, 1, 1 ) );
        setFocusable ( true );
        setFont ( SwingUtils.getDefaultLabelFont ().deriveFont ( Font.BOLD ).deriveFont ( 20f ) );

        // Files TransferHandler
        setTransferHandler ( new FileDropHandler ()
        {
            protected boolean isDropEnabled ()
            {
                return filesDropEnabled;
            }

            protected boolean filesImported ( List<File> files )
            {
                // Adding dragged files
                boolean anyAdded = false;
                for ( File file : files )
                {
                    if ( fileFilter != null && !fileFilter.accept ( file ) ||
                            !allowSameFiles && FileUtils.containtsFile ( selectedFiles, file ) )
                    {
                        continue;
                    }
                    selectedFiles.add ( file );
                    WebFileDrop.this.add ( new FilePlate ( file ) );
                    anyAdded = true;
                }
                if ( anyAdded )
                {
                    WebFileDrop.this.revalidate ();
                    WebFileDrop.this.repaint ();
                    fireSelectionChanged ( selectedFiles );
                }
                return anyAdded;
            }
        } );

        // Focus "catcher"
        addMouseListener ( new MouseAdapter ()
        {
            public void mousePressed ( MouseEvent e )
            {
                WebFileDrop.this.requestFocusInWindow ();
            }
        } );

        // Background animation listener
        addFileSelectionListener ( new FilesSelectionListener ()
        {
            private int filesCount = 0;
            private Timer animator = null;

            public void selectionChanged ( List<File> selectedFiles )
            {
                if ( filesCount == 0 && selectedFiles.size () > 0 )
                {
                    stopAnimator ();
                    filesCount = selectedFiles.size ();
                    animator = new Timer ( 1000 / StyleConstants.animationFps, new ActionListener ()
                    {
                        public void actionPerformed ( ActionEvent e )
                        {
                            if ( dropFilesLabelOpacity > 0f )
                            {
                                dropFilesLabelOpacity -= 0.1f;
                            }
                            dropFilesLabelOpacity = Math.max ( dropFilesLabelOpacity, 0f );
                            WebFileDrop.this.repaint ();
                            if ( dropFilesLabelOpacity <= 0f )
                            {
                                animator.stop ();
                            }
                        }
                    } );
                    animator.start ();
                }
                else if ( filesCount > 0 && selectedFiles.size () == 0 )
                {
                    stopAnimator ();
                    filesCount = selectedFiles.size ();
                    animator = new Timer ( 1000 / StyleConstants.animationFps, new ActionListener ()
                    {
                        public void actionPerformed ( ActionEvent e )
                        {
                            if ( dropFilesLabelOpacity < 1f )
                            {
                                dropFilesLabelOpacity += 0.1f;
                            }
                            dropFilesLabelOpacity = Math.min ( dropFilesLabelOpacity, 1f );
                            WebFileDrop.this.repaint ();
                            if ( dropFilesLabelOpacity >= 1f )
                            {
                                animator.stop ();
                            }
                        }
                    } );
                    animator.start ();
                }
            }

            private void stopAnimator ()
            {
                if ( animator != null && animator.isRunning () )
                {
                    animator.stop ();
                }
            }
        } );
    }

    public boolean isShowDropFilesLabel ()
    {
        return showDropFilesLabel;
    }

    public void setShowDropFilesLabel ( boolean showDropFilesLabel )
    {
        this.showDropFilesLabel = showDropFilesLabel;
        WebFileDrop.this.repaint ();
    }

    public boolean isFilesDropEnabled ()
    {
        return filesDropEnabled;
    }

    public void setFilesDropEnabled ( boolean filesDropEnabled )
    {
        this.filesDropEnabled = filesDropEnabled;
    }

    public List<File> getSelectedFiles ()
    {
        return selectedFiles;
    }

    public void setSelectedFiles ( List<File> selectedFiles )
    {
        // Filtering only accepted by filter files
        List<File> accepted = new ArrayList<File> ();
        for ( File file : selectedFiles )
        {
            if ( ( fileFilter == null || fileFilter.accept ( file ) ) &&
                    ( allowSameFiles || !FileUtils.containtsFile ( accepted, file ) ) )
            {
                accepted.add ( file );
            }
        }

        // Checking if lists are equal
        if ( FileUtils.areEqual ( accepted, this.selectedFiles ) )
        {
            this.selectedFiles = accepted;
            updateFilesList ();
            fireSelectionChanged ( accepted );
        }
    }

    public boolean isAllowSameFiles ()
    {
        return allowSameFiles;
    }

    public void setAllowSameFiles ( boolean allowSameFiles )
    {
        this.allowSameFiles = allowSameFiles;
        setSelectedFiles ( selectedFiles );
    }

    public DefaultFileFilter getFileFilter ()
    {
        return fileFilter;
    }

    public void setFileFilter ( DefaultFileFilter fileFilter )
    {
        this.fileFilter = fileFilter;
        setSelectedFiles ( selectedFiles );
    }

    public boolean isShowRemoveButton ()
    {
        return showRemoveButton;
    }

    public void setShowRemoveButton ( boolean showRemoveButton )
    {
        this.showRemoveButton = showRemoveButton;
        updateFilesList ();
    }

    public boolean isShowFileExtensions ()
    {
        return showFileExtensions;
    }

    public void setShowFileExtensions ( boolean showFileExtensions )
    {
        this.showFileExtensions = showFileExtensions;
        updateFilesList ();
    }

    public Color getDropBackground ()
    {
        return dropBackground;
    }

    public void setDropBackground ( Color dropBackground )
    {
        this.dropBackground = dropBackground;
    }

    public Color getDropBorder ()
    {
        return dropBorder;
    }

    public void setDropBorder ( Color dropBorder )
    {
        this.dropBorder = dropBorder;
    }

    public int getDashRound ()
    {
        return dashRound;
    }

    public void setDashRound ( int dashRound )
    {
        this.dashRound = dashRound;
    }

    public int getDashSideSpacing ()
    {
        return dashSideSpacing;
    }

    public void setDashSideSpacing ( int dashSideSpacing )
    {
        this.dashSideSpacing = dashSideSpacing;
    }

    protected void paintComponent ( Graphics g )
    {
        super.paintComponent ( g );

        if ( showDropFilesLabel && dropFilesLabelOpacity > 0f )
        {
            Graphics2D g2d = ( Graphics2D ) g;
            g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON );

            Composite old = LafUtils.setupAlphaComposite ( g2d, dropFilesLabelOpacity );

            g2d.setPaint ( dropBackground );
            g2d.fillRoundRect ( dashSideSpacing + Math.round ( dashStroke.getLineWidth () / 2 ),
                    dashSideSpacing + Math.round ( dashStroke.getLineWidth () / 2 ),
                    getWidth () - dashSideSpacing * 2 - Math.round ( dashStroke.getLineWidth () ),
                    getHeight () - dashSideSpacing * 2 - Math.round ( dashStroke.getLineWidth () ),
                    dashRound * 2, dashRound * 2 );

            g2d.setPaint ( dropBorder );

            FontMetrics fm = g2d.getFontMetrics ();
            String text = UIManager.getString ( "WebFileDrop.draghere" );
            g2d.drawString ( text, getWidth () / 2 - fm.stringWidth ( text ) / 2,
                    getHeight () / 2 + fm.getAscent () / 2 );

            Stroke stroke = g2d.getStroke ();
            g2d.setStroke ( dashStroke );
            g2d.drawRoundRect ( dashSideSpacing, dashSideSpacing,
                    getWidth () - dashSideSpacing * 2 - 1, getHeight () - dashSideSpacing * 2 - 1,
                    dashRound * 2, dashRound * 2 );
            g2d.setStroke ( stroke );

            LafUtils.restoreComposite ( g2d, old );
        }
    }

    private void updateFilesList ()
    {
        WebFileDrop.this.removeAll ();
        for ( File file : selectedFiles )
        {
            WebFileDrop.this.add ( new FilePlate ( file ) );
        }
        WebFileDrop.this.revalidate ();
        WebFileDrop.this.repaint ();
    }

    private class FilePlate extends JComponent
    {
        private Timer animator = null;
        private float opacity = 0f;

        public FilePlate ( final File file )
        {
            super ();

            final TableLayout tableLayout = new TableLayout (
                    new double[][]{ { TableLayout.FILL, TableLayout.PREFERRED },
                            { TableLayout.PREFERRED } } );
            tableLayout.setHGap ( 0 );
            tableLayout.setVGap ( 0 );
            setLayout ( tableLayout );
            setBorder ( BorderFactory.createEmptyBorder ( 3, 5, 3, showRemoveButton ? 3 : 5 ) );

            // To block parent container events
            addMouseListener ( new MouseAdapter ()
            {
                public void mousePressed ( MouseEvent e )
                {
                    WebFileDrop.this.requestFocusInWindow ();
                }
            } );

            // Displayed file name
            String name = FileUtils.getDisplayFileName ( file );
            JLabel fileName = new JLabel ( showFileExtensions || file.isDirectory () ? name :
                    FileUtils.getFileNamePart ( name ) );
            fileName.setIcon ( FileUtils.getFileIcon ( file, false ) );
            fileName.setBorder (
                    BorderFactory.createEmptyBorder ( 0, 0, 0, showRemoveButton ? 1 : 0 ) );
            add ( fileName, "0,0" );

            // Remove button
            if ( showRemoveButton )
            {
                WebButton remove = WebButton
                        .createIconWebButton ( CROSS_ICON, StyleConstants.smallRound, 3, 2, true,
                                false, false );
                remove.setFocusable ( false );
                remove.addActionListener ( new ActionListener ()
                {
                    public void actionPerformed ( ActionEvent e )
                    {
                        if ( animator.isRunning () )
                        {
                            animator.stop ();
                        }
                        animator = new Timer ( 1000 / StyleConstants.animationFps,
                                new ActionListener ()
                                {
                                    public void actionPerformed ( ActionEvent e )
                                    {
                                        opacity -= 0.1f;
                                        if ( opacity > 0f )
                                        {
                                            FilePlate.this.repaint ();
                                        }
                                        else
                                        {
                                            // Remove file
                                            opacity = 0f;
                                            selectedFiles.remove ( file );
                                            WebFileDrop.this.remove ( FilePlate.this );
                                            WebFileDrop.this.revalidate ();
                                            WebFileDrop.this.repaint ();

                                            // Inform that selected files changed
                                            fireSelectionChanged ( selectedFiles );

                                            // Stopping animation
                                            animator.stop ();
                                        }
                                    }
                                } );
                        animator.start ();
                    }
                } );
                add ( remove, "1,0" );
            }

            addAncestorListener ( new AncestorListener ()
            {
                public void ancestorAdded ( AncestorEvent event )
                {
                    animator = new Timer ( 1000 / StyleConstants.animationFps, null );
                    animator.addActionListener ( new ActionListener ()
                    {
                        public void actionPerformed ( ActionEvent e )
                        {
                            opacity += 0.1f;
                            if ( opacity < 1f )
                            {
                                FilePlate.this.repaint ();
                            }
                            else
                            {
                                opacity = 1f;
                                FilePlate.this.repaint ();
                                animator.stop ();
                            }
                        }
                    } );
                    animator.start ();
                }

                public void ancestorRemoved ( AncestorEvent event )
                {
                    //
                }

                public void ancestorMoved ( AncestorEvent event )
                {
                    //
                }
            } );
        }

        protected void paintComponent ( Graphics g )
        {
            if ( opacity < 1f )
            {
                Graphics2D g2d = ( Graphics2D ) g;
                g2d.setComposite (
                        AlphaComposite.getInstance ( AlphaComposite.SRC_OVER, opacity ) );
            }

            super.paintComponent ( g );
            LafUtils.drawWebBorder ( ( Graphics2D ) g, this, StyleConstants.shadeColor, 2,
                    StyleConstants.smallRound, true, true );
        }
    }

    public void addFileSelectionListener ( FilesSelectionListener listener )
    {
        listeners.add ( listener );
    }

    public void removeFileSelectionListener ( FilesSelectionListener listener )
    {
        listeners.remove ( listener );
    }

    private void fireSelectionChanged ( List<File> selectedFiles )
    {
        for ( FilesSelectionListener listener : listeners )
        {
            listener.selectionChanged ( selectedFiles );
        }
    }
}
