/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.image;

import com.alee.utils.ImageUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

/**
 * User: mgarin Date: 26.10.11 Time: 18:41
 */

public class WebImage extends JComponent implements SwingConstants
{
    private BufferedImage image;

    private DisplayType displayType;
    private int horizontalAlignment;
    private int verticalAlignment;

    public WebImage ()
    {
        this ( ( Image ) null );
    }

    public WebImage ( String src )
    {
        this ( ImageUtils.loadImage ( src ) );
    }

    public WebImage ( Class nearClass, String src )
    {
        this ( ImageUtils.loadImage ( nearClass, src ) );
    }

    public WebImage ( ImageIcon icon )
    {
        this ( icon.getImage () );
    }

    public WebImage ( Image image )
    {
        super ();

        this.image = ImageUtils.getBufferedImage ( image );

        this.displayType = DisplayType.fitImage;
        this.horizontalAlignment = CENTER;
        this.verticalAlignment = CENTER;

        setOpaque ( false );
    }

    public BufferedImage getImage ()
    {
        return image;
    }

    public void setImage ( ImageIcon icon )
    {
        setImage ( icon.getImage () );
    }

    public void setImage ( Image image )
    {
        this.image = ImageUtils.getBufferedImage ( image );
        revalidate ();
        repaint ();
    }

    public DisplayType getDisplayType ()
    {
        return displayType;
    }

    public void setDisplayType ( DisplayType displayType )
    {
        this.displayType = displayType;
    }

    public int getHorizontalAlignment ()
    {
        return horizontalAlignment;
    }

    public void setHorizontalAlignment ( int horizontalAlignment )
    {
        this.horizontalAlignment = horizontalAlignment;
    }

    public int getVerticalAlignment ()
    {
        return verticalAlignment;
    }

    public void setVerticalAlignment ( int verticalAlignment )
    {
        this.verticalAlignment = verticalAlignment;
    }

    protected void paintComponent ( Graphics g )
    {
        if ( image != null )
        {
            switch ( displayType )
            {
                case fitImage:
                {
                    // Drawing preferred sized image at specified side
                    int x = horizontalAlignment == LEFT ? 0 :
                            ( horizontalAlignment == RIGHT ? getWidth () - image.getWidth () :
                                    getWidth () / 2 - image.getWidth () / 2 );
                    int y = verticalAlignment == TOP ? 0 :
                            ( verticalAlignment == BOTTOM ? getHeight () - image.getHeight () :
                                    getHeight () / 2 - image.getHeight () / 2 );
                    g.drawImage ( image, x, y, null );
                    break;
                }
                case fitComponent:
                {
                    // Drawing sized to fit object image
                    BufferedImage preview = getPreviewImage ();
                    g.drawImage ( preview, getWidth () / 2 - preview.getWidth () / 2,
                            getHeight () / 2 - preview.getHeight () / 2, null );
                    break;
                }
                case repeat:
                {
                    // Drawing repeated in background image
                    int x = horizontalAlignment == LEFT ? 0 :
                            ( horizontalAlignment == RIGHT ? getWidth () - image.getWidth () :
                                    getWidth () / 2 - image.getWidth () / 2 );
                    int y = verticalAlignment == TOP ? 0 :
                            ( verticalAlignment == BOTTOM ? getHeight () - image.getHeight () :
                                    getHeight () / 2 - image.getHeight () / 2 );
                    Graphics2D g2d = ( Graphics2D ) g;
                    g2d.setPaint ( new TexturePaint ( image,
                            new Rectangle2D.Double ( x, y, image.getWidth (),
                                    image.getHeight () ) ) );
                    g2d.fillRect ( 0, 0, getWidth (), getHeight () );
                    break;
                }
            }
        }
    }

    private Dimension lastDimention = null;
    private BufferedImage lastPreviewImage = null;

    private BufferedImage getPreviewImage ()
    {
        if ( image.getWidth () > getWidth () || image.getHeight () > getHeight () )
        {
            if ( lastPreviewImage == null ||
                    lastDimention != null && !lastDimention.equals ( getSize () ) )
            {
                if ( lastPreviewImage != null )
                {
                    lastPreviewImage.flush ();
                    lastPreviewImage = null;
                }
                lastPreviewImage = ImageUtils.createPreviewImage ( image, getSize () );
            }
            return lastPreviewImage;
        }
        else
        {
            return image;
        }
    }

    public Dimension getPreferredSize ()
    {
        if ( !isPreferredSizeSet () )
        {
            if ( image != null )
            {
                return new Dimension ( image.getWidth (), image.getHeight () );
            }
            else
            {
                return new Dimension ( 0, 0 );
            }
        }
        else
        {
            return super.getPreferredSize ();
        }
    }
}
