/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.image;

import com.alee.extended.image.transition.*;
import com.alee.laf.StyleConstants;
import com.alee.utils.LafUtils;

import javax.swing.*;
import javax.swing.Timer;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.GeneralPath;
import java.awt.image.BufferedImage;
import java.util.*;
import java.util.List;

/**
 * User: mgarin Date: 27.10.11 Time: 14:58
 */

public class WebImageTransition extends JComponent
{
    // Transition listeners
    private List<TransitionListener> transitionListeners = new ArrayList<TransitionListener> ();

    // Effect settings
    private TransitionEffect transitionEffect = TransitionEffect.fade;
    private Map<String, Object> effectSettings = new HashMap<String, Object> ();

    // Images
    private BufferedImage currentImage = null;
    private BufferedImage otherImage = null;

    // Variables
    private Timer animator = null;
    private boolean animating = false;
    private boolean blocked = false;

    // Fade effect variables
    public static final String FADE_SPEED = "FADE_SPEED";
    private float fadeSpeed = 0.05f;
    private float fadeOpacity = 0f;

    // Block effect variables
    public static final String BLOCK_TRANSITION_TRANSPARENCY = "BLOCK_TRANSITION_TRANSPARENCY";
    public static final String BLOCK_SIZE = "BLOCK_SIZE";
    public static final String BLOCK_GROW_SPEED = "BLOCK_GROW_SPEED";
    private int blockSize = 80;
    private int blocksGrowSpeed = 4;
    private int[][] blocksProgress = null;
    private Shape blocksClip = null;

    // Curtain effect variables
    public static final String CURTAIN_TYPE = "CURTAIN_TYPE";
    public static final String CURTAIN_SIZE = "CURTAIN_SIZE";
    public static final String CURTAIN_GROW_SPEED = "CURTAIN_GROW_SPEED";
    private int curtainSize = 80;
    private int curtainGrowSpeed = 4;
    private int[] curtainProgress = null;
    private Shape curtainClip = null;
    private CurtainType actualCurtainType = null;

    // Slide effect variables
    public static final String SLIDE_TYPE = "SLIDE_TYPE";
    private int slideProgress = 0;
    private SlideType actualSlideType = null;

    // Zoom effect variables
    public static final String ZOOM_INITIAL_RECT = "ZOOM_INITIAL_RECT";
    public static final String ZOOM_SPEED = "ZOOM_SPEED";
    private float zoomSpeed = 0.05f;
    private float zoomProgress = 0f;
    private Rectangle newRect = null;

    // Fade slide effect variables
    //    private final int fadeSlideWidth = 40;
    //    private final int fadeSlideSpeed = 10;
    //    private int fadeSlideProgress = 0;

    // General variables
    public static final String DIRECTION = "DIRECTION";
    private Direction actualDirection = null;

    // Current transition effect
    private TransitionEffect actualTransitionEffect = null;

    public WebImageTransition ()
    {
        this ( null, null );
    }

    public WebImageTransition ( BufferedImage currentImage )
    {
        this ( currentImage, null );
    }

    public WebImageTransition ( BufferedImage currentImage, BufferedImage otherImage )
    {
        super ();
        this.currentImage = currentImage;
        this.otherImage = otherImage;

        setOpaque ( false );
    }

    public Image getCurrentImage ()
    {
        return currentImage;
    }

    public void setCurrentImage ( BufferedImage currentImage )
    {
        this.currentImage = currentImage;
    }

    public BufferedImage getOtherImage ()
    {
        return otherImage;
    }

    public void setOtherImage ( BufferedImage otherImage )
    {
        this.otherImage = otherImage;
    }

    public TransitionEffect getTransitionEffect ()
    {
        return transitionEffect;
    }

    public void setTransitionEffect ( TransitionEffect transitionEffect )
    {
        this.transitionEffect = transitionEffect;
    }

    public void destroy ()
    {
        if ( animator.isRunning () )
        {
            animator.stop ();
            animator = null;
        }
        if ( transitionListeners.size () > 0 )
        {
            transitionListeners.clear ();
        }
        if ( currentImage != null )
        {
            currentImage.flush ();
            currentImage = null;
        }
        if ( otherImage != null )
        {
            otherImage.flush ();
            otherImage = null;
        }
    }

    public void changeImage ( BufferedImage otherImage )
    {
        // Start only if there is not transition running
        if ( !isAnimating () )
        {
            setOtherImage ( otherImage );
            performTransition ();
        }
    }

    public void performTransition ()
    {
        // Don't allow new transition before old one is finished or transitions blocked
        if ( isAnimating () || isBlocked () )
        {
            return;
        }

        // Defining actual effect for this transaction
        TransitionEffect[] transitionEffects = TransitionEffect.values ();
        actualTransitionEffect = transitionEffect.equals ( TransitionEffect.random ) ?
                transitionEffects[ new Random ().nextInt ( transitionEffects.length - 2 ) + 1 ] :
                transitionEffect;

        Direction direction = getDirection ( DIRECTION );
        Direction[] directions = Direction.values ();
        actualDirection = direction.equals ( Direction.random ) ?
                directions[ new Random ().nextInt ( directions.length - 1 ) + 1 ] : direction;

        // Starting new transition
        animator = new Timer ( 1000 / StyleConstants.averageAnimationFps, new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                if ( actualTransitionEffect.equals ( TransitionEffect.fade ) )
                {
                    // Fade effect
                    if ( !animating )
                    {
                        animating = true;

                        // Updating settings
                        int fs = getInt ( FADE_SPEED );
                        if ( fs != -1 )
                        {
                            fadeSpeed = fs;
                        }

                        fadeOpacity = 0f;
                        WebImageTransition.this.repaint ();
                    }
                    else
                    {
                        if ( fadeOpacity < 1f )
                        {
                            fadeOpacity = Math.min ( fadeOpacity + fadeSpeed, 1f );
                            WebImageTransition.this.repaint ();
                        }
                        else
                        {
                            animating = false;
                            animator.stop ();
                            finishTransition ();
                        }
                    }
                }
                else if ( actualTransitionEffect.equals ( TransitionEffect.blocks ) )
                {
                    // Blocks effect
                    if ( !animating )
                    {
                        animating = true;

                        // Updating settings
                        int bsi = getInt ( BLOCK_SIZE );
                        if ( bsi != -1 )
                        {
                            blockSize = bsi;
                        }
                        int bgs = getInt ( BLOCK_GROW_SPEED );
                        if ( bgs != -1 )
                        {
                            blocksGrowSpeed = bgs;
                        }

                        int h = WebImageTransition.this.getHeight ();
                        h = ( h % blockSize == 0 ? h : ( h / blockSize ) * blockSize + blockSize );
                        int w = WebImageTransition.this.getWidth ();
                        w = ( w % blockSize == 0 ? w : ( w / blockSize ) * blockSize + blockSize );
                        int cols = w / blockSize;
                        int rows = h / blockSize;

                        // Initial progress values
                        blocksProgress = new int[ cols ][ rows ];
                        for ( int i = 0; i < cols; i++ )
                        {
                            for ( int j = 0; j < rows; j++ )
                            {
                                blocksProgress[ i ][ j ] = 0;
                            }
                        }
                        if ( actualDirection.equals ( Direction.right ) ||
                                actualDirection.equals ( Direction.down ) )
                        {
                            blocksProgress[ 0 ][ 0 ] = blocksGrowSpeed;
                        }
                        else
                        {
                            blocksProgress[ cols - 1 ][ rows - 1 ] = blocksGrowSpeed;
                        }

                        // Updating blocks clip
                        if ( !getBoolean ( BLOCK_TRANSITION_TRANSPARENCY ) )
                        {
                            blocksClip = getBlocksProgressShape ( blocksProgress );
                        }

                        WebImageTransition.this.repaint ();
                    }
                    else
                    {
                        // Incrementing grow states
                        boolean allMax = true;
                        int cols = blocksProgress.length;
                        int rows = blocksProgress[ 0 ].length;
                        for ( int i = 0; i < cols; i++ )
                        {
                            for ( int j = 0; j < rows; j++ )
                            {
                                if ( blocksProgress[ i ][ j ] < blockSize )
                                {
                                    if ( blocksProgress[ i ][ j ] > 0 )
                                    {
                                        // Was already growing
                                        blocksProgress[ i ][ j ] += blocksGrowSpeed;
                                    }
                                    else if ( canStartGrow ( i, j, blocksProgress ) )
                                    {
                                        // Starts growing now
                                        blocksProgress[ i ][ j ] += blocksGrowSpeed;
                                    }

                                    // Check if still not max
                                    if ( allMax && blocksProgress[ i ][ j ] < blockSize )
                                    {
                                        allMax = false;
                                    }
                                }
                            }
                        }

                        // Updating blocks clip
                        if ( !getBoolean ( BLOCK_TRANSITION_TRANSPARENCY ) )
                        {
                            blocksClip = getBlocksProgressShape ( blocksProgress );
                        }

                        if ( !allMax )
                        {
                            WebImageTransition.this.repaint ();
                        }
                        else
                        {
                            blocksClip = null;
                            animating = false;
                            animator.stop ();
                            finishTransition ();
                        }
                    }
                }
                else if ( actualTransitionEffect.equals ( TransitionEffect.curtain ) )
                {
                    // Curtain effect
                    if ( !animating )
                    {
                        animating = true;

                        // Updating settings
                        int csi = getInt ( CURTAIN_SIZE );
                        if ( csi != -1 )
                        {
                            curtainSize = csi;
                        }
                        int cgs = getInt ( CURTAIN_GROW_SPEED );
                        if ( cgs != -1 )
                        {
                            curtainGrowSpeed = cgs;
                        }

                        CurtainType curtainType = ( CurtainType ) get ( CURTAIN_TYPE );
                        curtainType = curtainType != null ? curtainType : CurtainType.random;
                        CurtainType[] slideTypes = CurtainType.values ();
                        actualCurtainType = curtainType.equals ( CurtainType.random ) ?
                                slideTypes[ new Random ().nextInt ( slideTypes.length - 1 ) + 1 ] :
                                curtainType;

                        int h = actualDirection.equals ( Direction.up ) ||
                                actualDirection.equals ( Direction.down ) ?
                                WebImageTransition.this.getHeight () :
                                WebImageTransition.this.getWidth ();
                        h = ( h % curtainSize == 0 ? h :
                                ( h / curtainSize ) * curtainSize + curtainSize );
                        int rows = h / curtainSize;

                        // Initial progress values
                        curtainProgress = new int[ rows ];
                        for ( int i = 0; i < rows; i++ )
                        {
                            curtainProgress[ i ] = 0;
                        }
                        if ( actualDirection.equals ( Direction.right ) ||
                                actualDirection.equals ( Direction.down ) )
                        {
                            curtainProgress[ 0 ] = curtainGrowSpeed;
                        }
                        else
                        {
                            curtainProgress[ rows - 1 ] = curtainGrowSpeed;
                        }

                        // Updating curtains clip
                        if ( actualCurtainType.equals ( CurtainType.fill ) )
                        {
                            curtainClip = getCurtainProgressShape ( curtainProgress );
                        }

                        WebImageTransition.this.repaint ();
                    }
                    else
                    {
                        // Incrementing grow states
                        boolean allMax = true;
                        int rows = curtainProgress.length;
                        for ( int i = 0; i < rows; i++ )
                        {
                            if ( curtainProgress[ i ] < curtainSize )
                            {
                                if ( curtainProgress[ i ] > 0 )
                                {
                                    // Was already growing
                                    curtainProgress[ i ] += curtainGrowSpeed;
                                }
                                else if ( canStartGrow ( i, curtainProgress ) )
                                {
                                    // Starts growing now
                                    curtainProgress[ i ] += curtainGrowSpeed;
                                }

                                // Check if still not max
                                if ( allMax && curtainProgress[ i ] < curtainSize )
                                {
                                    allMax = false;
                                }
                            }
                        }

                        // Updating curtains clip
                        if ( actualCurtainType.equals ( CurtainType.fill ) )
                        {
                            curtainClip = getCurtainProgressShape ( curtainProgress );
                        }

                        if ( !allMax )
                        {
                            WebImageTransition.this.repaint ();
                        }
                        else
                        {
                            curtainClip = null;
                            actualCurtainType = null;
                            animating = false;
                            animator.stop ();
                            finishTransition ();
                        }
                    }
                }
                else if ( actualTransitionEffect.equals ( TransitionEffect.slide ) )
                {
                    // Slide effect
                    if ( !animating )
                    {
                        animating = true;

                        SlideType slideType = ( SlideType ) get ( SLIDE_TYPE );
                        slideType = slideType != null ? slideType : SlideType.random;
                        SlideType[] slideTypes = SlideType.values ();
                        actualSlideType = slideType.equals ( SlideType.random ) ?
                                slideTypes[ new Random ().nextInt ( slideTypes.length - 1 ) + 1 ] :
                                slideType;

                        slideProgress = 0;
                        WebImageTransition.this.repaint ();
                    }
                    else
                    {
                        // Determining max progress
                        int maxProgress = actualDirection.equals ( Direction.left ) ||
                                actualDirection.equals ( Direction.right ) ?
                                WebImageTransition.this.getWidth () :
                                WebImageTransition.this.getHeight ();

                        // Incrementing grow states
                        int speed = Math.max ( Math.abs ( maxProgress - slideProgress ) / 7, 8 );
                        slideProgress = Math.min ( slideProgress + speed, maxProgress );

                        if ( slideProgress < maxProgress )
                        {
                            WebImageTransition.this.repaint ();
                        }
                        else
                        {
                            actualSlideType = null;
                            animating = false;
                            animator.stop ();
                            finishTransition ();
                        }
                    }
                }
                //                else if ( actualTransitionEffect.equals ( TransitionEffect.zoom ) )
                //                {
                //                    // Zoom effect
                //                    if ( !animating )
                //                    {
                //                        animating = true;
                //
                //                        // Updating settings
                //                        int fs = getInt ( ZOOM_SPEED );
                //                        if ( fs != -1 )
                //                        {
                //                            zoomSpeed = fs;
                //                        }
                //
                //                        // todo
                //                        if ( newRect == null )
                //                        {
                //                            newRect = new Rectangle ( WebImageTransition.this.getWidth () / 2,
                //                                    WebImageTransition.this.getHeight () / 2, 0, 0 );
                //                        }
                //
                //                        zoomProgress = 0f;
                //                        WebImageTransition.this.repaint ();
                //                    }
                //                    else
                //                    {
                //                        if ( zoomProgress < 1f )
                //                        {
                //                            zoomProgress = Math.min ( zoomProgress + zoomSpeed, 1f );
                //                            WebImageTransition.this.repaint ();
                //                        }
                //                        else
                //                        {
                //                            animating = false;
                //                            animator.stop ();
                //                            finishTransition ();
                //                        }
                //                    }
                //                }
                //                else if ( actualTransitionEffect.equals ( TransitionEffect.fadeSlide ) )
                //                {
                //                    // Fade slide effect
                //                    if ( !animating )
                //                    {
                //                        animating = true;
                //                        fadeSlideProgress = -fadeSlideWidth;
                //                        WebImageTransition.this.repaint ();
                //                    }
                //                    else
                //                    {
                //                        // Incrementing grow states
                //                        fadeSlideProgress = Math.min ( fadeSlideProgress + fadeSlideSpeed,
                //                                WebImageTransition.this.getWidth () + fadeSlideWidth );
                //
                //                        if ( fadeSlideProgress < WebImageTransition.this.getWidth () )
                //                        {
                //                            WebImageTransition.this.repaint ();
                //                        }
                //                        else
                //                        {
                //                            animating = false;
                //                            animator.stop ();
                //                            finishTransition ();
                //                        }
                //                    }
                //                }
                else
                {
                    // No proper effect or "none" specified
                    animator.stop ();
                    finishTransition ();
                }
            }

            private boolean canStartGrow ( int i, int j, int[][] blocksProgress )
            {
                if ( actualDirection.equals ( Direction.right ) ||
                        actualDirection.equals ( Direction.down ) )
                {
                    return i > 0 && blocksProgress[ i - 1 ][ j ] > blockSize / 6 ||
                            j > 0 && blocksProgress[ i ][ j - 1 ] > blockSize / 6 ||

                            i > 1 && blocksProgress[ i - 2 ][ j ] > blockSize * 2 / 6 ||
                            j > 1 && blocksProgress[ i ][ j - 2 ] > blockSize * 2 / 6 ||

                            i > 2 && blocksProgress[ i - 3 ][ j ] > blockSize * 3 / 6 ||
                            j > 2 && blocksProgress[ i ][ j - 3 ] > blockSize * 3 / 6 ||

                            i > 3 && blocksProgress[ i - 4 ][ j ] > blockSize * 4 / 6 ||
                            j > 3 && blocksProgress[ i ][ j - 4 ] > blockSize * 4 / 6 ||

                            i > 4 && blocksProgress[ i - 5 ][ j ] > blockSize * 5 / 6 ||
                            j > 4 && blocksProgress[ i ][ j - 5 ] > blockSize * 5 / 6;
                }
                else
                {
                    return i < blocksProgress.length - 1 &&
                            blocksProgress[ i + 1 ][ j ] > blockSize / 6 ||
                            j < blocksProgress[ 0 ].length - 1 &&
                                    blocksProgress[ i ][ j + 1 ] > blockSize / 6 ||

                            i < blocksProgress.length - 2 &&
                                    blocksProgress[ i + 2 ][ j ] > blockSize * 2 / 6 ||
                            j < blocksProgress[ 0 ].length - 2 &&
                                    blocksProgress[ i ][ j + 2 ] > blockSize * 2 / 6 ||

                            i < blocksProgress.length - 3 &&
                                    blocksProgress[ i + 3 ][ j ] > blockSize * 3 / 6 ||
                            j < blocksProgress[ 0 ].length - 3 &&
                                    blocksProgress[ i ][ j + 3 ] > blockSize * 3 / 6 ||

                            i < blocksProgress.length - 4 &&
                                    blocksProgress[ i + 4 ][ j ] > blockSize * 4 / 6 ||
                            j < blocksProgress[ 0 ].length - 4 &&
                                    blocksProgress[ i ][ j + 4 ] > blockSize * 4 / 6 ||

                            i < blocksProgress.length - 5 &&
                                    blocksProgress[ i + 5 ][ j ] > blockSize * 5 / 6 ||
                            j < blocksProgress[ 0 ].length - 5 &&
                                    blocksProgress[ i ][ j + 5 ] > blockSize * 5 / 6;
                }
            }

            private Shape getBlocksProgressShape ( int[][] blocksProgress )
            {
                GeneralPath gp = new GeneralPath ( GeneralPath.WIND_EVEN_ODD );
                for ( int i = 0; i < blocksProgress.length; i++ )
                {
                    for ( int j = 0; j < blocksProgress[ 0 ].length; j++ )
                    {
                        gp.append ( new Rectangle ( i * blockSize, j * blockSize,
                                blocksProgress[ i ][ j ], blocksProgress[ i ][ j ] ), false );
                    }
                }
                return gp;
            }

            //            private boolean canStartGrow ( int i, int[] shuttersProgress )
            //            {
            //                return i > 0 && shuttersProgress[ i - 1 ] > shutterSize / 6 ||
            //                        i > 1 && shuttersProgress[ i - 2 ] > shutterSize * 2 / 6 ||
            //                        i > 2 && shuttersProgress[ i - 3 ] > shutterSize * 3 / 6 ||
            //                        i > 3 && shuttersProgress[ i - 4 ] > shutterSize * 4 / 6 ||
            //                        i > 4 && shuttersProgress[ i - 5 ] > shutterSize * 5 / 6;
            //            }

            private boolean canStartGrow ( int i, int[] curtainProgress )
            {
                if ( actualDirection.equals ( Direction.right ) ||
                        actualDirection.equals ( Direction.down ) )
                {
                    return i > 0 && curtainProgress[ i - 1 ] > curtainSize / 4 ||
                            i > 1 && curtainProgress[ i - 2 ] > curtainSize * 2 / 4 ||
                            i > 2 && curtainProgress[ i - 3 ] > curtainSize * 3 / 4;
                }
                else
                {
                    return i < curtainProgress.length - 1 &&
                            curtainProgress[ i + 1 ] > curtainSize / 4 ||
                            i < curtainProgress.length - 2 &&
                                    curtainProgress[ i + 2 ] > curtainSize * 2 / 4 ||
                            i < curtainProgress.length - 3 &&
                                    curtainProgress[ i + 3 ] > curtainSize * 3 / 4;
                }
            }

            private Shape getCurtainProgressShape ( int[] curtainProgress )
            {
                GeneralPath gp = new GeneralPath ( GeneralPath.WIND_EVEN_ODD );
                for ( int i = 0; i < curtainProgress.length; i++ )
                {
                    if ( actualDirection.equals ( Direction.up ) ||
                            actualDirection.equals ( Direction.down ) )
                    {
                        gp.append ( new Rectangle ( 0, i * curtainSize,
                                WebImageTransition.this.getWidth (), curtainProgress[ i ] ),
                                false );
                    }
                    else
                    {
                        gp.append ( new Rectangle ( i * curtainSize, 0, curtainProgress[ i ],
                                WebImageTransition.this.getHeight () ), false );
                    }
                }
                return gp;
            }
        } );
        animator.start ();
    }

    public void cancelTransition ()
    {
        animator.stop ();
    }

    private void finishTransition ()
    {
        BufferedImage oldCurrent = this.currentImage;
        this.currentImage = this.otherImage;
        this.otherImage = oldCurrent;

        actualTransitionEffect = null;
        actualDirection = null;

        WebImageTransition.this.repaint ();

        fireTransitionFinished ();
    }

    public boolean isAnimating ()
    {
        return animator != null && animator.isRunning () && animating;
    }

    public boolean isBlocked ()
    {
        return blocked;
    }

    public void setBlocked ( boolean blocked )
    {
        this.blocked = blocked;
    }

    protected void paintComponent ( Graphics g )
    {
        super.paintComponent ( g );

        Graphics2D g2d = ( Graphics2D ) g;
        if ( currentImage != null && otherImage != null )
        {
            // Transition effect
            if ( otherImage != null && isAnimating () && actualTransitionEffect != null )
            {
                if ( actualTransitionEffect.equals ( TransitionEffect.fade ) )
                {
                    // Fading out old image
                    Composite old = LafUtils.setupAlphaComposite ( g2d, 1f - fadeOpacity );
                    g2d.drawImage ( currentImage, 0, 0, getWidth (), getHeight (), null );
                    LafUtils.restoreComposite ( g2d, old );

                    // Fading in new image
                    LafUtils.setupAlphaComposite ( g2d, fadeOpacity );
                    g2d.drawImage ( otherImage, 0, 0, getWidth (), getHeight (), null );
                    LafUtils.restoreComposite ( g2d, old );
                }
                else if ( actualTransitionEffect.equals ( TransitionEffect.blocks ) )
                {
                    // Old image as background
                    g2d.drawImage ( currentImage, 0, 0, getWidth (), getHeight (), null );

                    if ( getBoolean ( BLOCK_TRANSITION_TRANSPARENCY ) )
                    {
                        // Separately drawn transparent blocks
                        int cols = blocksProgress.length;
                        int rows = blocksProgress[ 0 ].length;
                        for ( int i = 0; i < cols; i++ )
                        {
                            for ( int j = 0; j < rows; j++ )
                            {
                                int block = blocksProgress[ i ][ j ];
                                if ( block > 0 )
                                {
                                    int w = Math.min ( otherImage.getWidth () - i * blockSize,
                                            block );
                                    int h = Math.min ( otherImage.getHeight () - j * blockSize,
                                            block );
                                    BufferedImage image = otherImage
                                            .getSubimage ( i * blockSize, j * blockSize, w, h );
                                    if ( block < blockSize )
                                    {
                                        Composite old = LafUtils.setupAlphaComposite ( g2d,
                                                ( float ) block / blockSize );
                                        g2d.drawImage ( image, i * blockSize, j * blockSize, null );
                                        LafUtils.restoreComposite ( g2d, old );
                                    }
                                    else
                                    {
                                        g2d.drawImage ( image, i * blockSize, j * blockSize, null );
                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        // New image with decreasing clipped area
                        Shape old = LafUtils.intersectClip ( g2d, blocksClip );
                        g2d.drawImage ( otherImage, 0, 0, getWidth (), getHeight (), null );
                        LafUtils.restoreClip ( g2d, old );
                    }
                }
                else if ( actualTransitionEffect.equals ( TransitionEffect.curtain ) )
                {
                    // Old image as background
                    g2d.drawImage ( currentImage, 0, 0, getWidth (), getHeight (), null );

                    // Separately drawn transparent curtains
                    if ( actualCurtainType.equals ( CurtainType.fill ) )
                    {
                        // New image with decreasing clipped area
                        Shape old = LafUtils.intersectClip ( g2d, curtainClip );
                        g2d.drawImage ( otherImage, 0, 0, getWidth (), getHeight (), null );
                        LafUtils.restoreClip ( g2d, old );
                    }
                    else
                    {
                        int rows = curtainProgress.length;
                        for ( int i = 0; i < rows; i++ )
                        {
                            int curtain = curtainProgress[ i ];
                            if ( curtain > 0 )
                            {
                                if ( actualDirection.equals ( Direction.up ) ||
                                        actualDirection.equals ( Direction.down ) )
                                {
                                    int h = Math.min ( otherImage.getHeight () - i * curtainSize,
                                            actualCurtainType.equals ( CurtainType.fillAndFade ) ?
                                                    curtain : curtainSize );
                                    BufferedImage image = otherImage
                                            .getSubimage ( 0, i * curtainSize,
                                                    otherImage.getWidth (), h );
                                    if ( curtain < curtainSize )
                                    {
                                        Composite old = LafUtils.setupAlphaComposite ( g2d,
                                                ( float ) curtain / curtainSize );
                                        g2d.drawImage ( image, 0, i * curtainSize, null );
                                        LafUtils.restoreComposite ( g2d, old );
                                    }
                                    else
                                    {
                                        g2d.drawImage ( image, 0, i * curtainSize, null );
                                    }
                                }
                                else
                                {
                                    int w = Math.min ( otherImage.getTileWidth () - i * curtainSize,
                                            actualCurtainType.equals ( CurtainType.fillAndFade ) ?
                                                    curtain : curtainSize );
                                    BufferedImage image = otherImage
                                            .getSubimage ( i * curtainSize, 0, w,
                                                    otherImage.getHeight () );
                                    if ( curtain < curtainSize )
                                    {
                                        Composite old = LafUtils.setupAlphaComposite ( g2d,
                                                ( float ) curtain / curtainSize );
                                        g2d.drawImage ( image, i * curtainSize, 0, null );
                                        LafUtils.restoreComposite ( g2d, old );
                                    }
                                    else
                                    {
                                        g2d.drawImage ( image, i * curtainSize, 0, null );
                                    }
                                }
                            }
                        }
                    }
                }
                else if ( actualTransitionEffect.equals ( TransitionEffect.slide ) )
                {
                    if ( actualSlideType.equals ( SlideType.moveNew ) )
                    {
                        Point np = new Point ( 0, 0 );
                        if ( actualDirection.equals ( Direction.left ) )
                        {
                            np.x = getWidth () - slideProgress;
                        }
                        else if ( actualDirection.equals ( Direction.right ) )
                        {
                            np.x = -getWidth () + slideProgress;
                        }
                        else if ( actualDirection.equals ( Direction.up ) )
                        {
                            np.y = getHeight () - slideProgress;
                        }
                        else if ( actualDirection.equals ( Direction.down ) )
                        {
                            np.y = -getHeight () + slideProgress;
                        }

                        // Old image as background
                        g2d.drawImage ( currentImage, 0, 0, getWidth (), getHeight (), null );

                        // New image sliding in
                        g2d.drawImage ( otherImage, np.x, np.y, getWidth (), getHeight (), null );
                    }
                    else if ( actualSlideType.equals ( SlideType.moveOld ) )
                    {
                        Point cp = new Point ( 0, 0 );
                        if ( actualDirection.equals ( Direction.left ) )
                        {
                            cp.x = -slideProgress;
                        }
                        else if ( actualDirection.equals ( Direction.right ) )
                        {
                            cp.x = slideProgress;
                        }
                        else if ( actualDirection.equals ( Direction.up ) )
                        {
                            cp.y = -slideProgress;
                        }
                        else if ( actualDirection.equals ( Direction.down ) )
                        {
                            cp.y = slideProgress;
                        }

                        // New image as background
                        g2d.drawImage ( otherImage, 0, 0, getWidth (), getHeight (), null );

                        // Old image sliding out
                        g2d.drawImage ( currentImage, cp.x, cp.y, getWidth (), getHeight (), null );
                    }
                    else if ( actualSlideType.equals ( SlideType.moveBoth ) )
                    {
                        Point cp = new Point ( 0, 0 );
                        Point np = new Point ( 0, 0 );
                        if ( actualDirection.equals ( Direction.left ) )
                        {
                            cp.x = -slideProgress;
                            np.x = getWidth () - slideProgress;
                        }
                        else if ( actualDirection.equals ( Direction.right ) )
                        {
                            cp.x = slideProgress;
                            np.x = -getWidth () + slideProgress;
                        }
                        else if ( actualDirection.equals ( Direction.up ) )
                        {
                            cp.y = -slideProgress;
                            np.y = getHeight () - slideProgress;
                        }
                        else if ( actualDirection.equals ( Direction.down ) )
                        {
                            cp.y = slideProgress;
                            np.y = -getHeight () + slideProgress;
                        }

                        // Old image sliding out
                        g2d.drawImage ( currentImage, cp.x, cp.y, getWidth (), getHeight (), null );

                        // New image sliding in
                        g2d.drawImage ( otherImage, np.x, np.y, getWidth (), getHeight (), null );
                    }
                }
                //                else if ( actualTransitionEffect.equals ( TransitionEffect.zoom ) )
                //                {
                //                    int width = WebImageTransition.this.getWidth ();
                //                    int height = WebImageTransition.this.getHeight ();
                //                    Rectangle finalRect = new Rectangle ( 0, 0, width, height );
                //
                //                    Rectangle currentRect = new Rectangle ( 0, 0,
                //                            Math.round ( width * ( 1 + zoomProgress * width / newRect.width ) ),
                //                            Math.round (
                //                                    height * ( 1 + zoomProgress * height / newRect.height ) ) );
                //                    currentRect.x = ( width - currentRect.width ) / 2;
                //                    currentRect.y =
                //                            ( WebImageTransition.this.getHeight () - currentRect.height ) / 2;
                //
                //                    Composite old = LafUtils.setupAlphaComposite ( g2d, 1f - zoomProgress );
                //                    g2d.drawImage ( currentImage, currentRect.x, currentRect.y, currentRect.width,
                //                            currentRect.height, null );
                //                    LafUtils.restoreComposite ( g2d, old );
                //                }
                //                else if ( actualTransitionEffect.equals ( TransitionEffect.fadeSlide ) )
                //                {
                //                    // Old image as background
                //                    g2d.drawImage ( currentImage, 0, 0, getWidth (), getHeight (), null );
                //
                //                    if ( fadeSlideProgress + fadeSlideWidth > 0 )
                //                    {
                //                        // Fading from side image
                //
                //                        BufferedImage bg = new BufferedImage ( getWidth (), getHeight (),
                //                                BufferedImage.TYPE_INT_ARGB );
                //                        Graphics2D gg = bg.createGraphics ();
                //                        gg.setPaint ( new GradientPaint ( fadeSlideProgress, 0, Color.BLACK,
                //                                fadeSlideProgress + fadeSlideWidth, 0, StyleConstants.transparent,
                //                                false ) );
                //                        gg.fillRect ( 0, 0, fadeSlideProgress + fadeSlideWidth, getHeight () );
                //                        gg.setComposite ( AlphaComposite.getInstance ( AlphaComposite.SRC_IN ) );
                //                        gg.drawImage ( otherImage, 0, 0, getWidth (), getHeight (), null );
                //                        gg.dispose ();
                //
                //                        g2d.drawImage ( bg, 0, 0, getWidth (), getHeight (), null );
                //                    }
                //                }
                else
                {
                    // Old image as background
                    g2d.drawImage ( currentImage, 0, 0, getWidth (), getHeight (), null );
                }
            }
            else
            {
                // Old image as background
                g2d.drawImage ( currentImage, 0, 0, getWidth (), getHeight (), null );
            }
        }
        else if ( currentImage != null )
        {
            // Static image
            g2d.drawImage ( currentImage, 0, 0, getWidth (), getHeight (), null );
        }
    }

    public Map<String, Object> getEffectSettings ()
    {
        return effectSettings;
    }

    public void setEffectSettings ( Map<String, Object> effectSettings )
    {
        this.effectSettings = effectSettings;
    }

    public void put ( String property, Object value )
    {
        effectSettings.put ( property, value );
    }

    public Object get ( String property )
    {
        return effectSettings.get ( property );
    }

    public void remove ( String property )
    {
        effectSettings.remove ( property );
    }

    public void clearSettings ()
    {
        effectSettings.clear ();
    }

    private boolean getBoolean ( String property )
    {
        if ( effectSettings.containsKey ( property ) )
        {
            return ( Boolean ) effectSettings.get ( property );
        }
        else
        {
            return false;
        }
    }

    private int getInt ( String property )
    {
        if ( effectSettings.containsKey ( property ) )
        {
            return ( Integer ) effectSettings.get ( property );
        }
        else
        {
            return -1;
        }
    }

    private Direction getDirection ( String property )
    {
        if ( effectSettings.containsKey ( property ) )
        {
            return ( Direction ) effectSettings.get ( property );
        }
        else
        {
            return Direction.random;
        }
    }

    public Dimension getPreferredSize ()
    {
        if ( currentImage != null )
        {
            return new Dimension ( currentImage.getWidth (), currentImage.getHeight () );
        }
        else if ( otherImage != null )
        {
            return new Dimension ( otherImage.getWidth (), otherImage.getHeight () );
        }
        else
        {
            return new Dimension ( 0, 0 );
        }
    }

    public void addTransitionListener ( TransitionListener listener )
    {
        transitionListeners.add ( listener );
    }

    public void removeTransitionListener ( TransitionListener listener )
    {
        transitionListeners.remove ( listener );
    }

    private void fireTransitionFinished ()
    {
        List<TransitionListener> listeners = new ArrayList<TransitionListener> ();
        listeners.addAll ( transitionListeners );
        for ( TransitionListener listener : listeners )
        {
            listener.transitionFinished ();
        }
        listeners.clear ();
    }
}
