/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.image;

import com.alee.laf.StyleConstants;
import com.alee.laf.panel.WebPanel;
import com.alee.utils.LafUtils;
import com.alee.utils.laf.ShapeProducer;
import com.alee.utils.laf.WebShapeProducer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.GeneralPath;

/**
 * User: mgarin Date: 12/1/11 Time: 5:15 PM
 */

public class WebLoadPainter extends WebPanel
{
    private boolean showLoad = false;
    private ShapeProducer clipShapeProducer = null;
    private int progressWidth = 15;
    private int speed = 1;

    private Timer opacityAnimator = null;
    private Timer animator = null;
    private int opacity = 0;
    private int loadProgress = 0;

    public WebLoadPainter ()
    {
        this ( null );
    }

    public WebLoadPainter ( Component component )
    {
        super ();

        // Fill layout
        setLayout ( new BorderLayout () );

        // Add initial component if not null
        if ( component != null )
        {
            setComponent ( component );
        }
    }

    public void setComponent ( Component component )
    {
        removeAll ();
        if ( component != null )
        {
            // Add componet onto loader area
            add ( component, BorderLayout.CENTER );

            // Default Web clip shape producer
            setClipShapeProducer ( new WebShapeProducer ( component ) );
        }
        else
        {
            // Null shape producer for empty component
            setClipShapeProducer ( null );
        }
        revalidate ();
    }

    public Component getComponent ()
    {
        if ( getComponentCount () > 0 )
        {
            return getComponent ( 0 );
        }
        else
        {
            return null;
        }
    }

    public boolean isShowLoad ()
    {
        return showLoad;
    }

    public void setShowLoad ( boolean showLoad )
    {
        this.showLoad = showLoad;
        if ( showLoad )
        {
            stopOpacityAnimator ();
            opacityAnimator = new Timer ( 1000 / StyleConstants.animationFps, new ActionListener ()
            {
                public void actionPerformed ( ActionEvent e )
                {
                    if ( opacity < 128 )
                    {
                        opacity += 8;
                    }
                    else
                    {
                        opacity = 128;
                        opacityAnimator.stop ();
                    }
                }
            } );
            opacityAnimator.start ();

            stopAnimator ();
            animator = new Timer ( 1000 / StyleConstants.averageAnimationFps, new ActionListener ()
            {
                public void actionPerformed ( ActionEvent e )
                {
                    if ( loadProgress < getProgressWidth () * 2 )
                    {
                        loadProgress += speed;
                    }
                    else
                    {
                        loadProgress = 0;
                    }
                    if ( WebLoadPainter.this.isVisible () )
                    {
                        WebLoadPainter.this.repaint ();
                    }
                }
            } );
            animator.start ();
        }
        else
        {
            stopOpacityAnimator ();
            opacityAnimator =
                    new Timer ( 1000 / StyleConstants.averageAnimationFps, new ActionListener ()
                    {
                        public void actionPerformed ( ActionEvent e )
                        {
                            if ( opacity > 0 )
                            {
                                opacity -= 8;
                            }
                            else
                            {
                                opacity = 0;
                                opacityAnimator.stop ();
                                stopAnimator ();
                            }
                        }
                    } );
            opacityAnimator.start ();
        }
    }

    private void stopAnimator ()
    {
        if ( animator != null && animator.isRunning () )
        {
            animator.stop ();
            animator = null;
        }
    }

    private void stopOpacityAnimator ()
    {
        if ( opacityAnimator != null && opacityAnimator.isRunning () )
        {
            opacityAnimator.stop ();
        }
    }

    public int getSpeed ()
    {
        return speed;
    }

    public void setSpeed ( int speed )
    {
        this.speed = speed;
    }

    protected void paintChildren ( Graphics g )
    {
        super.paintChildren ( g );

        if ( opacity > 0 )
        {
            int w = getWidth ();
            int h = getHeight ();
            int pw = getProgressWidth ();
            if ( w > 0 && h > 0 && pw > 0 )
            {
                Graphics2D g2d = ( Graphics2D ) g;
                LafUtils.setupAntialias ( g2d );

                Shape clip = getClipShape ();
                Shape oldClip = LafUtils.intersectClip ( g, clip, clip != null );

                // todo Draw correctly when width is less than height
                g2d.setPaint ( new Color ( 128, 128, 128, opacity ) );
                for ( int i = loadProgress % ( pw * 2 ); i < w + 2 * h; i += pw * 2 )
                {
                    if ( i <= 0 )
                    {
                        continue;
                    }

                    GeneralPath gp = new GeneralPath ( GeneralPath.WIND_EVEN_ODD );

                    // Top side lines
                    gp.moveTo ( i < w ? i : w, i < w ? 0 : Math.min ( h, i - w ) );
                    if ( i - pw < w && i > w )
                    {
                        gp.lineTo ( w, 0 );
                    }
                    gp.lineTo ( i - pw < w ? Math.max ( 0, i - pw ) : w,
                            i - pw < w ? 0 : i - pw - w );

                    // Bottom side lines
                    gp.lineTo ( i - pw < h ? 0 : i - pw - h,
                            i - pw < h ? Math.max ( 0, i - pw ) : h );
                    if ( i - pw < h && i > h )
                    {
                        gp.lineTo ( 0, h );
                    }
                    gp.lineTo ( i < h ? 0 : Math.min ( w, i - h ), i < h ? i : h );

                    gp.closePath ();
                    g2d.fill ( gp );
                }

                LafUtils.restoreClip ( g, oldClip, clip != null );
            }
        }
    }

    private int getProgressWidth ()
    {
        return progressWidth;
    }

    public void setProgressWidth ( int progressWidth )
    {
        this.progressWidth = progressWidth;
    }

    public void setClipShapeProducer ( ShapeProducer clipShapeProducer )
    {
        this.clipShapeProducer = clipShapeProducer;
    }

    public ShapeProducer getClipShapeProducer ()
    {
        return clipShapeProducer;
    }

    protected Shape getClipShape ()
    {
        if ( clipShapeProducer != null )
        {
            return clipShapeProducer.produce ();
        }
        else
        {
            return null;
        }
    }
}
