/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.panel;

import com.alee.extended.image.WebImageTransition;
import com.alee.extended.image.transition.TransitionEffect;
import com.alee.extended.image.transition.TransitionListener;
import com.alee.extended.layout.StackLayout;
import com.alee.laf.panel.WebPanel;
import com.alee.utils.SwingUtils;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * User: mgarin Date: 26.10.11 Time: 12:57
 */

public class TransitionPanel extends WebPanel
{
    // Transition listeners
    private List<TransitionListener> transitionListeners = new ArrayList<TransitionListener> ();

    // Effect settings
    private TransitionEffect transitionEffect = TransitionEffect.fade;
    private Map<String, Object> effectSettings = new HashMap<String, Object> ();

    // Variables
    private boolean animating = false;
    //    private Timer animator = null;

    // Current transition
    private WebImageTransition transition = null;

    public TransitionPanel ()
    {
        this ( null );
    }

    public TransitionPanel ( Component content )
    {
        super ();

        // Special layout allowing double content
        setLayout ( new StackLayout () );

        // Add initial content
        if ( content != null )
        {
            add ( content );
        }
    }

    public void switchContent ( final Component content )
    {
        // Ignore repeated change or change to same content
        if ( TransitionPanel.this.getComponentZOrder ( content ) != -1 )
        {
            return;
        }

        // Check for old transition
        if ( transition != null )
        {
            if ( isAnimating () )
            {
                // Cancel old transition if it is already running
                transition.cancelTransition ();
            }
            else
            {
                // Block transition if it has not yet started
                transition.setBlocked ( true );
            }
        }

        // Width and height
        final int width = TransitionPanel.this.getWidth ();
        final int height = TransitionPanel.this.getHeight ();

        // Current content image
        BufferedImage currentSnapshot;
        if ( transition != null )
        {
            currentSnapshot = transition.getOtherImage ();
        }
        else
        {
            Component currentContent = TransitionPanel.this.getComponent ( 0 );
            currentSnapshot = SwingUtils.createComponentSnapshot ( currentContent, width, height );
        }

        // New content image
        TransitionPanel.this.removeAll ();
        TransitionPanel.this.add ( content, StackLayout.HIDDEN );
        BufferedImage otherSnapshot = SwingUtils.createComponentSnapshot ( content, width, height );

        // Transition panel
        TransitionPanel.this.removeAll ();
        transition = new WebImageTransition ( currentSnapshot, otherSnapshot );
        transition.setTransitionEffect ( transitionEffect );
        transition.setEffectSettings ( effectSettings );
        TransitionPanel.this.add ( transition );
        TransitionPanel.this.revalidate ();
        TransitionPanel.this.repaint ();

        // After-transition actions
        transition.addTransitionListener ( new TransitionListener ()
        {
            public void transitionFinished ()
            {
                // Destroying all image links
                TransitionPanel.this.removeAll ();
                transition.destroy ();

                // Adding finl content to area
                TransitionPanel.this.add ( content );
                TransitionPanel.this.revalidate ();
                TransitionPanel.this.repaint ();
                animating = false;

                // Cleaning up animation resourcs
                transition.destroy ();
                transition = null;

                // Informing listeners
                fireTransitionFinished ();
            }
        } );

        // Start transition
        animating = true;
        transition.performTransition ();
    }

    private boolean isAnimating ()
    {
        return animating;
    }

    public TransitionEffect getTransitionEffect ()
    {
        return transitionEffect;
    }

    public void setTransitionEffect ( TransitionEffect transitionEffect )
    {
        this.transitionEffect = transitionEffect;
    }

    public Map<String, Object> getEffectSettings ()
    {
        return effectSettings;
    }

    public void setEffectSettings ( Map<String, Object> effectSettings )
    {
        this.effectSettings = effectSettings;
    }

    public void put ( String property, Object value )
    {
        effectSettings.put ( property, value );
    }

    public Object get ( String property )
    {
        return effectSettings.get ( property );
    }

    public void remove ( String property )
    {
        effectSettings.remove ( property );
    }

    public void clearSettings ()
    {
        effectSettings.clear ();
    }

    public void addTransitionListener ( TransitionListener listener )
    {
        transitionListeners.add ( listener );
    }

    public void removeTransitionListener ( TransitionListener listener )
    {
        transitionListeners.remove ( listener );
    }

    private void fireTransitionFinished ()
    {
        List<TransitionListener> listeners = new ArrayList<TransitionListener> ();
        listeners.addAll ( transitionListeners );
        for ( TransitionListener listener : listeners )
        {
            listener.transitionFinished ();
        }
        listeners.clear ();
    }
}
