/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.statusbar;

import com.alee.laf.StyleConstants;
import com.alee.laf.label.WebLabel;
import com.alee.managers.tooltip.TooltipManager;
import com.alee.utils.LafUtils;
import com.alee.utils.laf.ShapeProvider;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.RoundRectangle2D;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryUsage;

/**
 * User: mgarin Date: 10.10.11 Time: 17:51
 */

public class WebMemoryBar extends WebLabel implements ShapeProvider
{
    private static Color allocatedBorderColor = Color.GRAY;
    private static Color usedBorderColor = new Color ( 130, 130, 183 );
    private static Color usedFillColor = new Color ( 0, 0, 255, 50 );

    private int leftRightSpacing = StyleConstants.largeLeftRightSpacing;
    private int shadeWidth = StyleConstants.shadeWidth;
    private int round = StyleConstants.smallRound;

    private boolean showTooltip = true;
    private int tooltipDelay = 1000;

    private int preferredWidth = -1;

    private long usedMemory = 0;
    private long allocatedMemory = 0;
    private long maxMemory = 0;

    private int refreshRate = 1000;
    private Timer checker = null;

    private boolean pressed = false;

    private JLabel tooltipLabel;

    public WebMemoryBar ()
    {
        super ();

        setOpaque ( false );
        setFocusable ( false );
        setHorizontalAlignment ( WebLabel.CENTER );

        updateChecker ();
        updateBorder ();

        tooltipLabel = new JLabel ();
        updateTooltip ();

        updateMemory ();

        addMouseListener ( new MouseAdapter ()
        {
            public void mousePressed ( MouseEvent e )
            {
                pressed = true;
                WebMemoryBar.this.repaint ();

                System.gc ();
                updateMemory ();
            }

            public void mouseReleased ( MouseEvent e )
            {
                pressed = false;
                WebMemoryBar.this.repaint ();
            }
        } );
    }

    private void updateBorder ()
    {
        setBorder ( BorderFactory
                .createEmptyBorder ( shadeWidth + 2, shadeWidth + 2 + leftRightSpacing,
                        shadeWidth + 2, shadeWidth + 2 + leftRightSpacing ) );
    }

    private void updateChecker ()
    {
        if ( checker != null && checker.isRunning () )
        {
            checker.stop ();
        }
        checker = new Timer ( refreshRate, new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                if ( showTooltip )
                {
                    updateMemory ();
                }
            }
        } );
        checker.start ();
    }

    private void updateMemory ()
    {
        MemoryUsage mu = ManagementFactory.getMemoryMXBean ().getHeapMemoryUsage ();
        usedMemory = mu.getUsed ();
        allocatedMemory = mu.getCommitted ();
        maxMemory = mu.getMax ();

        WebMemoryBar.this.setText ( getMemoryBarText ( usedMemory, allocatedMemory, maxMemory ) );
        if ( showTooltip )
        {
            tooltipLabel
                    .setText ( getMemoryBarTooltipText ( usedMemory, allocatedMemory, maxMemory ) );
        }

        WebMemoryBar.this.repaint ();
    }

    protected String getMemoryBarText ( long usedMemory, long allocatedMemory, long maxMemory )
    {
        return ( usedMemory / 1024 / 1024 ) + getMegabytesString () + " " +
                UIManager.getString ( "WebMemoryBar.of" ) + " " + ( maxMemory / 1024 / 1024 ) +
                getMegabytesString ();
    }

    protected String getMemoryBarTooltipText ( long usedMemory, long allocatedMemory,
                                               long maxMemory )
    {
        return "<html>" + UIManager.getString ( "WebMemoryBar.allocated" ) + " <b>" +
                ( allocatedMemory / 1024 / 1024 ) + getMegabytesString () + "</b> " +
                UIManager.getString ( "WebMemoryBar.used" ) + " <b>" +
                ( usedMemory / 1024 / 1024 ) + getMegabytesString () + "</b> " +
                UIManager.getString ( "WebMemoryBar.maximum" ) + " <b>" +
                ( maxMemory / 1024 / 1024 ) + getMegabytesString () + "</b></html>";
    }

    private String getMegabytesString ()
    {
        return " " + UIManager.getString ( "FileUtils.file.size.mb" );
    }

    public int getRefreshRate ()
    {
        return refreshRate;
    }

    public void setRefreshRate ( int refreshRate )
    {
        this.refreshRate = refreshRate;
        updateChecker ();
    }

    public int getRound ()
    {
        return round;
    }

    public void setRound ( int round )
    {
        this.round = round;
    }

    public int getShadeWidth ()
    {
        return shadeWidth;
    }

    public void setShadeWidth ( int shadeWidth )
    {
        this.shadeWidth = shadeWidth;
        updateBorder ();
    }

    public int getLeftRightSpacing ()
    {
        return leftRightSpacing;
    }

    public void setLeftRightSpacing ( int leftRightSpacing )
    {
        this.leftRightSpacing = leftRightSpacing;
        updateBorder ();
    }

    public boolean isShowTooltip ()
    {
        return showTooltip;
    }

    public void setShowTooltip ( boolean showTooltip )
    {
        this.showTooltip = showTooltip;
        updateTooltip ();
    }

    private void updateTooltip ()
    {
        if ( showTooltip )
        {
            TooltipManager.setTooltip ( this, tooltipLabel, tooltipDelay );
        }
        else
        {
            TooltipManager.removeTooltips ( tooltipLabel );
        }
    }

    public long getAllocatedMemory ()
    {
        return allocatedMemory;
    }

    public long getUsedMemory ()
    {
        return usedMemory;
    }

    public long getMaxMemory ()
    {
        return maxMemory;
    }

    public Shape provideShape ()
    {
        return LafUtils.getWebBorderShape ( WebMemoryBar.this, getShadeWidth (), getRound () );
    }

    protected void paintComponent ( Graphics g )
    {
        Graphics2D g2d = ( Graphics2D ) g;
        Object aa = g2d.getRenderingHint ( RenderingHints.KEY_ANTIALIASING );
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );

        // Shade
        LafUtils.drawWebBorder ( g2d, this, StyleConstants.shadeColor, shadeWidth, round, true,
                !pressed );

        // Allocated memory line
        g2d.setPaint ( allocatedBorderColor );
        int allocatedWidth = getProgressWidth ( allocatedMemory, false );
        g2d.drawLine ( shadeWidth + allocatedWidth, shadeWidth + 2, shadeWidth + allocatedWidth,
                getHeight () - shadeWidth - 3 );

        // Used memory fill
        g2d.setPaint ( usedFillColor );
        g2d.fill ( getProgressShape ( usedMemory, true ) );
        // Used memory border
        g2d.setPaint ( usedBorderColor );
        g2d.draw ( getProgressShape ( usedMemory, false ) );

        // Text
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, aa );
        super.paintComponent ( g2d );
    }

    private Shape getProgressShape ( long progress, boolean fill )
    {
        int arcRound = ( Math.max ( 0, round - 1 ) ) * 2;
        return new RoundRectangle2D.Double ( shadeWidth + 2, shadeWidth + 2,
                getProgressWidth ( progress, fill ),
                getHeight () - 4 - shadeWidth * 2 - ( fill ? 0 : 1 ), arcRound, arcRound );
    }

    private int getProgressWidth ( long progress, boolean fill )
    {
        return Math.round (
                ( float ) ( getWidth () - 4 - shadeWidth * 2 - ( fill ? 0 : 1 ) ) * progress /
                        maxMemory );
    }

    public void setPreferredWidth ( int preferredWidth )
    {
        this.preferredWidth = preferredWidth;
    }

    public int getPreferredWidth ()
    {
        return preferredWidth;
    }

    public Dimension getPreferredSize ()
    {
        Dimension ps = super.getPreferredSize ();
        if ( preferredWidth != -1 )
        {
            ps.width = preferredWidth;
        }
        return ps;
    }
}
