/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf;

import com.alee.utils.SwingUtils;
import com.alee.utils.image.NinePatchIcon;
import com.alee.utils.laf.DefaultBackgroundPainter;

import javax.swing.*;
import java.awt.*;
import java.util.HashMap;
import java.util.Map;

/**
 * User: mgarin Date: 14.12.11 Time: 16:19
 */

public class StateBackgroundPainter extends DefaultBackgroundPainter
{
    private Map<String, NinePatchIcon> stateIcons;

    public StateBackgroundPainter ()
    {
        super ();
        this.stateIcons = new HashMap<String, NinePatchIcon> ();
    }

    public Map<String, NinePatchIcon> getStateIcons ()
    {
        return stateIcons;
    }

    public void setStateIcons ( Map<String, NinePatchIcon> stateIcons )
    {
        this.stateIcons = stateIcons;
    }

    public void addStateIcon ( String componentState, NinePatchIcon ninePatchIcon )
    {
        stateIcons.put ( componentState, ninePatchIcon );
    }

    public void removeStateIcon ( ComponentState componentState )
    {
        stateIcons.remove ( componentState );
    }

    public boolean hasStateIcons ()
    {
        return stateIcons != null && stateIcons.size () > 0;
    }

    public void paintBackground ( Graphics g, Rectangle bounds, JComponent c )
    {
        if ( hasStateIcons () && c != null )
        {
            NinePatchIcon ninePatchIcon;
            if ( c instanceof AbstractButton )
            {
                // Button component
                ButtonModel bm = ( ( AbstractButton ) c ).getModel ();
                if ( bm.isPressed () )
                {
                    ninePatchIcon = getStateIcon (
                            bm.isSelected () ? ComponentState.selectedPressed :
                                    ComponentState.pressed );
                }
                else if ( bm.isSelected () )
                {
                    if ( bm.isEnabled () )
                    {
                        ninePatchIcon = getStateIcon (
                                bm.isRollover () ? ComponentState.selectedRollover :
                                        ComponentState.selected );
                    }
                    else
                    {
                        ninePatchIcon = getStateIcon ( ComponentState.selectedDisabled );
                    }
                }
                else
                {
                    if ( bm.isEnabled () )
                    {
                        ninePatchIcon = getStateIcon ( bm.isRollover () ? ComponentState.rollover :
                                ComponentState.normal );
                    }
                    else
                    {
                        ninePatchIcon = getStateIcon ( ComponentState.disabled );
                    }
                }
            }
            else if ( c instanceof JToolBar )
            {
                // Toolbar component
                JToolBar toolBar = ( JToolBar ) c;
                if ( toolBar.isFloatable () )
                {
                    ninePatchIcon = getStateIcon ( c.isEnabled () ? ComponentState.floatable :
                            ComponentState.floatableDisabled );
                }
                else
                {
                    ninePatchIcon = getStateIcon (
                            c.isEnabled () ? ComponentState.normal : ComponentState.disabled );
                }
            }
            else
            {
                // Any basic component
                ninePatchIcon = getStateIcon (
                        c.isEnabled () ? ComponentState.normal : ComponentState.disabled );
            }

            // Draw background
            if ( ninePatchIcon != null )
            {
                ninePatchIcon.setComponent ( c );
                ninePatchIcon.paintIcon ( c, g, 0, 0 );
            }

            // Draw focus
            if ( c.isFocusOwner () )
            {
                NinePatchIcon focusIcon = getStateIcon ( ComponentState.focused );
                if ( focusIcon != null )
                {
                    focusIcon.setComponent ( c );
                    focusIcon.paintIcon ( c, g, 0, 0 );
                }
            }
        }
    }

    public NinePatchIcon getStateIcon ( String componentState )
    {
        if ( componentState.equals ( ComponentState.normal ) )
        {
            return stateIcons.containsKey ( componentState ) ? stateIcons.get ( componentState ) :
                    null;
        }
        else if ( componentState.equals ( ComponentState.rollover ) )
        {
            return stateIcons.containsKey ( componentState ) ? stateIcons.get ( componentState ) :
                    getStateIcon ( ComponentState.normal );
        }
        else if ( componentState.equals ( ComponentState.disabled ) )
        {
            return stateIcons.containsKey ( componentState ) ? stateIcons.get ( componentState ) :
                    getStateIcon ( ComponentState.normal );
        }
        else if ( componentState.equals ( ComponentState.pressed ) )
        {
            return stateIcons.containsKey ( componentState ) ? stateIcons.get ( componentState ) :
                    getStateIcon ( ComponentState.selected );
        }
        else if ( componentState.equals ( ComponentState.selected ) )
        {
            return stateIcons.containsKey ( componentState ) ? stateIcons.get ( componentState ) :
                    getStateIcon ( ComponentState.normal );
        }
        else if ( componentState.equals ( ComponentState.selectedRollover ) )
        {
            return stateIcons.containsKey ( componentState ) ? stateIcons.get ( componentState ) :
                    getStateIcon ( ComponentState.selected );
        }
        else if ( componentState.equals ( ComponentState.selectedDisabled ) )
        {
            return stateIcons.containsKey ( componentState ) ? stateIcons.get ( componentState ) :
                    getStateIcon ( ComponentState.selected );
        }
        else if ( componentState.equals ( ComponentState.selectedPressed ) )
        {
            return stateIcons.containsKey ( componentState ) ? stateIcons.get ( componentState ) :
                    getStateIcon ( ComponentState.pressed );
        }
        else if ( componentState.equals ( ComponentState.focused ) )
        {
            return stateIcons.containsKey ( componentState ) ? stateIcons.get ( componentState ) :
                    null;
        }
        else if ( componentState.equals ( ComponentState.floatable ) )
        {
            return stateIcons.containsKey ( componentState ) ? stateIcons.get ( componentState ) :
                    getStateIcon ( ComponentState.normal );
        }
        else if ( componentState.equals ( ComponentState.floatableDisabled ) )
        {
            return stateIcons.containsKey ( componentState ) ? stateIcons.get ( componentState ) :
                    getStateIcon ( ComponentState.floatable );
        }
        else
        {
            return null;
        }
    }

    public Dimension getBackgroundPreferredSize ( JComponent c )
    {
        if ( hasStateIcons () )
        {
            Dimension maxDimension = new Dimension ( 0, 0 );
            for ( NinePatchIcon npi : stateIcons.values () )
            {
                npi.setComponent ( c );
                maxDimension = SwingUtils.max ( maxDimension, npi.getPreferredSize () );
            }
            return maxDimension;
        }
        else
        {
            return super.getBackgroundPreferredSize ( c );
        }
    }

    public Insets getBackgroundMargin ( JComponent c )
    {
        if ( hasStateIcons () )
        {
            Insets maxInsets = new Insets ( 0, 0, 0, 0 );
            for ( NinePatchIcon npi : stateIcons.values () )
            {
                npi.setComponent ( c );
                maxInsets = SwingUtils.max ( maxInsets, npi.getMargin () );
            }
            return maxInsets;
        }
        else
        {
            return super.getBackgroundMargin ( c );
        }
    }
}
