/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.button;

import com.alee.laf.BackgroundPainter;
import com.alee.laf.StyleConstants;
import com.alee.utils.ColorUtils;
import com.alee.utils.LafUtils;
import com.alee.utils.SwingUtils;
import com.alee.utils.laf.ShapeProvider;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicButtonUI;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;

/**
 * User: mgarin Date: 27.04.11 Time: 17:41
 */

public class WebButtonUI extends BasicButtonUI implements ShapeProvider, SwingConstants
{
    private Color bgTop = StyleConstants.topBgColor;
    private Color bgBottom = StyleConstants.bottomBgColor;
    private Color selectedBgColor = StyleConstants.selectedBgColor;

    private boolean rolloverShine = true;
    private Color shineColor = StyleConstants.shineColor;

    private boolean rolloverDarkBorderOnly = StyleConstants.rolloverDarkBorderOnly;

    private int round = StyleConstants.round;
    private boolean rolloverShadeOnly = StyleConstants.rolloverShadeOnly;
    private int shadeWidth = StyleConstants.shadeWidth;
    private Color shadeColor = StyleConstants.shadeColor;
    private int innerShadeWidth = StyleConstants.innerShadeWidth;
    private Color innerShadeColor = StyleConstants.innerShadeColor;
    private int leftRightSpacing = StyleConstants.leftRightSpacing;

    private boolean shadeToggleIcon = StyleConstants.shadeToggleIcon;
    private float shadeToggleIconTransparency = StyleConstants.shadeToggleIconTransparency;

    private boolean drawFocus = StyleConstants.drawFocus;
    private boolean rolloverDecoratedOnly = StyleConstants.rolloverDecoratedOnly;
    private boolean animate = StyleConstants.animate;
    private boolean undecorated = StyleConstants.undecorated;
    private BackgroundPainter backgroundPainter = StyleConstants.backgroundPainter;
    private boolean moveIconOnPress = false;

    private Dimension minimumSize = null;

    private boolean drawTop = true;
    private boolean drawLeft = true;
    private boolean drawBottom = true;
    private boolean drawRight = true;

    private boolean drawTopLine = false;
    private boolean drawLeftLine = false;
    private boolean drawBottomLine = false;
    private boolean drawRightLine = false;

    private Color transarentShineColor =
            new Color ( shineColor.getRed (), shineColor.getGreen (), shineColor.getBlue (), 0 );

    private boolean rollover = false;
    private float transparency = 0f;

    private Point mousePoint = null;
    private Timer animator = null;
    private AbstractButton button = null;

    private MouseAdapter mouseAdapter;

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebButtonUI ();
    }

    public void installUI ( final JComponent c )
    {
        super.installUI ( c );

        button = ( AbstractButton ) c;
        button.setMargin ( new Insets ( 0, 0, 0, 0 ) );
        button.setFocusPainted ( false );
        button.setContentAreaFilled ( false );
        button.setBorderPainted ( false );

        button.setFocusable ( true );
        button.setOpaque ( false );

        updateBorder ();

        mouseAdapter = new MouseAdapter ()
        {
            public void mouseEntered ( MouseEvent e )
            {
                rollover = true;
                button.getModel ().setRollover ( true );
                mousePoint = e.getPoint ();

                stopAnimator ();
                if ( backgroundPainter == null && animate )
                {
                    animator = new Timer ( 1000 / StyleConstants.fastAnimationFps,
                            new ActionListener ()
                            {
                                public void actionPerformed ( ActionEvent e )
                                {
                                    transparency += 0.075f;
                                    if ( transparency >= 1f )
                                    {
                                        transparency = 1f;
                                        animator.stop ();
                                    }
                                    updateTransparentShineColor ();
                                    if ( c.isEnabled () )
                                    {
                                        c.repaint ();
                                    }
                                }
                            } );
                    animator.start ();
                }
                else
                {
                    transparency = 1f;
                    updateTransparentShineColor ();
                    c.repaint ();
                }

                refresh ( c );
            }

            public void mouseExited ( MouseEvent e )
            {
                mousePoint = e.getPoint ();

                stopAnimator ();
                if ( backgroundPainter == null && animate )
                {
                    animator = new Timer ( 1000 / StyleConstants.fastAnimationFps,
                            new ActionListener ()
                            {
                                public void actionPerformed ( ActionEvent e )
                                {
                                    transparency -= 0.075f;
                                    if ( transparency <= 0f )
                                    {
                                        rollover = false;
                                        button.getModel ().setRollover ( false );
                                        transparency = 0f;
                                        mousePoint = null;
                                        animator.stop ();
                                    }
                                    updateTransparentShineColor ();
                                    if ( c.isEnabled () )
                                    {
                                        c.repaint ();
                                    }
                                }
                            } );
                    animator.start ();
                }
                else
                {
                    rollover = false;
                    button.getModel ().setRollover ( false );
                    transparency = 0f;
                    mousePoint = null;
                    updateTransparentShineColor ();
                    c.repaint ();
                }

                refresh ( c );
            }

            public void mouseReleased ( MouseEvent e )
            {
                // Fix for highlight stuck
                if ( !c.isShowing () || windowLostFocus () )
                {
                    mousePoint = null;
                    c.repaint ();
                }
            }

            private boolean windowLostFocus ()
            {
                Window wa = SwingUtilities.getWindowAncestor ( c );
                return wa == null || !wa.isVisible () || !wa.isActive ();
            }

            private void stopAnimator ()
            {
                if ( animator != null && animator.isRunning () )
                {
                    animator.stop ();
                }
            }

            public void mouseDragged ( MouseEvent e )
            {
                mousePoint = e.getPoint ();
                if ( backgroundPainter == null )
                {
                    refresh ( c );
                }
            }

            public void mouseMoved ( MouseEvent e )
            {
                mousePoint = e.getPoint ();
                if ( backgroundPainter == null )
                {
                    refresh ( c );
                }
            }

            private void refresh ( JComponent c )
            {
                if ( c.isEnabled () )
                {
                    if ( animator == null || !animator.isRunning () )
                    {
                        c.repaint ();
                    }
                }
            }
        };
        c.addMouseListener ( mouseAdapter );
        c.addMouseMotionListener ( mouseAdapter );

        //        ( ( AbstractButton ) c ).addActionListener ( new ActionListener ()
        //        {
        //            public void actionPerformed ( ActionEvent e )
        //            {
        //                mousePoint = null;
        //                c.repaint ();
        //            }
        //        } );
    }

    public void uninstallUI ( JComponent c )
    {
        c.removeMouseListener ( mouseAdapter );
        c.removeMouseMotionListener ( mouseAdapter );

        super.uninstallUI ( c );
    }

    public Shape provideShape ()
    {
        return LafUtils.getWebBorderShape ( button, getShadeWidth (), getRound () );
    }

    public boolean isRolloverDarkBorderOnly ()
    {
        return rolloverDarkBorderOnly;
    }

    public void setRolloverDarkBorderOnly ( boolean rolloverDarkBorderOnly )
    {
        this.rolloverDarkBorderOnly = rolloverDarkBorderOnly;
    }

    public boolean isRolloverShine ()
    {
        return rolloverShine;
    }

    public void setRolloverShine ( boolean rolloverShine )
    {
        this.rolloverShine = rolloverShine;
    }

    public Color getShineColor ()
    {
        return shineColor;
    }

    public void setShineColor ( Color shineColor )
    {
        this.shineColor = shineColor;
        updateTransparentShineColor ();
    }

    private void updateTransparentShineColor ()
    {
        transarentShineColor =
                new Color ( shineColor.getRed (), shineColor.getGreen (), shineColor.getBlue (),
                        Math.round ( transparency * shineColor.getAlpha () ) );
    }

    private void updateBorder ()
    {
        if ( button != null )
        {
            if ( backgroundPainter != null )
            {
                // Background insets
                Insets bi = backgroundPainter.getBackgroundMargin ( button );
                button.setBorder ( BorderFactory
                        .createEmptyBorder ( bi.top, bi.left + leftRightSpacing, bi.bottom,
                                bi.right + leftRightSpacing ) );
            }
            else if ( !undecorated )
            {
                // Button insets
                int top = ( drawTop ? shadeWidth : ( drawTopLine ? 1 : 0 ) ) + innerShadeWidth;
                int left = ( drawLeft ? shadeWidth : ( drawLeftLine ? 1 : 0 ) ) + innerShadeWidth +
                        leftRightSpacing;
                int bottom =
                        ( drawBottom ? shadeWidth : ( drawBottomLine ? 1 : 0 ) ) + innerShadeWidth;
                int right =
                        ( drawRight ? shadeWidth : ( drawRightLine ? 1 : 0 ) ) + innerShadeWidth +
                                leftRightSpacing;
                button.setBorder ( BorderFactory.createEmptyBorder ( top, left, bottom, right ) );
            }
            else
            {
                // Empty insets
                button.setBorder ( BorderFactory
                        .createEmptyBorder ( 0, leftRightSpacing, 0, leftRightSpacing ) );
            }
        }
    }

    public boolean isRolloverShadeOnly ()
    {
        return rolloverShadeOnly;
    }

    public void setRolloverShadeOnly ( boolean rolloverShadeOnly )
    {
        this.rolloverShadeOnly = rolloverShadeOnly;
    }

    public int getShadeWidth ()
    {
        return shadeWidth;
    }

    public void setShadeWidth ( int shadeWidth )
    {
        this.shadeWidth = shadeWidth;
        updateBorder ();
    }

    public Color getShadeColor ()
    {
        return shadeColor;
    }

    public void setShadeColor ( Color shadeColor )
    {
        this.shadeColor = shadeColor;
    }

    public int getInnerShadeWidth ()
    {
        return innerShadeWidth;
    }

    public void setInnerShadeWidth ( int innerShadeWidth )
    {
        this.innerShadeWidth = innerShadeWidth;
        updateBorder ();
    }

    public Color getInnerShadeColor ()
    {
        return innerShadeColor;
    }

    public void setInnerShadeColor ( Color innerShadeColor )
    {
        this.innerShadeColor = innerShadeColor;
    }

    public int getLeftRightSpacing ()
    {
        return leftRightSpacing;
    }

    public void setLeftRightSpacing ( int leftRightSpacing )
    {
        this.leftRightSpacing = leftRightSpacing;
        updateBorder ();
    }

    public int getRound ()
    {
        return round;
    }

    public void setRound ( int round )
    {
        this.round = round;
    }

    public boolean isShadeToggleIcon ()
    {
        return shadeToggleIcon;
    }

    public void setShadeToggleIcon ( boolean shadeToggleIcon )
    {
        this.shadeToggleIcon = shadeToggleIcon;
    }

    public float getShadeToggleIconTransparency ()
    {
        return shadeToggleIconTransparency;
    }

    public void setShadeToggleIconTransparency ( float shadeToggleIconTransparency )
    {
        this.shadeToggleIconTransparency = shadeToggleIconTransparency;
    }

    public boolean isUndecorated ()
    {
        return undecorated;
    }

    public void setUndecorated ( boolean undecorated )
    {
        this.undecorated = undecorated;
        updateBorder ();
    }

    public BackgroundPainter getBackgroundPainter ()
    {
        return backgroundPainter;
    }

    public void setBackgroundPainter ( BackgroundPainter backgroundPainter )
    {
        this.backgroundPainter = backgroundPainter;
        updateBorder ();
    }

    public boolean isMoveIconOnPress ()
    {
        return moveIconOnPress;
    }

    public void setMoveIconOnPress ( boolean moveIconOnPress )
    {
        this.moveIconOnPress = moveIconOnPress;
    }

    public Dimension getMinimumSize ()
    {
        return minimumSize;
    }

    public void setMinimumSize ( Dimension minimumSize )
    {
        this.minimumSize = minimumSize;
    }

    public boolean isRolloverDecoratedOnly ()
    {
        return rolloverDecoratedOnly;
    }

    public void setRolloverDecoratedOnly ( boolean rolloverDecoratedOnly )
    {
        this.rolloverDecoratedOnly = rolloverDecoratedOnly;
    }

    public boolean isDrawFocus ()
    {
        return drawFocus;
    }

    public void setDrawFocus ( boolean drawFocus )
    {
        this.drawFocus = drawFocus;
    }

    public boolean isDrawBottom ()
    {
        return drawBottom;
    }

    public void setDrawBottom ( boolean drawBottom )
    {
        this.drawBottom = drawBottom;
        updateBorder ();
    }

    public boolean isDrawLeft ()
    {
        return drawLeft;
    }

    public void setDrawLeft ( boolean drawLeft )
    {
        this.drawLeft = drawLeft;
        updateBorder ();
    }

    public boolean isDrawRight ()
    {
        return drawRight;
    }

    public void setDrawRight ( boolean drawRight )
    {
        this.drawRight = drawRight;
        updateBorder ();
    }

    public boolean isDrawTop ()
    {
        return drawTop;
    }

    public void setDrawTop ( boolean drawTop )
    {
        this.drawTop = drawTop;
        updateBorder ();
    }

    public boolean isDrawTopLine ()
    {
        return drawTopLine;
    }

    public void setDrawTopLine ( boolean drawTopLine )
    {
        this.drawTopLine = drawTopLine;
        updateBorder ();
    }

    public boolean isDrawLeftLine ()
    {
        return drawLeftLine;
    }

    public void setDrawLeftLine ( boolean drawLeftLine )
    {
        this.drawLeftLine = drawLeftLine;
        updateBorder ();
    }

    public boolean isDrawBottomLine ()
    {
        return drawBottomLine;
    }

    public void setDrawBottomLine ( boolean drawBottomLine )
    {
        this.drawBottomLine = drawBottomLine;
        updateBorder ();
    }

    public boolean isDrawRightLine ()
    {
        return drawRightLine;
    }

    public void setDrawRightLine ( boolean drawRightLine )
    {
        this.drawRightLine = drawRightLine;
        updateBorder ();
    }

    public void paint ( Graphics g, JComponent c )
    {
        AbstractButton button = ( AbstractButton ) c;
        ButtonModel buttonModel = button.getModel ();
        Graphics2D g2d = ( Graphics2D ) g;

        if ( backgroundPainter != null || !undecorated )
        {
            Object aa = LafUtils.setupAntialias ( g2d );

            if ( backgroundPainter != null )
            {
                // Use background painter instead of default UI graphics
                backgroundPainter.paintBackground ( g, SwingUtils.size ( c ), c );
            }
            else if ( !undecorated )
            {
                boolean pressed = buttonModel.isPressed () || buttonModel.isSelected ();

                Shape bs = getButtonShape ( button );

                if ( rolloverDecoratedOnly && rollover && c.isEnabled () ||
                        animate && transparency > 0f && c.isEnabled () || !rolloverDecoratedOnly ||
                        buttonModel.isSelected () || buttonModel.isPressed () )
                {
                    // Rollover decorated only transparency
                    boolean animatedTransparency = animate && rolloverDecoratedOnly && !pressed;
                    Composite oldComposite = LafUtils.setupAlphaComposite ( g2d, transparency,
                            animatedTransparency );

                    // Shade drawing
                    if ( c.isEnabled () && !pressed && ( !rolloverShadeOnly || rollover ) )
                    {
                        boolean shouldSetInner = !animatedTransparency && rolloverShadeOnly;
                        Composite oc =
                                LafUtils.setupAlphaComposite ( g2d, oldComposite, transparency,
                                        shouldSetInner );
                        LafUtils.drawShade ( g2d, bs, shadeColor, shadeWidth );
                        LafUtils.restoreComposite ( g2d, oc, shouldSetInner );
                    }

                    // Background drawing
                    if ( pressed )
                    {
                        g2d.setPaint ( selectedBgColor );
                    }
                    else
                    {
                        g2d.setPaint ( new GradientPaint ( 0, drawTop ? shadeWidth : 0, bgTop, 0,
                                button.getHeight () - ( drawBottom ? shadeWidth : 0 ), bgBottom ) );
                    }
                    g2d.fill ( bs );

                    // Cursor-following highlight drawing
                    if ( rolloverShine && mousePoint != null && c.isEnabled () )
                    {
                        Shape oldClip = g2d.getClip ();
                        Area newClip = new Area ( bs );
                        newClip.intersect ( new Area ( oldClip ) );
                        g2d.setClip ( newClip );
                        g2d.setPaint ( new RadialGradientPaint ( mousePoint.x, c.getHeight (),
                                c.getWidth (), new float[]{ 0f, 1f },
                                new Color[]{ transarentShineColor, StyleConstants.transparent } ) );
                        g2d.fill ( bs );
                        g2d.setClip ( oldClip );
                    }

                    // Inner shade drawing
                    if ( pressed )
                    {
                        LafUtils.drawShade ( g2d, bs, innerShadeColor, innerShadeWidth, bs );
                    }

                    // Border drawing
                    g2d.setPaint ( c.isEnabled () ?
                            ( rolloverDarkBorderOnly && rollover ? getBorderColor () :
                                    ( !rolloverDarkBorderOnly ? StyleConstants.darkBorderColor :
                                            StyleConstants.borderColor ) ) :
                            StyleConstants.disabledBorderColor );
                    g2d.draw ( bs );

                    //                g2d.setPaint ( Color.WHITE );
                    //                g2d.drawRoundRect ( shadeWidth + 1, shadeWidth + 1,
                    //                        c.getWidth () - shadeWidth * 2 - 3, c.getHeight () - shadeWidth * 2 - 3,
                    //                        round * 2, round * 2 );

                    // Side-border
                    g2d.setPaint ( c.isEnabled () ? StyleConstants.darkBorderColor :
                            StyleConstants.disabledBorderColor );
                    if ( drawTopLine )
                    {
                        int x = drawLeft ? shadeWidth : 0;
                        g2d.drawLine ( x, 0, x + c.getWidth () - ( drawLeft ? shadeWidth : 0 ) -
                                ( drawRight ? shadeWidth + 1 : 0 ), 0 );
                    }
                    if ( drawBottomLine )
                    {
                        int x = drawLeft ? shadeWidth : 0;
                        g2d.drawLine ( x, c.getHeight () - 1,
                                x + c.getWidth () - ( drawLeft ? shadeWidth : 0 ) -
                                        ( drawRight ? shadeWidth + 1 : 0 ), c.getHeight () - 1 );
                    }
                    if ( drawLeftLine )
                    {
                        int y = drawTop ? shadeWidth : 0;
                        g2d.drawLine ( 0, y, 0, y + c.getHeight () - ( drawTop ? shadeWidth : 0 ) -
                                ( drawBottom ? shadeWidth + 1 : 0 ) );
                    }
                    if ( drawRightLine )
                    {
                        int y = drawTop ? shadeWidth : 0;
                        g2d.drawLine ( c.getWidth () - 1, y, c.getWidth () - 1,
                                y + c.getHeight () - ( drawTop ? shadeWidth : 0 ) -
                                        ( drawBottom ? shadeWidth + 1 : 0 ) );
                    }

                    LafUtils.restoreComposite ( g2d, oldComposite, animatedTransparency );
                }

                // Focus drawing
                if ( drawFocus && c.isEnabled () )
                {
                    LafUtils.drawCustonWebFocus ( g2d, c, StyleConstants.focusType, bs );
                }
            }
        }

        // Special text and icon translation effect on button press
        if ( buttonModel.isPressed () && moveIconOnPress )
        {
            g2d.translate ( 1, 1 );
        }

        // Default text and icon drawing
        super.paint ( g, c );
    }

    private Color getBorderColor ()
    {
        return ColorUtils.getProgress ( StyleConstants.borderColor, StyleConstants.darkBorderColor,
                transparency );
    }

    protected void paintIcon ( Graphics g, JComponent c, Rectangle iconRect )
    {
        AbstractButton button = ( AbstractButton ) c;
        ButtonModel buttonModel = button.getModel ();
        Graphics2D g2d = ( Graphics2D ) g;

        boolean shadeToggleIcon = this.shadeToggleIcon && button instanceof JToggleButton &&
                !buttonModel.isSelected ();
        Composite old =
                LafUtils.setupAlphaComposite ( g2d, shadeToggleIconTransparency, shadeToggleIcon );

        super.paintIcon ( g, c, iconRect );

        LafUtils.restoreComposite ( g2d, old, shadeToggleIcon );
    }

    protected Shape getButtonShape ( AbstractButton button )
    {
        int x = drawLeft ? shadeWidth : -shadeWidth - round - 1;
        int y = drawTop ? shadeWidth : -shadeWidth - round - 1;
        int maxX = drawRight ? button.getWidth () - shadeWidth - 1 :
                button.getWidth () + shadeWidth + round;
        int maxY = drawBottom ? button.getHeight () - shadeWidth - 1 :
                button.getHeight () + shadeWidth + round;
        int width = maxX - x;
        int height = maxY - y;

        if ( round > 0 )
        {
            return new RoundRectangle2D.Double ( x, y, width, height, round * 2, round * 2 );
        }
        else
        {
            return new Rectangle2D.Double ( x, y, width, height );
        }
    }

    public Dimension getPreferredSize ( JComponent c )
    {
        Dimension ps = super.getPreferredSize ( c );

        // Minimum size
        ps = SwingUtils.max ( ps, minimumSize );

        // Background painter preferred size
        if ( backgroundPainter != null )
        {
            ps = SwingUtils.max ( ps, backgroundPainter.getBackgroundPreferredSize ( c ) );
        }

        return ps;
    }

    //    private Shape getButtonFocusShape ( AbstractButton button )
    //    {
    //        int x = drawLeft ? shadeWidth : -shadeWidth - round - 1;
    //        int y = drawTop ? shadeWidth : -shadeWidth - round - 1;
    //        int maxX = drawRight ? button.getWidth () - shadeWidth - 1 :
    //                button.getWidth () + shadeWidth + round;
    //        int maxY = drawBottom ? button.getHeight () - shadeWidth - 1 :
    //                button.getHeight () + shadeWidth + round;
    //        int width = maxX - x;
    //        int height = maxY - y;
    //
    //        x = x + 1;
    //        y = y + 1;
    //        width = width - 1;
    //        height = height - 1;
    //
    //        if ( round > 0 )
    //        {
    //            return new RoundRectangle2D.Double ( x, y, width, height, round * 2-2, round * 2-2 );
    //        }
    //        else
    //        {
    //            return new Rectangle2D.Double ( x, y, width, height );
    //        }
    //    }

    //    private Shape getButtonFocusShape ( AbstractButton button )
    //    {
    //        int x = ( drawLeft ? shadeWidth : 0 ) + ( drawLeftLine || drawLeft ? 2 : 1 );
    //        int y = ( drawTop ? shadeWidth : 0 ) + ( drawTopLine || drawTop ? 2 : 1 );
    //        int width = button.getWidth () -
    //                shadeWidth * ( drawLeft && drawRight ? 2 : ( drawRight || drawLeft ? 1 : 0 ) ) -
    //                ( drawLeftLine || drawLeft ? 2 : 1 ) - ( drawRightLine || drawRight ? 2 : 1 ) - 1;
    //        int height = button.getHeight () -
    //                shadeWidth * ( drawTop && drawBottom ? 2 : ( drawTop || drawBottom ? 1 : 0 ) ) -
    //                ( drawTopLine || drawTop ? 2 : 1 ) - ( drawBottomLine || drawBottom ? 2 : 1 ) - 1;
    //
    //        if ( round > 0 && ( drawTop && drawLeft || drawTop && drawRight || drawBottom && drawLeft ||
    //                drawBottom && drawRight ) )
    //        {
    //            return new RoundRectangle2D.Double ( x, y, width, height, round, round );
    //        }
    //        else
    //        {
    //            return new Rectangle2D.Double ( x, y, width, height );
    //        }
    //    }
}
