/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.checkbox;

import com.alee.laf.StyleConstants;
import com.alee.laf.list.WebListElement;
import com.alee.laf.tree.WebTreeCellRenderer;
import com.alee.utils.ColorUtils;
import com.alee.utils.ImageUtils;
import com.alee.utils.LafUtils;
import com.alee.utils.laf.ShapeProvider;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicCheckBoxUI;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.RoundRectangle2D;
import java.util.ArrayList;
import java.util.List;

/**
 * User: mgarin Date: 28.04.11 Time: 14:47
 */

public class WebCheckBoxUI extends BasicCheckBoxUI implements ShapeProvider
{
    public static final int MAX_DARKNESS = 5;

    public static List<ImageIcon> CHECK_STATES = new ArrayList<ImageIcon> ();
    public static ImageIcon DISABLED_CHECK = null;

    public static int updateDelay = 40;

    static
    {
        CHECK_STATES.add ( StyleConstants.EMPTY_ICON );
        for ( int i = 1; i <= 4; i++ )
        {
            CHECK_STATES.add ( new ImageIcon (
                    WebCheckBoxUI.class.getResource ( "icons/c" + i + ".png" ) ) );
        }

        DISABLED_CHECK = ImageUtils.getGrayscaleCopy ( "WebCheckBox.disabled.check",
                CHECK_STATES.get ( CHECK_STATES.size () - 1 ) );
    }

    private Color borderColor = WebCheckBoxStyle.borderColor;
    private Color darkBorderColor = WebCheckBoxStyle.darkBorderColor;
    private Color disabledBorderColor = WebCheckBoxStyle.disabledBorderColor;

    private Color topBgColor = WebCheckBoxStyle.topBgColor;
    private Color bottomBgColor = WebCheckBoxStyle.bottomBgColor;
    private Color topSelectedBgColor = WebCheckBoxStyle.topSelectedBgColor;
    private Color bottomSelectedBgColor = WebCheckBoxStyle.bottomSelectedBgColor;

    private int round = WebCheckBoxStyle.round;
    private int shadeWidth = WebCheckBoxStyle.shadeWidth;

    private boolean animated = WebCheckBoxStyle.animated;
    private boolean rolloverDarkBorderOnly = WebCheckBoxStyle.rolloverDarkBorderOnly;

    private int bgDarkness = 0;
    private boolean rollover;
    private Timer bgTimer;

    private int checkIcon = 0;
    private boolean checking;
    private Timer checkTimer;

    private JCheckBox checkBox = null;

    private MouseAdapter mouseAdapter;
    private ItemListener itemListener;

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebCheckBoxUI ();
    }

    public void installUI ( final JComponent c )
    {
        super.installUI ( c );

        checkBox = ( JCheckBox ) c;
        checkBox.setFocusable ( true );
        checkBox.setOpaque ( false );
        checkBox.setBorder ( BorderFactory.createEmptyBorder ( 0, 0, 0, 0 ) );
        checkBox.setMargin ( new Insets ( 0, 0, 0, 0 ) );

        updateIcon ( checkBox );

        bgTimer = new Timer ( updateDelay, new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                if ( rollover && bgDarkness < MAX_DARKNESS )
                {
                    bgDarkness++;
                    c.repaint ();
                }
                else if ( !rollover && bgDarkness > 0 )
                {
                    bgDarkness--;
                    c.repaint ();
                }
                else
                {
                    bgTimer.stop ();
                }
            }
        } );
        mouseAdapter = new MouseAdapter ()
        {
            public void mouseEntered ( MouseEvent e )
            {
                rollover = true;
                if ( isAnimated () && c.isEnabled () )
                {
                    bgTimer.start ();
                }
                else
                {
                    bgDarkness = MAX_DARKNESS;
                    c.repaint ();
                }
            }

            public void mouseExited ( MouseEvent e )
            {
                rollover = false;
                if ( isAnimated () && c.isEnabled () )
                {
                    bgTimer.start ();
                }
                else
                {
                    bgDarkness = 0;
                    c.repaint ();
                }
            }
        };
        checkBox.addMouseListener ( mouseAdapter );

        checkTimer = new Timer ( updateDelay, new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                if ( checking && checkIcon < CHECK_STATES.size () - 1 )
                {
                    checkIcon++;
                    c.repaint ();
                }
                else if ( !checking && checkIcon > 0 )
                {
                    checkIcon--;
                    c.repaint ();
                }
                else
                {
                    checkTimer.stop ();
                }
            }
        } );
        itemListener = new ItemListener ()
        {
            public void itemStateChanged ( ItemEvent e )
            {
                if ( isAnimated () && c.isEnabled () )
                {
                    if ( checkBox.isSelected () )
                    {
                        checking = true;
                        checkTimer.start ();
                    }
                    else
                    {
                        checking = false;
                        checkTimer.start ();
                    }
                }
                else
                {
                    checkIcon = checkBox.isSelected () ? CHECK_STATES.size () - 1 : 0;
                    c.repaint ();
                }
            }
        };
        checkBox.addItemListener ( itemListener );
    }

    public void uninstallUI ( JComponent c )
    {
        checkBox.removeMouseListener ( mouseAdapter );
        checkBox.removeItemListener ( itemListener );

        super.uninstallUI ( c );
    }

    public Shape provideShape ()
    {
        return LafUtils.getWebBorderShape ( checkBox, getShadeWidth (), getRound () );
    }

    public boolean isAnimated ()
    {
        return animated && ( checkBox == null || checkBox.getParent () == null ||
                !( checkBox.getParent () instanceof WebListElement ||
                        checkBox.getParent () instanceof WebTreeCellRenderer.WebTreeElement ) );
    }

    public void setAnimated ( boolean animated )
    {
        this.animated = animated;
    }

    public boolean isRolloverDarkBorderOnly ()
    {
        return rolloverDarkBorderOnly;
    }

    public void setRolloverDarkBorderOnly ( boolean rolloverDarkBorderOnly )
    {
        this.rolloverDarkBorderOnly = rolloverDarkBorderOnly;
    }

    public Color getBorderColor ()
    {
        return borderColor;
    }

    public void setBorderColor ( Color borderColor )
    {
        this.borderColor = borderColor;
    }

    public Color getDarkBorderColor ()
    {
        return darkBorderColor;
    }

    public void setDarkBorderColor ( Color darkBorderColor )
    {
        this.darkBorderColor = darkBorderColor;
    }

    public Color getDisabledBorderColor ()
    {
        return disabledBorderColor;
    }

    public void setDisabledBorderColor ( Color disabledBorderColor )
    {
        this.disabledBorderColor = disabledBorderColor;
    }

    public Color getTopBgColor ()
    {
        return topBgColor;
    }

    public void setTopBgColor ( Color topBgColor )
    {
        this.topBgColor = topBgColor;
    }

    public Color getBottomBgColor ()
    {
        return bottomBgColor;
    }

    public void setBottomBgColor ( Color bottomBgColor )
    {
        this.bottomBgColor = bottomBgColor;
    }

    public Color getTopSelectedBgColor ()
    {
        return topSelectedBgColor;
    }

    public void setTopSelectedBgColor ( Color topSelectedBgColor )
    {
        this.topSelectedBgColor = topSelectedBgColor;
    }

    public Color getBottomSelectedBgColor ()
    {
        return bottomSelectedBgColor;
    }

    public void setBottomSelectedBgColor ( Color bottomSelectedBgColor )
    {
        this.bottomSelectedBgColor = bottomSelectedBgColor;
    }

    public int getRound ()
    {
        return round;
    }

    public void setRound ( int round )
    {
        this.round = round;
    }

    public int getShadeWidth ()
    {
        return shadeWidth;
    }

    public void setShadeWidth ( int shadeWidth )
    {
        this.shadeWidth = shadeWidth;
    }

    private void updateIcon ( final JCheckBox checkBox )
    {
        checkBox.setIcon ( new Icon ()
        {
            public void paintIcon ( Component c, Graphics g, int x, int y )
            {
                Graphics2D g2d = ( Graphics2D ) g;
                g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING,
                        RenderingHints.VALUE_ANTIALIAS_ON );
                g2d.setStroke ( StyleConstants.fieldFocusStroke );

                // Button size and shape
                Rectangle iconRect = new Rectangle ( x + 1, y + 1, 13, 13 );
                RoundRectangle2D shape =
                        new RoundRectangle2D.Double ( iconRect.x, iconRect.y, iconRect.width,
                                iconRect.height, round * 2, round * 2 );

                // Shade
                if ( c.isEnabled () )
                {
                    LafUtils.drawShade ( g2d, shape, StyleConstants.shadeColor, shadeWidth );
                }

                // Background
                int radius =
                        Math.round ( ( float ) Math.sqrt ( iconRect.width * iconRect.width / 2 ) );
                g2d.setPaint ( new RadialGradientPaint ( iconRect.x + iconRect.width / 2,
                        iconRect.y + iconRect.height / 2, radius, new float[]{ 0f, 1f },
                        getBgColors ( checkBox ) ) );
                g2d.fill ( shape );

                // Border
                g2d.setPaint ( c.isEnabled () ? ( rolloverDarkBorderOnly ?
                        ColorUtils.getProgress ( borderColor, darkBorderColor, getProgress () ) :
                        darkBorderColor ) : disabledBorderColor );
                g2d.draw ( shape );

                // Focus
                LafUtils.drawCustonWebFocus ( g2d, checkBox, StyleConstants.focusType, shape );

                // Check icon
                if ( checkIcon > 0 )
                {
                    if ( checkBox.isEnabled () )
                    {
                        g2d.drawImage ( CHECK_STATES.get ( checkIcon ).getImage (), x, y, null );
                    }
                    else
                    {
                        g2d.drawImage ( DISABLED_CHECK.getImage (), x, y, null );
                    }
                }
            }

            public int getIconWidth ()
            {
                return 16;
            }

            public int getIconHeight ()
            {
                return 16;
            }
        } );
    }

    private Color[] getBgColors ( JCheckBox checkBox )
    {
        if ( checkBox.isEnabled () )
        {
            float progress = getProgress ();
            if ( progress < 1f )
            {
                return new Color[]{
                        ColorUtils.getProgress ( topBgColor, topSelectedBgColor, progress ),
                        ColorUtils.getProgress ( bottomBgColor, bottomSelectedBgColor, progress ) };
            }
            else
            {
                return new Color[]{ topSelectedBgColor, bottomSelectedBgColor };
            }
        }
        else
        {
            return new Color[]{ topBgColor, bottomBgColor };
        }
    }

    private float getProgress ()
    {
        return ( float ) bgDarkness / MAX_DARKNESS;
    }
}
