/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.panel;

import com.alee.laf.BackgroundPainter;
import com.alee.laf.StyleConstants;
import com.alee.utils.LafUtils;
import com.alee.utils.SwingUtils;
import com.alee.utils.laf.ShapeProvider;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicPanelUI;
import java.awt.*;

/**
 * User: mgarin Date: 26.07.11 Time: 13:11
 */

public class WebPanelUI extends BasicPanelUI implements ShapeProvider
{
    private boolean undecorated = WebPanelStyle.undecorated;
    private int round = WebPanelStyle.round;
    private int shadeWidth = WebPanelStyle.shadeWidth;
    private boolean fillBackground = WebPanelStyle.fillBackground;
    private boolean webColored = WebPanelStyle.webColored;
    private BackgroundPainter backgroundPainter = WebPanelStyle.backgroundPainter;

    private JPanel panel = null;

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebPanelUI ();
    }

    public void installUI ( JComponent c )
    {
        super.installUI ( c );

        panel = ( JPanel ) c;

        c.setOpaque ( true );
        c.setBackground ( WebPanelStyle.backgroundColor );

        updateBorder ( c );
    }

    public Shape provideShape ()
    {
        return LafUtils.getWebBorderShape ( panel, getShadeWidth (), getRound () );
    }

    private void updateBorder ( JComponent component )
    {
        if ( backgroundPainter != null )
        {
            // Background insets
            Insets bi = backgroundPainter.getBackgroundMargin ( component );
            component.setBorder (
                    BorderFactory.createEmptyBorder ( bi.top, bi.left, bi.bottom, bi.right ) );
        }
        else if ( !undecorated )
        {
            // Panel insets
            component.setBorder ( BorderFactory
                    .createEmptyBorder ( shadeWidth + 1, shadeWidth + 1, shadeWidth + 1,
                            shadeWidth + 1 ) );
        }
        else
        {
            // Empty insets
            component.setBorder ( BorderFactory.createEmptyBorder ( 0, 0, 0, 0 ) );
        }
    }

    public boolean isUndecorated ()
    {
        return undecorated;
    }

    public void setUndecorated ( boolean undecorated )
    {
        this.undecorated = undecorated;

        // Updating border
        updateBorder ( panel );

        // Updating opaque value
        if ( !undecorated )
        {
            panel.setOpaque ( false );
        }
    }

    public BackgroundPainter getBackgroundPainter ()
    {
        return backgroundPainter;
    }

    public void setBackgroundPainter ( BackgroundPainter backgroundPainter )
    {
        this.backgroundPainter = backgroundPainter;
        updateBorder ( panel );
    }

    public int getRound ()
    {
        if ( undecorated )
        {
            return 0;
        }
        else
        {
            return round;
        }
    }

    public void setRound ( int round )
    {
        this.round = round;
    }

    public int getShadeWidth ()
    {
        if ( undecorated )
        {
            return 0;
        }
        else
        {
            return shadeWidth;
        }
    }

    public void setShadeWidth ( int shadeWidth )
    {
        this.shadeWidth = shadeWidth;
        updateBorder ( panel );
    }

    public boolean isFillBackground ()
    {
        return fillBackground;
    }

    public void setFillBackground ( boolean fillBackground )
    {
        this.fillBackground = fillBackground;
    }

    public boolean isWebColored ()
    {
        return webColored;
    }

    public void setWebColored ( boolean webColored )
    {
        this.webColored = webColored;
    }

    public void paint ( Graphics g, JComponent c )
    {
        if ( backgroundPainter != null )
        {
            // Use background painter instead of default UI graphics
            backgroundPainter.paintBackground ( g, SwingUtils.size ( c ), c );
        }
        else if ( !undecorated )
        {
            // Border and background
            LafUtils.drawWebBorder ( ( Graphics2D ) g, c, StyleConstants.shadeColor, shadeWidth,
                    round, fillBackground, webColored );
        }
    }

    public Dimension getPreferredSize ( JComponent c )
    {
        Dimension ps = c.getLayout () != null ? c.getLayout ().preferredLayoutSize ( c ) : null;
        if ( backgroundPainter != null )
        {
            ps = SwingUtils.max ( ps, backgroundPainter.getBackgroundPreferredSize ( c ) );
        }
        return ps;
    }
}
