/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.text;

import com.alee.laf.BackgroundPainter;
import com.alee.laf.StyleConstants;
import com.alee.utils.SwingUtils;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicTextAreaUI;
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

/**
 * User: mgarin Date: 17.08.11 Time: 23:01
 */

public class WebTextAreaUI extends BasicTextAreaUI
{
    private BackgroundPainter backgroundPainter = WebTextAreaStyle.backgroundPainter;

    private FocusListener focusListener;

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebTextAreaUI ();
    }

    public void installUI ( JComponent c )
    {
        super.installUI ( c );

        final JTextComponent component = getComponent ();

        updateBorder ();

        component.setFocusable ( true );
        component.setOpaque ( true );
        component.setBackground ( Color.WHITE );

        component.setSelectionColor ( StyleConstants.textSelectionColor );
        component.setForeground ( Color.BLACK );
        component.setSelectedTextColor ( Color.BLACK );

        component.setCaretColor ( Color.GRAY );

        focusListener = new FocusListener ()
        {
            public void focusLost ( FocusEvent e )
            {
                component.repaint ();
            }

            public void focusGained ( FocusEvent e )
            {
                component.repaint ();
            }
        };
        component.addFocusListener ( focusListener );
    }

    public void uninstallUI ( JComponent c )
    {
        getComponent ().removeFocusListener ( focusListener );

        super.uninstallUI ( c );
    }

    public BackgroundPainter getBackgroundPainter ()
    {
        return backgroundPainter;
    }

    public void setBackgroundPainter ( BackgroundPainter backgroundPainter )
    {
        this.backgroundPainter = backgroundPainter;
        updateBorder ();
    }

    private void updateBorder ()
    {
        JTextComponent component = getComponent ();
        if ( component != null )
        {
            if ( backgroundPainter != null )
            {
                // Background insets
                Insets margin = backgroundPainter.getBackgroundMargin ( component );
                component.setBorder ( BorderFactory
                        .createEmptyBorder ( margin.top, margin.left, margin.bottom,
                                margin.right ) );
            }
            else
            {
                // Empty insets
                component.setBorder ( BorderFactory.createEmptyBorder ( 2, 2, 2, 2 ) );
            }
        }
    }

    protected void paintBackground ( Graphics g )
    {
        if ( backgroundPainter == null )
        {
            super.paintBackground ( g );
        }
    }

    protected void paintSafely ( Graphics g )
    {
        if ( backgroundPainter != null )
        {
            // Use background painter instead of default UI graphics
            JTextComponent c = getComponent ();
            backgroundPainter.paintBackground ( g, SwingUtils.size ( c ), c );
        }

        super.paintSafely ( g );
    }

    public Dimension getPreferredSize ( JComponent c )
    {
        Dimension ps = super.getPreferredSize ( c );
        if ( backgroundPainter != null )
        {
            if ( c.getLayout () != null )
            {
                ps = SwingUtils.max ( ps, c.getLayout ().preferredLayoutSize ( c ) );
            }
            ps = SwingUtils.max ( ps, backgroundPainter.getBackgroundPreferredSize ( c ) );
        }
        return ps;
    }
}
