/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.text;

import com.alee.extended.layout.ToolbarLayout;
import com.alee.laf.BackgroundPainter;
import com.alee.laf.StyleConstants;
import com.alee.utils.LafUtils;
import com.alee.utils.SwingUtils;
import com.alee.utils.laf.ShapeProvider;

import javax.accessibility.AccessibleContext;
import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicTextFieldUI;
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * User: mgarin Date: 16.05.11 Time: 16:39
 */

public class WebTextFieldUI extends BasicTextFieldUI implements ShapeProvider
{
    private JTextField textField = null;

    private boolean drawBorder = StyleConstants.drawBorder;
    private boolean drawFocus = StyleConstants.drawFocus;
    private int round = StyleConstants.smallRound;
    private int shadeWidth = StyleConstants.shadeWidth;
    private BackgroundPainter backgroundPainter = StyleConstants.backgroundPainter;

    private int componentSpacing = StyleConstants.contentSpacing;
    private JComponent leadingComponent = null;
    private JComponent trailingComponent = null;
    private Insets layoutMargin = null;

    //    private boolean mouseover = false;

    private MouseAdapter mouseAdapter;
    private KeyAdapter keyAdapter;
    private FocusListener focusListener;
    private PropertyChangeListener propertyChangeListener;

    public WebTextFieldUI ( final JTextField textField )
    {
        this ( textField, true );
    }

    public WebTextFieldUI ( final JTextField textField, boolean drawBorder )
    {
        super ();
        this.textField = textField;
        this.drawBorder = drawBorder;
    }

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebTextFieldUI ( ( JTextField ) c );
    }

    public void installUI ( JComponent c )
    {
        super.installUI ( c );

        updateBorder ();

        textField.setFocusable ( true );
        textField.setOpaque ( false );
        textField.setBackground ( Color.WHITE );

        //        textField.setHighlighter ( new WebHighlighter () );
        textField.setSelectionColor ( StyleConstants.textSelectionColor );
        textField.setForeground ( Color.BLACK );
        textField.setSelectedTextColor ( Color.BLACK );

        textField.setCaretColor ( Color.GRAY );

        mouseAdapter = new MouseAdapter ()
        {
            public void mousePressed ( MouseEvent e )
            {
                textField.repaint ();
            }

            public void mouseDragged ( MouseEvent e )
            {
                textField.repaint ();
            }

            public void mouseReleased ( MouseEvent e )
            {
                textField.repaint ();
            }

            //            public void mouseEntered ( MouseEvent e )
            //            {
            //                textField.repaint ();
            //                mouseover = true;
            //            }
            //
            //            public void mouseExited ( MouseEvent e )
            //            {
            //                textField.repaint ();
            //                mouseover = false;
            //            }
        };
        textField.addMouseListener ( mouseAdapter );
        textField.addMouseMotionListener ( mouseAdapter );

        keyAdapter = new KeyAdapter ()
        {
            public void keyPressed ( KeyEvent e )
            {
                textField.repaint ();
            }
        };
        textField.addKeyListener ( keyAdapter );

        focusListener = new FocusListener ()
        {
            public void focusLost ( FocusEvent e )
            {
                textField.repaint ();
            }

            public void focusGained ( FocusEvent e )
            {
                textField.repaint ();
            }
        };
        textField.addFocusListener ( focusListener );

        propertyChangeListener = new PropertyChangeListener ()
        {
            public void propertyChange ( PropertyChangeEvent evt )
            {
                updateInnerComponents ();
            }
        };
        textField.addPropertyChangeListener ( AccessibleContext.ACCESSIBLE_STATE_PROPERTY,
                propertyChangeListener );
    }

    public void uninstallUI ( JComponent c )
    {
        textField.removeMouseListener ( mouseAdapter );
        textField.removeMouseMotionListener ( mouseAdapter );
        textField.removeKeyListener ( keyAdapter );
        textField.removeFocusListener ( focusListener );
        textField.removePropertyChangeListener ( propertyChangeListener );

        if ( this.leadingComponent != null )
        {
            textField.remove ( this.leadingComponent );
        }
        if ( this.trailingComponent != null )
        {
            textField.remove ( this.trailingComponent );
        }
        textField.setLayout ( null );

        super.uninstallUI ( c );
    }

    public Shape provideShape ()
    {
        return LafUtils.getWebBorderShape ( textField, getShadeWidth (), getRound () );
    }

    private void updateInnerComponents ()
    {
        if ( leadingComponent != null )
        {
            leadingComponent.setEnabled ( textField.isEnabled () );
        }
        if ( trailingComponent != null )
        {
            trailingComponent.setEnabled ( textField.isEnabled () );
        }
    }

    public JComponent getLeadingComponent ()
    {
        return leadingComponent;
    }

    public void setLeadingComponent ( JComponent leadingComponent )
    {
        if ( this.leadingComponent != null )
        {
            textField.remove ( this.leadingComponent );
        }
        this.leadingComponent = leadingComponent;
        updateBorder ();
        textField.add ( leadingComponent, ToolbarLayout.START );
        updateInnerComponents ();
    }

    public JComponent getTrailingComponent ()
    {
        return trailingComponent;
    }

    public void setTrailingComponent ( JComponent trailingComponent )
    {
        if ( this.trailingComponent != null )
        {
            textField.remove ( this.trailingComponent );
        }
        this.trailingComponent = trailingComponent;
        updateBorder ();
        textField.add ( trailingComponent, ToolbarLayout.END );
        updateInnerComponents ();
    }

    public int getComponentSpacing ()
    {
        return componentSpacing;
    }

    public void setComponentSpacing ( int componentSpacing )
    {
        this.componentSpacing = componentSpacing;
        updateBorder ();
    }

    public void setLayoutMargin ( Insets margin )
    {
        this.layoutMargin = margin;
        updateBorder ();
    }

    public Insets getLayoutMargin ()
    {
        return layoutMargin;
    }

    private void updateBorder ()
    {
        if ( textField != null )
        {
            // Default web border
            Insets border;
            if ( backgroundPainter != null )
            {
                border = backgroundPainter.getBackgroundMargin ( getComponent () );
            }
            else if ( drawBorder )
            {
                border = new Insets ( shadeWidth + 3, shadeWidth + 3, shadeWidth + 3,
                        shadeWidth + 3 );
            }
            else
            {
                border = new Insets ( 1, 1, 1, 1 );
            }

            // Proper layout for inner components that ignores additional border
            ToolbarLayout textFieldLayout =
                    new ToolbarLayout ( 0, componentSpacing * 2, ToolbarLayout.HORIZONTAL );
            textFieldLayout.setMargin ( new Insets (
                    layoutMargin != null && layoutMargin.top != -1 ? layoutMargin.top : border.top,
                    layoutMargin != null && layoutMargin.left != -1 ? layoutMargin.left :
                            border.left,
                    layoutMargin != null && layoutMargin.bottom != -1 ? layoutMargin.bottom :
                            border.bottom,
                    layoutMargin != null && layoutMargin.right != -1 ? layoutMargin.right :
                            border.right ) );
            // Restoring added components constraints
            if ( textField.getLayout () instanceof ToolbarLayout )
            {
                textFieldLayout.setConstraints (
                        ( ( ToolbarLayout ) textField.getLayout () ).getConstraints () );
            }
            textField.setLayout ( textFieldLayout );

            // Additional border to avoid components intersection
            if ( leadingComponent != null )
            {
                border.left += leadingComponent.getPreferredSize ().width + componentSpacing;
            }
            if ( trailingComponent != null )
            {
                border.right += componentSpacing + trailingComponent.getPreferredSize ().width;
            }

            // Setting final border
            textField.setBorder ( BorderFactory
                    .createEmptyBorder ( border.top, border.left, border.bottom, border.right ) );
        }
    }

    public int getShadeWidth ()
    {
        return shadeWidth;
    }

    public void setShadeWidth ( int shadeWidth )
    {
        this.shadeWidth = shadeWidth;
        updateBorder ();
    }

    public int getRound ()
    {
        return round;
    }

    public void setRound ( int round )
    {
        this.round = round;
    }

    public boolean isDrawBorder ()
    {
        return drawBorder;
    }

    public void setDrawBorder ( boolean drawBorder )
    {
        this.drawBorder = drawBorder;
        updateBorder ();
    }

    public boolean isDrawFocus ()
    {
        return drawFocus;
    }

    public void setDrawFocus ( boolean drawFocus )
    {
        this.drawFocus = drawFocus;
    }

    public BackgroundPainter getBackgroundPainter ()
    {
        return backgroundPainter;
    }

    public void setBackgroundPainter ( BackgroundPainter backgroundPainter )
    {
        this.backgroundPainter = backgroundPainter;
        updateBorder ();
    }

    protected void paintBackground ( Graphics g )
    {
        if ( backgroundPainter == null && !drawBorder )
        {
            super.paintBackground ( g );
        }
    }

    protected void paintSafely ( Graphics g )
    {
        if ( backgroundPainter != null || drawBorder )
        {
            JTextComponent c = getComponent ();
            Graphics2D g2d = ( Graphics2D ) g;
            Object aa = LafUtils.setupAntialias ( g2d );

            if ( backgroundPainter != null )
            {
                // Use background painter instead of default UI graphics
                backgroundPainter.paintBackground ( g, SwingUtils.size ( c ), c );
            }
            else if ( drawBorder )
            {
                // Border and background
                LafUtils.drawWebBorder ( g2d, c, StyleConstants.shadeColor, shadeWidth, round, true,
                        false );

                // Focus
                if ( drawFocus )
                {
                    LafUtils.drawWebFocus ( g2d, c, StyleConstants.focusType, shadeWidth, round );
                }
            }
            LafUtils.restoreAntialias ( g2d, aa );
        }

        super.paintSafely ( g );
    }

    public Dimension getPreferredSize ( JComponent c )
    {
        Dimension ps = super.getPreferredSize ( c );

        // Fix for 1 pixel scrolling when preferred size reached
        ps.width += 1;

        // Height might be changed due to inner components
        if ( leadingComponent != null || trailingComponent != null )
        {
            Dimension lps = c.getLayout ().preferredLayoutSize ( c );
            Insets border = c.getInsets ();
            ps.height = Math.max ( ps.height, lps.height );
        }

        // Background painter preferred size
        if ( backgroundPainter != null )
        {
            ps = SwingUtils.max ( ps, backgroundPainter.getBackgroundPreferredSize ( c ) );
        }

        return ps;
    }
}
