/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.toolbar;

import com.alee.extended.layout.ToolbarLayout;
import com.alee.laf.StyleConstants;
import com.alee.utils.LafUtils;
import com.alee.utils.laf.ShapeProvider;

import javax.swing.*;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicToolBarUI;
import java.awt.*;
import java.awt.event.WindowListener;
import java.awt.geom.RoundRectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * User: mgarin Date: 17.08.11 Time: 23:06
 */

public class WebToolBarUI extends BasicToolBarUI implements ShapeProvider
{
    public static final int gripperSpace = 5;

    private Color topBgColor = WebToolbarStyle.topBgColor;
    private Color bottomBgColor = WebToolbarStyle.bottomBgColor;
    private Color borderColor = WebToolbarStyle.borderColor;

    private int round = WebToolbarStyle.round;
    private int shadeWidth = WebToolbarStyle.shadeWidth;
    private int spacing = WebToolbarStyle.spacing;
    private int contentSpacing = WebToolbarStyle.contentSpacing;

    private ToolbarStyle toolbarStyle = WebToolbarStyle.toolbarStyle;

    private AncestorListener ancestorListener;
    private PropertyChangeListener propertyChangeListener;

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebToolBarUI ();
    }

    public void installUI ( final JComponent c )
    {
        super.installUI ( c );

        c.setOpaque ( false );

        updateBorder ( c );
        updateLayout ( c );

        ancestorListener = new AncestorListener ()
        {
            public void ancestorAdded ( AncestorEvent event )
            {
                updateBorder ( c );
                updateLayout ( c );
            }

            public void ancestorRemoved ( AncestorEvent event )
            {
                //
            }

            public void ancestorMoved ( AncestorEvent event )
            {
                //
            }
        };
        c.addAncestorListener ( ancestorListener );

        propertyChangeListener = new PropertyChangeListener ()
        {
            public void propertyChange ( PropertyChangeEvent evt )
            {
                updateBorder ( c );
                updateLayout ( c );
            }
        };
        c.addPropertyChangeListener ( "floatable", propertyChangeListener );
    }

    public void uninstallUI ( JComponent c )
    {
        c.removeAncestorListener ( ancestorListener );
        c.removePropertyChangeListener ( propertyChangeListener );

        super.uninstallUI ( c );
    }

    public Shape provideShape ()
    {
        return LafUtils.getWebBorderShape ( toolBar, getShadeWidth (), getRound () );
    }

    public int getRound ()
    {
        return round;
    }

    public void setRound ( int round )
    {
        this.round = round;
    }

    public Color getTopBgColor ()
    {
        return topBgColor;
    }

    public void setTopBgColor ( Color topBgColor )
    {
        this.topBgColor = topBgColor;
    }

    public Color getBottomBgColor ()
    {
        return bottomBgColor;
    }

    public void setBottomBgColor ( Color bottomBgColor )
    {
        this.bottomBgColor = bottomBgColor;
    }

    public Color getBorderColor ()
    {
        return borderColor;
    }

    public void setBorderColor ( Color borderColor )
    {
        this.borderColor = borderColor;
    }

    public int getShadeWidth ()
    {
        return shadeWidth;
    }

    public void setShadeWidth ( int shadeWidth )
    {
        this.shadeWidth = shadeWidth;
        updateBorder ( toolBar );
    }

    public ToolbarStyle getToolbarStyle ()
    {
        return toolbarStyle;
    }

    public void setToolbarStyle ( ToolbarStyle toolbarStyle )
    {
        this.toolbarStyle = toolbarStyle;
        updateBorder ( toolBar );
    }

    public int getSpacing ()
    {
        return spacing;
    }

    public void setSpacing ( int spacing )
    {
        this.spacing = spacing;
        updateLayout ( toolBar );
    }

    private void updateBorder ( JComponent c )
    {
        int gripperSpacing = ( ( JToolBar ) c ).isFloatable () ? gripperSpace : 0;
        boolean horizontal = ( ( JToolBar ) c ).getOrientation () == WebToolBar.HORIZONTAL;
        if ( toolbarStyle.equals ( ToolbarStyle.standalone ) )
        {
            if ( isFloating () )
            {
                c.setBorder ( BorderFactory
                        .createEmptyBorder ( contentSpacing + ( !horizontal ? gripperSpacing : 0 ),
                                contentSpacing + ( horizontal ? gripperSpacing : 0 ),
                                contentSpacing, contentSpacing ) );
            }
            else
            {
                c.setBorder ( BorderFactory.createEmptyBorder (
                        contentSpacing + 1 + shadeWidth + ( !horizontal ? gripperSpacing : 0 ),
                        contentSpacing + 1 + shadeWidth + ( horizontal ? gripperSpacing : 0 ),
                        contentSpacing + 1 + shadeWidth, contentSpacing + 1 + shadeWidth ) );
            }
        }
        else
        {
            boolean leftToRight = c.getComponentOrientation ().isLeftToRight ();
            if ( isFloating () )
            {
                c.setBorder ( BorderFactory
                        .createEmptyBorder ( contentSpacing + ( !horizontal ? gripperSpacing : 0 ),
                                contentSpacing + ( horizontal ? gripperSpacing : 0 ),
                                contentSpacing, contentSpacing ) );
            }
            else
            {
                c.setBorder ( BorderFactory
                        .createEmptyBorder ( contentSpacing + ( !horizontal ? gripperSpacing : 0 ) +
                                ( horizontal && !leftToRight ? 1 : 0 ),
                                contentSpacing + ( horizontal ? gripperSpacing : 0 ) +
                                        ( !horizontal && !leftToRight ? 1 : 0 ),
                                contentSpacing + ( horizontal && leftToRight ? 1 : 0 ),
                                contentSpacing + ( !horizontal && leftToRight ? 1 : 0 ) ) );
            }
        }
    }

    private void updateLayout ( JComponent c )
    {
        ToolbarLayout layout = new ToolbarLayout ( spacing, toolBar.getOrientation () );
        if ( c.getLayout () instanceof ToolbarLayout )
        {
            ToolbarLayout old = ( ToolbarLayout ) c.getLayout ();
            layout.setConstraints ( old.getConstraints () );
        }
        c.setLayout ( layout );
    }

    private Color color = new Color ( 158, 158, 158 );
    private Color transparent = new Color ( 0, 0, 0, 0 );
    private Color[] gradient = new Color[]{ transparent, color, color, transparent };
    private float[] fractions = { 0f, 0.33f, 0.66f, 1f };

    public void paint ( Graphics g, JComponent c )
    {
        Graphics2D g2d = ( Graphics2D ) g;

        Object aa = g2d.getRenderingHint ( RenderingHints.KEY_ANTIALIASING );
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );

        boolean horizontal = toolBar.getOrientation () == WebToolBar.HORIZONTAL;

        if ( isFloating () )
        {
            if ( horizontal )
            {
                g2d.setPaint (
                        new GradientPaint ( 0, c.getHeight () / 2, topBgColor, 0, c.getHeight (),
                                bottomBgColor ) );
            }
            else
            {
                g2d.setPaint (
                        new GradientPaint ( c.getWidth () / 2, 0, topBgColor, c.getWidth (), 0,
                                bottomBgColor ) );
            }
            g2d.fillRect ( 0, 0, c.getWidth (), c.getHeight () );
        }
        else
        {
            if ( toolbarStyle.equals ( ToolbarStyle.standalone ) )
            {
                RoundRectangle2D rr = new RoundRectangle2D.Double ( shadeWidth, shadeWidth,
                        c.getWidth () - shadeWidth * 2 - 1, c.getHeight () - shadeWidth * 2 - 1,
                        round, round );
                LafUtils.drawShade ( g2d, rr, StyleConstants.shadeColor, shadeWidth );

                if ( horizontal )
                {
                    g2d.setPaint ( new GradientPaint ( 0, c.getHeight () / 2, topBgColor, 0,
                            c.getHeight (), bottomBgColor ) );
                }
                else
                {
                    g2d.setPaint (
                            new GradientPaint ( c.getWidth () / 2, 0, topBgColor, c.getWidth (), 0,
                                    bottomBgColor ) );
                }
                g2d.fill ( rr );

                g2d.setPaint ( borderColor );
                g2d.draw ( rr );
            }
            else
            {
                boolean leftToRight = c.getComponentOrientation ().isLeftToRight ();

                if ( horizontal )
                {
                    g2d.setPaint ( new GradientPaint ( 0, c.getHeight () / 2, topBgColor, 0,
                            c.getHeight (), bottomBgColor ) );
                }
                else
                {
                    g2d.setPaint (
                            new GradientPaint ( c.getWidth () / 2, 0, topBgColor, c.getWidth (), 0,
                                    bottomBgColor ) );
                }
                g2d.fillRect ( 0, 0, c.getWidth (), c.getHeight () );

                g2d.setPaint ( borderColor );
                if ( horizontal )
                {
                    if ( leftToRight )
                    {
                        g2d.drawLine ( 0, c.getHeight () - 1, c.getWidth () - 1,
                                c.getHeight () - 1 );
                    }
                    else
                    {
                        g2d.drawLine ( 0, 0, c.getWidth () - 1, 0 );
                    }
                }
                else
                {
                    if ( leftToRight )
                    {
                        g2d.drawLine ( c.getWidth () - 1, 0, c.getWidth () - 1,
                                c.getHeight () - 1 );
                    }
                    else
                    {
                        g2d.drawLine ( 0, 0, 0, c.getHeight () - 1 );
                    }
                }
            }
        }

        if ( toolBar.isFloatable () )
        {
            if ( toolBar.getOrientation () == WebToolBar.HORIZONTAL )
            {
                int gradY = shadeWidth + 1;
                int gradEndY = c.getHeight () - shadeWidth - 2;
                if ( gradEndY > gradY )
                {
                    g2d.setPaint ( new LinearGradientPaint ( 0, gradY, 0, gradEndY, fractions,
                            gradient ) );

                    int x = toolbarStyle.equals ( ToolbarStyle.standalone ) ?
                            shadeWidth + 1 + contentSpacing + ( isFloating () ? -1 : 1 ) :
                            contentSpacing + gripperSpace / 2 - 1;
                    for ( int i = c.getHeight () / 2 - 3; i >= gradY; i -= 4 )
                    {
                        g2d.fillRect ( x, i, 2, 2 );
                    }
                    for ( int i = c.getHeight () / 2 + 1; i + 2 <= gradEndY; i += 4 )
                    {
                        g2d.fillRect ( x, i, 2, 2 );
                    }
                }
            }
            else
            {
                int gradX = shadeWidth + 1;
                int gradEndX = c.getWidth () - shadeWidth - 2;
                if ( gradEndX > gradX )
                {
                    g2d.setPaint ( new LinearGradientPaint ( gradX, 0, gradEndX, 0, fractions,
                            gradient ) );

                    int y = toolbarStyle.equals ( ToolbarStyle.standalone ) ?
                            shadeWidth + 1 + contentSpacing + ( isFloating () ? -1 : 1 ) :
                            contentSpacing + gripperSpace / 2 - 1;
                    for ( int i = c.getWidth () / 2 - 3; i >= gradX; i -= 4 )
                    {
                        g2d.fillRect ( i, y, 2, 2 );
                    }
                    for ( int i = c.getWidth () / 2 + 1; i + 2 <= gradEndX; i += 4 )
                    {
                        g2d.fillRect ( i, y, 2, 2 );
                    }
                }
            }
        }

        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, aa );
    }

    protected RootPaneContainer createFloatingWindow ( JToolBar toolbar )
    {
        class ToolBarDialog extends JDialog
        {
            public ToolBarDialog ( Frame owner, String title, boolean modal )
            {
                super ( owner, title, modal );
            }

            public ToolBarDialog ( Dialog owner, String title, boolean modal )
            {
                super ( owner, title, modal );
            }

            // Override createRootPane() to automatically resize
            // the frame when contents change
            protected JRootPane createRootPane ()
            {
                JRootPane rootPane = new JRootPane ()
                {
                    private boolean packing = false;

                    public void validate ()
                    {
                        super.validate ();
                        if ( !packing )
                        {
                            packing = true;
                            pack ();
                            packing = false;
                        }
                    }
                };
                rootPane.setOpaque ( true );
                return rootPane;
            }
        }

        JDialog dialog;
        Window window = SwingUtilities.getWindowAncestor ( toolbar );
        if ( window instanceof Frame )
        {
            dialog = new ToolBarDialog ( ( Frame ) window, toolbar.getName (), false );
        }
        else if ( window instanceof Dialog )
        {
            dialog = new ToolBarDialog ( ( Dialog ) window, toolbar.getName (), false );
        }
        else
        {
            dialog = new ToolBarDialog ( ( Frame ) null, toolbar.getName (), false );
        }

        dialog.getRootPane ().setName ( "ToolBar.FloatingWindow" );
        dialog.setTitle ( toolbar.getName () );
        dialog.setResizable ( false );
        WindowListener wl = createFrameListener ();
        dialog.addWindowListener ( wl );
        //        dialog.setUndecorated ( true );
        return dialog;
    }

    //    protected void paintDragWindow ( Graphics g )
    //    {
    //        super.paintDragWindow ( g );
    //        System.out.println (
    //                "can fock now: " + dragWindow.getBorderColor ().equals ( dockingBorderColor ) );
    //        System.out.println (
    //                "horizontal: " + ( dragWindow.getOrientation () == WebToolBar.HORIZONTAL ) );
    //        toolBar.printAll ( g );
    //    }


    //    protected DragWindow createDragWindow ( JToolBar toolbar )
    //    {
    //        DragWindow dragWindow = super.createDragWindow ( toolbar );
    //        return dragWindow;
    //    }
}
