/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.tree;

import com.alee.laf.StyleConstants;
import com.alee.laf.text.WebTextField;
import com.alee.utils.CollectionUtils;
import com.alee.utils.GeometryUtils;
import com.alee.utils.LafUtils;
import com.alee.utils.SwingUtils;
import com.alee.utils.laf.ShapeProvider;

import javax.swing.*;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicTreeUI;
import javax.swing.tree.TreeCellEditor;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.RoundRectangle2D;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * User: mgarin Date: 28.04.11 Time: 17:35
 */

public class WebTreeUI extends BasicTreeUI implements ShapeProvider
{
    public static ImageIcon EXPAND_ICON =
            new ImageIcon ( WebTreeUI.class.getResource ( "icons/expand.png" ) );
    public static ImageIcon COLLAPSE_ICON =
            new ImageIcon ( WebTreeUI.class.getResource ( "icons/collapse.png" ) );

    public static ImageIcon CLOSED_ICON =
            new ImageIcon ( WebTreeUI.class.getResource ( "icons/closed.png" ) );
    public static ImageIcon OPEN_ICON =
            new ImageIcon ( WebTreeUI.class.getResource ( "icons/open.png" ) );
    public static ImageIcon LEAF_ICON =
            new ImageIcon ( WebTreeUI.class.getResource ( "icons/leaf.png" ) );

    private TreeSelectionStyle selectionStyle = WebTreeStyle.selectionStyle;
    private boolean showMouseoverHighlight = true;

    private int round = WebTreeStyle.round;
    private int shadeWidth = WebTreeStyle.shadeWidth;

    private Color selectionColor = new Color ( 0, 0, 255, 50 );
    private Color selectionBorderColor = Color.GRAY;
    private int selectionRound = StyleConstants.smallRound;

    private boolean allowMultiselectorMode = true;

    private TreeSelectionListener treeSelectionListener;
    private TreeExpansionListener treeExpansionListener;

    private MouseAdapter mouseoverAdapter;
    private int rolloverRow = -1;
    private List<Integer> initialSelection = new ArrayList<Integer> ();
    private List<Integer> lastSelection = null;
    private Point selectionStart = null;
    private Point selectionEnd = null;

    private boolean paintLines = true;
    private Color linesColor = Color.GRAY;

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebTreeUI ();
    }

    public void installUI ( JComponent c )
    {
        super.installUI ( c );

        this.paintLines = UIManager.getBoolean ( "Tree.paintLines" );

        setExpandedIcon ( COLLAPSE_ICON );
        setCollapsedIcon ( EXPAND_ICON );

        final JTree tree = ( JTree ) c;
        tree.setRowHeight ( -1 );
        tree.setVisibleRowCount ( 10 );

        treeSelectionListener = new TreeSelectionListener ()
        {
            public void valueChanged ( TreeSelectionEvent e )
            {
                tree.repaint ();
            }
        };
        tree.addTreeSelectionListener ( treeSelectionListener );

        treeExpansionListener = new TreeExpansionListener ()
        {
            public void treeExpanded ( TreeExpansionEvent event )
            {
                tree.repaint ();
            }

            public void treeCollapsed ( TreeExpansionEvent event )
            {
                tree.repaint ();
            }
        };
        tree.addTreeExpansionListener ( treeExpansionListener );

        mouseoverAdapter = new MouseAdapter ()
        {
            public void mousePressed ( MouseEvent e )
            {
                boolean multiselection = isMultiselectorModeAvailable ();
                boolean hitCell = getRowForPoint ( e.getPoint (), false ) != -1;
                if ( multiselection && SwingUtilities.isLeftMouseButton ( e ) && !hitCell )
                {
                    // Selection
                    selectionStart = e.getPoint ();
                    selectionEnd = selectionStart;

                    // Initial tree selection
                    initialSelection = getSelectionRowsList ();

                    // Updating selection
                    validateSelection ( e );

                    tree.repaint ();
                }
            }

            public void mouseDragged ( MouseEvent e )
            {
                if ( isMultiselectorModeAvailable () && selectionStart != null )
                {
                    // Selection
                    selectionEnd = e.getPoint ();

                    // Updating selection
                    validateSelection ( e );

                    if ( !tree.getVisibleRect ().contains ( e.getPoint () ) )
                    {
                        tree.scrollRectToVisible (
                                new Rectangle ( e.getPoint (), new Dimension ( 0, 0 ) ) );
                    }
                    tree.repaint ();
                }
            }

            private void validateSelection ( MouseEvent e )
            {
                Rectangle selection =
                        GeometryUtils.getContainingRect ( selectionStart, selectionEnd );
                validateSelection ( e, selection );
            }

            private void validateSelection ( MouseEvent e, Rectangle selection )
            {
                // Compute new selection
                List<Integer> newSelection = new ArrayList<Integer> ();
                if ( SwingUtils.isShift ( e ) )
                {
                    for ( int row = 0; row < tree.getRowCount (); row++ )
                    {
                        if ( getRowBounds ( row ).intersects ( selection ) &&
                                !initialSelection.contains ( row ) )
                        {
                            newSelection.add ( row );
                        }
                    }
                    for ( int row : initialSelection )
                    {
                        newSelection.add ( row );
                    }
                }
                else if ( SwingUtils.isCtrl ( e ) )
                {
                    List<Integer> excludedRows = new ArrayList<Integer> ();
                    for ( int row = 0; row < tree.getRowCount (); row++ )
                    {
                        if ( getRowBounds ( row ).intersects ( selection ) )
                        {
                            if ( initialSelection.contains ( row ) )
                            {
                                excludedRows.add ( row );
                            }
                            else
                            {
                                newSelection.add ( row );
                            }
                        }
                    }
                    for ( int row : initialSelection )
                    {
                        if ( !excludedRows.contains ( row ) )
                        {
                            newSelection.add ( row );
                        }
                    }
                }
                else
                {
                    for ( int row = 0; row < tree.getRowCount (); row++ )
                    {
                        if ( getRowBounds ( row ).intersects ( selection ) )
                        {
                            newSelection.add ( row );
                        }
                    }
                }

                // Change selection if it is not the same as before
                if ( !CollectionUtils.areEqual ( getSelectionRowsList (), newSelection ) )
                {
                    if ( newSelection.size () > 0 )
                    {
                        tree.setSelectionRows ( CollectionUtils.toArray ( newSelection ) );
                    }
                    else
                    {
                        tree.clearSelection ();
                    }
                }
            }

            private List<Integer> getSelectionRowsList ()
            {
                List<Integer> selection = new ArrayList<Integer> ();
                final int[] selectionRows = tree.getSelectionRows ();
                if ( selectionRows != null )
                {
                    for ( int row : selectionRows )
                    {
                        selection.add ( row );
                    }
                }
                return selection;
            }

            public void mouseReleased ( MouseEvent e )
            {
                if ( isMultiselectorModeAvailable () && selectionStart != null )
                {
                    // Selection
                    selectionStart = null;
                    selectionEnd = null;

                    tree.repaint ();
                }
            }

            public void mouseEntered ( MouseEvent e )
            {
                if ( showMouseoverHighlight && selectionStart == null )
                {
                    updateMouseover ( e );
                }
            }

            public void mouseExited ( MouseEvent e )
            {
                if ( showMouseoverHighlight )
                {
                    rolloverRow = -1;
                    tree.repaint ();
                }
            }

            public void mouseMoved ( MouseEvent e )
            {
                if ( showMouseoverHighlight )
                {
                    updateMouseover ( e );
                }
            }

            private void updateMouseover ( MouseEvent e )
            {
                int index = getRowForPoint ( e.getPoint () );
                if ( rolloverRow != index )
                {
                    int oldRollover = rolloverRow;
                    rolloverRow = index;
                    if ( index != -1 )
                    {
                        tree.repaint ( getFullRowBounds ( index ) );
                    }
                    if ( oldRollover != -1 )
                    {
                        tree.repaint ( getFullRowBounds ( oldRollover ) );
                    }
                }
            }
        };
        if ( showMouseoverHighlight )
        {
            tree.addMouseListener ( mouseoverAdapter );
            tree.addMouseMotionListener ( mouseoverAdapter );
        }
    }

    public void uninstallUI ( JComponent c )
    {
        tree.removeTreeSelectionListener ( treeSelectionListener );
        tree.removeTreeExpansionListener ( treeExpansionListener );

        tree.removeMouseListener ( mouseoverAdapter );
        tree.removeMouseMotionListener ( mouseoverAdapter );

        super.uninstallUI ( c );
    }

    public Shape provideShape ()
    {
        return LafUtils.getWebBorderShape ( tree, getShadeWidth (), getRound () );
    }

    public int getRound ()
    {
        return round;
    }

    public void setRound ( int round )
    {
        this.round = round;
    }

    public int getShadeWidth ()
    {
        return shadeWidth;
    }

    public void setShadeWidth ( int shadeWidth )
    {
        this.shadeWidth = shadeWidth;
    }

    public Color getSelectionColor ()
    {
        return selectionColor;
    }

    public void setSelectionColor ( Color selectionColor )
    {
        this.selectionColor = selectionColor;
    }

    public Color getSelectionBorderColor ()
    {
        return selectionBorderColor;
    }

    public void setSelectionBorderColor ( Color selectionBorderColor )
    {
        this.selectionBorderColor = selectionBorderColor;
    }

    public int getSelectionRound ()
    {
        return selectionRound;
    }

    public void setSelectionRound ( int selectionRound )
    {
        this.selectionRound = selectionRound;
    }

    public boolean isShowMouseoverHighlight ()
    {
        return showMouseoverHighlight;
    }

    public void setShowMouseoverHighlight ( boolean showMouseoverHighlight )
    {
        this.showMouseoverHighlight = showMouseoverHighlight;
    }

    public boolean isMultiselectorModeAvailable ()
    {
        return allowMultiselectorMode && tree != null &&
                tree.getSelectionModel ().getSelectionMode () !=
                        TreeSelectionModel.SINGLE_TREE_SELECTION;
    }

    public boolean isAllowMultiselectorMode ()
    {
        return allowMultiselectorMode;
    }

    public void setAllowMultiselectorMode ( boolean allowMultiselectorMode )
    {
        this.allowMultiselectorMode = allowMultiselectorMode;
    }

    public TreePath getClosestPathForLocation ( JTree tree, int x, int y )
    {
        int row = getRowForPoint ( new Point ( x, y ) );
        return row != -1 ? getPathForRow ( tree, row ) : null;
    }

    private int getRowForPoint ( Point point )
    {
        return getRowForPoint ( point, selectionStyle.equals ( TreeSelectionStyle.line ) );
    }

    private int getRowForPoint ( Point point, boolean countFullRow )
    {
        if ( tree != null )
        {
            for ( int row = 0; row < tree.getRowCount (); row++ )
            {
                Rectangle bounds = getRowBounds ( row, countFullRow );
                if ( bounds.contains ( point ) )
                {
                    return row;
                }
            }
        }
        return -1;
    }

    private Rectangle getRowBounds ( int row )
    {
        return getRowBounds ( row, selectionStyle.equals ( TreeSelectionStyle.line ) );
    }

    private Rectangle getRowBounds ( int row, boolean countFullRow )
    {
        return countFullRow ? getFullRowBounds ( row ) : tree.getRowBounds ( row );
    }

    private Rectangle getFullRowBounds ( int index )
    {
        Rectangle b = tree.getRowBounds ( index );
        b.x = 0;
        b.width = tree.getWidth ();
        return b;
    }

    public TreeSelectionStyle getSelectionStyle ()
    {
        return selectionStyle;
    }

    public void setSelectionStyle ( TreeSelectionStyle selectionStyle )
    {
        this.selectionStyle = selectionStyle;
    }

    protected TreeCellEditor createDefaultCellEditor ()
    {
        // Editor size and view fix
        return new DefaultCellEditor ( new WebTextField () )
        {
            public Component getTreeCellEditorComponent ( JTree tree, Object value,
                                                          boolean isSelected, boolean expanded,
                                                          boolean leaf, int row )
            {
                WebTextField editor = ( WebTextField ) super
                        .getTreeCellEditorComponent ( tree, value, isSelected, expanded, leaf,
                                row );

                Component component = tree.getCellRenderer ()
                        .getTreeCellRendererComponent ( tree, value, isSelected, expanded, leaf,
                                row, true );
                editor.setPreferredSize ( component.getPreferredSize () );

                if ( component instanceof JLabel && ( ( JLabel ) component ).getIcon () != null )
                {
                    JLabel label = ( JLabel ) component;
                    editor.setLeadingComponent ( new JLabel ( label.getIcon () ) );
                    editor.setLayoutMargin ( label.getInsets () );
                    editor.setComponentSpacing ( label.getIconTextGap () - 1 );
                }

                return editor;
            }
        };
    }

    protected TreeCellRenderer createDefaultCellRenderer ()
    {
        return new WebTreeCellRenderer ();
    }

    protected void paintHorizontalPartOfLeg ( Graphics g, Rectangle clipBounds, Insets insets,
                                              Rectangle bounds, TreePath path, int row,
                                              boolean isExpanded, boolean hasBeenExpanded,
                                              boolean isLeaf )
    {
        if ( !paintLines )
        {
            return;
        }

        // Don't paint the legs for the root'ish node if the
        int depth = path.getPathCount () - 1;
        if ( ( depth == 0 || ( depth == 1 && !isRootVisible () ) ) && !getShowsRootHandles () )
        {
            return;
        }

        int clipLeft = clipBounds.x;
        int clipRight = clipBounds.x + clipBounds.width;
        int clipTop = clipBounds.y;
        int clipBottom = clipBounds.y + clipBounds.height;
        int lineY = bounds.y + bounds.height / 2;

        if ( tree.getComponentOrientation ().isLeftToRight () )
        {
            int leftX = bounds.x - getRightChildIndent ();
            int nodeX = bounds.x - getHorizontalLegBuffer ();

            if ( lineY >= clipTop && lineY < clipBottom && nodeX >= clipLeft && leftX < clipRight &&
                    leftX < nodeX )
            {

                g.setColor ( linesColor );
                paintHorizontalLine ( g, tree, lineY, leftX, nodeX - 1 );
            }
        }
        else
        {
            int nodeX = bounds.x + bounds.width + getHorizontalLegBuffer ();
            int rightX = bounds.x + bounds.width + getRightChildIndent ();

            if ( lineY >= clipTop && lineY < clipBottom && rightX >= clipLeft &&
                    nodeX < clipRight && nodeX < rightX )
            {

                g.setColor ( linesColor );
                paintHorizontalLine ( g, tree, lineY, nodeX, rightX - 1 );
            }
        }
    }

    protected void paintHorizontalLine ( Graphics g, JComponent c, int y, int left, int right )
    {
        left += 2;
        right += 2;

        left += ( left % 2 );
        for ( int x = left; x <= right; x += 2 )
        {
            g.drawLine ( x, y, x, y );
        }
    }

    protected void paintVerticalLine ( Graphics g, JComponent c, int x, int top, int bottom )
    {
        x += 2;

        top += ( top % 2 );
        for ( int y = top; y <= bottom; y += 2 )
        {
            g.drawLine ( x, y, x, y );
        }
    }

    public void paint ( Graphics g, JComponent c )
    {
        // Cells selection
        if ( tree != null && tree.getSelectionCount () > 0 )
        {
            List<Rectangle> selections = new ArrayList<Rectangle> ();

            Rectangle maxRect = null;
            int lastRow = -1;
            int[] rows = tree.getSelectionRows ();
            Arrays.sort ( rows );
            for ( int row : rows )
            {
                if ( selectionStyle.equals ( TreeSelectionStyle.single ) )
                {
                    // Needed bounds
                    Rectangle b = tree.getRowBounds ( row );
                    b.x += shadeWidth;
                    b.y += shadeWidth;
                    b.width -= shadeWidth * 2;
                    b.height -= shadeWidth * 2;
                    selections.add ( b );
                }
                else
                {
                    if ( lastRow != -1 && lastRow + 1 != row )
                    {
                        // Save determined group
                        selections.add ( maxRect );

                        // Reset counting
                        maxRect = null;
                        lastRow = -1;
                    }
                    if ( lastRow == -1 || lastRow + 1 == row )
                    {
                        // Needed bounds
                        Rectangle b = tree.getRowBounds ( row );
                        if ( selectionStyle.equals ( TreeSelectionStyle.group ) )
                        {
                            b.x += shadeWidth;
                            b.y += shadeWidth;
                            b.width -= shadeWidth * 2 + 1;
                            b.height -= shadeWidth * 2 + 1;
                        }
                        else if ( selectionStyle.equals ( TreeSelectionStyle.line ) )
                        {
                            b.x = shadeWidth;
                            b.y += shadeWidth;
                            b.width = tree.getWidth () - shadeWidth * 2 - 1;
                            b.height -= shadeWidth * 2 + 1;
                        }

                        // Increase rect
                        maxRect =
                                lastRow == -1 ? b : GeometryUtils.getContainingRect ( maxRect, b );

                        // Remember last row
                        lastRow = row;
                    }
                }
            }
            if ( maxRect != null )
            {
                selections.add ( maxRect );
            }

            // Draw final selections
            for ( Rectangle rect : selections )
            {
                LafUtils.drawCustomWebBorder ( ( Graphics2D ) g, tree,
                        new RoundRectangle2D.Double ( rect.x, rect.y, rect.width, rect.height,
                                round * 2, round * 2 ), StyleConstants.shadeColor, shadeWidth, true,
                        true );
            }
        }

        // Rollover cell
        if ( showMouseoverHighlight && rolloverRow != -1 && !tree.isRowSelected ( rolloverRow ) )
        {
            Rectangle rect;
            if ( selectionStyle.equals ( TreeSelectionStyle.line ) )
            {
                rect = getFullRowBounds ( rolloverRow );
            }
            else
            {
                rect = tree.getRowBounds ( rolloverRow );
            }
            rect.x += shadeWidth;
            rect.y += shadeWidth;
            rect.width -= shadeWidth * 2;
            rect.height -= shadeWidth * 2;

            Composite old = LafUtils.setupAlphaComposite ( ( Graphics2D ) g, 0.35f );
            LafUtils.drawCustomWebBorder ( ( Graphics2D ) g, tree,
                    new RoundRectangle2D.Double ( rect.x, rect.y, rect.width, rect.height,
                            round * 2, round * 2 ), StyleConstants.shadeColor, shadeWidth, true,
                    true );
            LafUtils.restoreComposite ( ( Graphics2D ) g, old );
        }

        super.paint ( g, c );

        // Multiselector
        if ( isMultiselectorModeAvailable () && selectionStart != null && selectionEnd != null )
        {
            Graphics2D g2d = ( Graphics2D ) g;
            LafUtils.setupAntialias ( g2d );
            Object aa = LafUtils.setupAntialias ( g2d );
            Stroke os = LafUtils.setupStroke ( g2d, StyleConstants.fieldFocusStroke );

            Rectangle bounds = GeometryUtils.getContainingRect ( selectionStart, selectionEnd );
            Shape selection =
                    new RoundRectangle2D.Double ( bounds.x, bounds.y, bounds.width, bounds.height,
                            selectionRound * 2, selectionRound * 2 );

            g2d.setPaint ( selectionColor );
            g2d.fill ( selection );

            g2d.setPaint ( selectionBorderColor );
            g2d.draw ( selection );

            LafUtils.restoreStroke ( g2d, os );
            LafUtils.restoreAntialias ( g2d, aa );
        }
    }
}

