/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.managers.hotkey;

import com.alee.managers.tooltip.TooltipManager;
import com.alee.managers.tooltip.TooltipWay;
import com.alee.utils.SwingUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.AWTEventListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * User: mgarin Date: 11.07.11 Time: 12:54
 * <p/>
 * This manager allows you to quickly register global hotkeys (like accelerators on menu items in
 * menubar menus) for any Swing component. Additionally you can specify a component which will limit
 * hotkey events to its area (meaning that hotkey event will occur only if this component or any of
 * its childs is focused when hotkey pressed). Also TooltipManager is integrated with this manager
 * to automatically show component hotkeys in its tooltip if needed.
 */

public class HotkeyManager
{
    // Global hotkeys block flag
    private static boolean hotkeysEnabled = true;

    // Added hotkeys
    private static List<HotkeyInfo> hotkeysInfo = new ArrayList<HotkeyInfo> ();

    // Pass focus to fired hotkey component
    private static boolean transferFocus = false;

    // Component hotkeys tooltip display way
    private static Map<Component, TooltipWay> componentsHotkeyDisplayWay =
            new HashMap<Component, TooltipWay> ();

    /*
    * HotkeyManager initialization
    */

    public static void initializeManager ()
    {
        // AWT hotkeys listener
        Toolkit.getDefaultToolkit ().addAWTEventListener ( new AWTEventListener ()
        {
            public void eventDispatched ( AWTEvent event )
            {
                // Only if hotkeys enabled and we recieved a KeyEvent
                if ( hotkeysEnabled && event instanceof KeyEvent )
                {
                    KeyEvent e = ( KeyEvent ) event;

                    // Ignore consumed and non-press events
                    if ( e.isConsumed () || e.getID () != KeyEvent.KEY_PRESSED )
                    {
                        return;
                    }

                    // Processing all added hotkeys
                    List<HotkeyInfo> clonedHotkeysInfo = new ArrayList<HotkeyInfo> ();
                    clonedHotkeysInfo.addAll ( hotkeysInfo );
                    for ( HotkeyInfo hi : clonedHotkeysInfo )
                    {
                        // Checking if componen or one of its childs has focus
                        Component topComponent =
                                hi.getTopComponent () != null ? hi.getTopComponent () :
                                        SwingUtilities.getWindowAncestor ( hi.getForComponent () );
                        if ( SwingUtils.hasFocusOwner ( topComponent ) )
                        {
                            // Checking hotkey
                            if ( hi.getHotkeyData ().isHotkeyTriggered ( e ) )
                            {
                                // Performing hotkey action
                                SwingUtils.invokeLater ( hi.getAction (), e );
                            }
                        }
                    }
                }
            }
        }, AWTEvent.KEY_EVENT_MASK );
    }

    /*
    * Hotkey register methods
    */

    public static void registerHotkey ( Component forComponent, HotkeyData hotkeyData,
                                        HotkeyRunnable action )
    {
        registerHotkey ( null, forComponent, hotkeyData, action );
    }

    public static void registerHotkey ( Component forComponent, HotkeyData hotkeyData,
                                        HotkeyRunnable action, boolean hidden )
    {
        registerHotkey ( null, forComponent, hotkeyData, action, hidden );
    }

    public static void registerHotkey ( Component topComponent, Component forComponent,
                                        HotkeyData hotkeyData, HotkeyRunnable action )
    {
        registerHotkey ( topComponent, forComponent, hotkeyData, action, false );
    }

    public static void registerHotkey ( Component topComponent, Component forComponent,
                                        HotkeyData hotkeyData, HotkeyRunnable action,
                                        TooltipWay tooltipWay )
    {
        registerHotkey ( topComponent, forComponent, hotkeyData, action, false, tooltipWay );
    }

    public static void registerHotkey ( Component topComponent, Component forComponent,
                                        HotkeyData hotkeyData, HotkeyRunnable action,
                                        boolean hidden )
    {
        registerHotkey ( topComponent, forComponent, hotkeyData, action, hidden, null );
    }

    public static void registerHotkey ( Component topComponent, Component forComponent,
                                        HotkeyData hotkeyData, HotkeyRunnable action,
                                        boolean hidden, TooltipWay tooltipWay )
    {
        HotkeyInfo hi = new HotkeyInfo ();
        hi.setHidden ( hidden );
        hi.setTopComponent ( topComponent );
        hi.setForComponent ( forComponent );
        hi.setHotkeyData ( hotkeyData );
        hi.setAction ( action );
        hotkeysInfo.add ( hi );

        setComponentHotkeyDisplayWay ( forComponent, tooltipWay );
    }

    public static void registerHotkey ( AbstractButton forComponent, HotkeyData hotkeyData )
    {
        registerHotkey ( null, forComponent, hotkeyData );
    }

    public static void registerHotkey ( AbstractButton forComponent, HotkeyData hotkeyData,
                                        boolean hidden )
    {
        registerHotkey ( null, forComponent, hotkeyData, hidden );
    }

    public static void registerHotkey ( Component topComponent, AbstractButton forComponent,
                                        HotkeyData hotkeyData )
    {
        registerHotkey ( topComponent, forComponent, hotkeyData, false );
    }

    public static void registerHotkey ( Component topComponent, final AbstractButton forComponent,
                                        HotkeyData hotkeyData, TooltipWay tooltipWay )
    {
        registerHotkey ( topComponent, forComponent, hotkeyData, false, tooltipWay );
    }

    public static void registerHotkey ( Component topComponent, final AbstractButton forComponent,
                                        HotkeyData hotkeyData, boolean hidden )
    {
        registerHotkey ( topComponent, forComponent, hotkeyData, hidden, null );
    }

    public static void registerHotkey ( Component topComponent, final AbstractButton forComponent,
                                        HotkeyData hotkeyData, boolean hidden,
                                        TooltipWay tooltipWay )
    {
        HotkeyInfo hi = new HotkeyInfo ();
        hi.setHidden ( hidden );
        hi.setTopComponent ( topComponent );
        hi.setForComponent ( forComponent );
        hi.setHotkeyData ( hotkeyData );
        hi.setAction ( new HotkeyRunnable ()
        {
            public void run ( KeyEvent e )
            {
                forComponent.doClick ( 0 );
            }
        } );
        hotkeysInfo.add ( hi );

        setComponentHotkeyDisplayWay ( forComponent, tooltipWay );
    }

    /*
    * Sets component hotkey tip way
    */

    public static void setComponentHotkeyDisplayWay ( Component component, TooltipWay tooltipWay )
    {
        if ( tooltipWay != null )
        {
            componentsHotkeyDisplayWay.put ( component, tooltipWay );
        }
        else
        {
            componentsHotkeyDisplayWay.remove ( component );
        }
    }

    /*
    * Hotkey removal method
    */

    public static void unregisterHotkey ( HotkeyInfo hotkeyInfo )
    {
        hotkeysInfo.remove ( hotkeyInfo );
    }

    /*
    * Shows all visible components hotkeys
    */

    public static void showComponentsHotkeys ()
    {
        TooltipManager.hideAllTooltips ();
        for ( Window window : Window.getWindows () )
        {
            showComponentsHotkeys ( window );
        }
    }

    public static void showComponentsHotkeys ( Component component )
    {
        TooltipManager.hideAllTooltips ();
        showComponentsHotkeys ( SwingUtilities.getWindowAncestor ( component ) );
    }

    private static void showComponentsHotkeys ( Window window )
    {
        List<Component> shown = new ArrayList<Component> ();
        for ( HotkeyInfo hi : HotkeyManager.getAllHotkeys () )
        {
            if ( !hi.isHidden () )
            {
                Component c = hi.getForComponent ();
                if ( c != null && !shown.contains ( c ) && c.isVisible () && c.isShowing () &&
                        SwingUtilities.getWindowAncestor ( c ) == window )
                {
                    TooltipManager.showOneTimeTooltip ( c, null,
                            new JLabel ( HotkeyManager.getComponentHotkeysString ( c ) )
                            {
                                {
                                    setFont ( getFont ().deriveFont ( Font.BOLD ) );
                                }
                            }, componentsHotkeyDisplayWay.get ( c ) );
                    shown.add ( c );
                }
            }
        }
    }

    /*
    * Installs "show all hotkeys" action on window or component
    */

    public static void installShowAllHotkeysAction ( JDialog dialog )
    {
        installShowAllHotkeysAction ( dialog.getRootPane (), Hotkey.F1 );
    }

    public static void installShowAllHotkeysAction ( JDialog dialog, HotkeyData hotkeyData )
    {
        installShowAllHotkeysAction ( dialog.getRootPane (), hotkeyData );
    }

    public static void installShowAllHotkeysAction ( JFrame frame, HotkeyData hotkeyData )
    {
        installShowAllHotkeysAction ( frame.getRootPane (), hotkeyData );
    }

    public static void installShowAllHotkeysAction ( JWindow window, HotkeyData hotkeyData )
    {
        installShowAllHotkeysAction ( window.getRootPane (), hotkeyData );
    }

    public static void installShowAllHotkeysAction ( final Component topComponent,
                                                     HotkeyData hotkeyData )
    {
        HotkeyManager.registerHotkey ( topComponent, topComponent, hotkeyData, new HotkeyRunnable ()
        {
            public void run ( KeyEvent e )
            {
                HotkeyManager.showComponentsHotkeys ( topComponent );
            }
        }, true );
    }

    /*
    * Global hotkey block
    */

    public static void disableHotkeys ()
    {
        hotkeysEnabled = false;
    }

    public static void enableHotkeys ()
    {
        hotkeysEnabled = true;
    }

    /*
    * All added component hotkeys
    */

    public static List<HotkeyInfo> getAllHotkeys ()
    {
        return hotkeysInfo;
    }

    /*
    * All component hotkeys list
    */

    public static List<HotkeyInfo> getComponentHotkeys ( Component component )
    {
        List<HotkeyInfo> infoList = new ArrayList<HotkeyInfo> ();
        List<HotkeyInfo> clonedHotkeysInfo = new ArrayList<HotkeyInfo> ();
        clonedHotkeysInfo.addAll ( hotkeysInfo );
        for ( final HotkeyInfo hi : clonedHotkeysInfo )
        {
            if ( hi.getForComponent () == component )
            {
                infoList.add ( hi );
            }
        }
        return infoList;
    }

    public static String getComponentHotkeysString ( Component component )
    {
        return getComponentHotkeysString ( getComponentHotkeys ( component ) );
    }

    private static String getComponentHotkeysString ( List<HotkeyInfo> infoList )
    {
        String hotkeys = "";
        for ( HotkeyInfo hi : infoList )
        {
            if ( !hi.isHidden () )
            {
                hotkeys += hi.getHotkeyData ().toString ();
                if ( infoList.indexOf ( hi ) < infoList.size () - 1 )
                {
                    hotkeys += ", ";
                }
            }
        }
        return hotkeys;
    }

    /*
    * Should transfer focus to fired hotkey component or not
    */

    public static boolean isTransferFocus ()
    {
        return transferFocus;
    }

    public static void setTransferFocus ( boolean transferFocus )
    {
        HotkeyManager.transferFocus = transferFocus;
    }
}
