/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.managers.proxy;

import com.alee.managers.settings.SettingsManager;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethodBase;
import org.apache.commons.httpclient.ProxyHost;
import sun.misc.BASE64Encoder;

import java.io.IOException;
import java.net.*;
import java.util.Properties;

/**
 * User: mgarin Date: 12.10.11 Time: 17:55
 * <p/>
 * This manager allows you to quickly manage proxy settings across your application to connect
 * through any proxy server (optionally with authentification)
 */

public class ProxyManager
{
    public static ProxySettings defaultSettings = null;

    private static boolean initialized = false;

    public static void initializeManager ()
    {
        if ( !initialized )
        {
            initialized = true;

            // ProxySettings class alias
            SettingsManager.alias ( "ProxySettings", ProxySettings.class );

            // Default proxy settings
            defaultSettings = new ProxySettings ();
            //            initSystemProxySettings ();
            //            defaultSettings = getSystemProxySettings ();
        }
    }

    /**
     * Last saved ProxySettings (or default if none were saved)
     */

    public static ProxySettings getProxySettings ()
    {
        return ( ProxySettings ) SettingsManager
                .get ( "ProxyManager", "ProxySettings", defaultSettings );
    }

    /**
     * Proxy setup method
     */

    private static boolean proxySet = false;

    public static void setProxySettings ()
    {
        if ( !proxySet )
        {
            setProxySettings ( getProxySettings () );
        }
    }

    public static void setEmptyProxySettings ()
    {
        setProxySettings ( new ProxySettings () );
    }

    public static void setProxySettings ( ProxySettings proxySettings )
    {
        proxySet = true;

        // Saving new ProxySettings
        SettingsManager.set ( "ProxyManager", "ProxySettings", proxySettings );

        // System properties
        Properties systemSettings = System.getProperties ();

        // Use proxy or not
        systemSettings.setProperty ( "proxySet", "" + proxySettings.isUseProxy () );

        // Proxy settings
        if ( proxySettings.isUseProxy () )
        {
            systemSettings.setProperty ( "proxyHost", "" + proxySettings.getProxyHost () );
            systemSettings.setProperty ( "proxyPort", "" + proxySettings.getProxyPort () );
            systemSettings.setProperty ( "nonProxyHosts", "" + proxySettings.getNonProxyHosts () );

            // Proxy authentification
            Authenticator.setDefault (
                    proxySettings.isUseProxyAuthentification () ? new WebProxyAuthenticator () :
                            null );
        }
        else
        {
            systemSettings.setProperty ( "proxyHost", "" );
            systemSettings.setProperty ( "proxyPort", "" );
            systemSettings.setProperty ( "nonProxyHosts", "" );

            // Proxy authentification
            Authenticator.setDefault ( null );
        }
    }

    public static void setProxySettings ( URLConnection urlConnection )
    {
        // Recheck that proxy is set
        setProxySettings ();

        // Proxy settings
        ProxySettings proxySettings = getProxySettings ();
        if ( proxySettings.isUseProxy () && proxySettings.isUseProxyAuthentification () )
        {
            BASE64Encoder encoder = new BASE64Encoder ();
            String encoded = encoder.encode (
                    ( proxySettings.getProxyLogin () + ":" + proxySettings.getProxyPassword () )
                            .getBytes () );
            urlConnection.setRequestProperty ( "Proxy-Authorization", "Basic " + encoded );
        }
    }

    public static void setProxySettings ( HttpClient httpClient, HttpMethodBase method )
    {
        // Recheck that proxy is set
        setProxySettings ();

        // Proxy settings
        ProxySettings ps = ProxyManager.getProxySettings ();
        if ( ps.isUseProxy () )
        {
            httpClient.getHostConfiguration ().setProxyHost (
                    new ProxyHost ( ps.getProxyHost (), Integer.parseInt ( ps.getProxyPort () ) ) );

            // Proxy auth
            if ( ps.isUseProxyAuthentification () )
            {
                BASE64Encoder encoder = new BASE64Encoder ();
                String encoded = encoder.encode (
                        ( ps.getProxyLogin () + ":" + ps.getProxyPassword () ).getBytes () );
                method.setRequestHeader ( "Proxy-Authorization", "Basic " + encoded );
            }
        }
    }

    /**
     * Open url connection with current proxy settings
     */

    public static URLConnection getURLConnection ( URL url ) throws IOException
    {
        // Recheck that proxy is set
        setProxySettings ();

        // Open connection properly
        ProxySettings proxySettings = getProxySettings ();
        if ( proxySettings.isUseProxy () )
        {
            URLConnection connection = url.openConnection ( new Proxy ( Proxy.Type.HTTP,
                    new InetSocketAddress ( proxySettings.getProxyHost (),
                            proxySettings.getProxyPortInt () ) ) );
            setProxySettings ( connection );
            return connection;
        }
        else
        {
            // Simple connection without proxy
            return url.openConnection ();
        }
    }

    /**
     * System proxy settings
     * Doesn't work properly yet
     */

    //    public static boolean initSystemProxySettings ()
    //    {
    //        // Initialization code was taken from com.sun.deploy.ClientContainer:
    //        ServiceManager.setService (
    //                System.getProperty ( "os.name" ).toLowerCase ().contains ( "windows" ) ?
    //                        PlatformType.STANDALONE_TIGER_WIN32 : PlatformType.STANDALONE_TIGER_UNIX );
    //
    //        try
    //        {
    //            // This will call ProxySelector.setDefault():
    //            DeployProxySelector.reset ();
    //        }
    //        catch ( Throwable throwable )
    //        {
    //            return false;
    //        }
    //
    //        return true;
    //    }
    //
    //    public static ProxySettings getSystemProxySettings ()
    //    {
    //        ProxySettings proxySettings = new ProxySettings ();
    //        System.setProperty ( "java.net.useSystemProxies", "true" );
    //        Proxy proxy = getSystemHttpProxy ();
    //        if ( proxy != null )
    //        {
    //            InetSocketAddress addr = ( InetSocketAddress ) proxy.address ();
    //            if ( addr != null && addr.getHostName () != null )
    //            {
    //                proxySettings.setUseProxy ( true );
    //                proxySettings.setProxyHost ( addr.getHostName () );
    //                proxySettings.setProxyPort ( "" + addr.getPort () );
    //            }
    //        }
    //        System.setProperty ( "java.net.useSystemProxies", "false" );
    //        return proxySettings;
    //    }
    //
    //    private static Proxy getSystemHttpProxy ()
    //    {
    //        List<Proxy> l = null;
    //        try
    //        {
    //            ProxySelector def = ProxySelector.getDefault ();
    //
    //            l = def.select ( new URI ( "http://foo/bar" ) );
    //            ProxySelector.setDefault ( null );
    //        }
    //        catch ( Exception e )
    //        {
    //            //
    //        }
    //        if ( l != null )
    //        {
    //            //noinspection LoopStatementThatDoesntLoop
    //            for ( Proxy proxy : l )
    //            {
    //                if ( proxy != null && proxy.type ().equals ( Proxy.Type.HTTP ) )
    //                {
    //                    return proxy;
    //                }
    //            }
    //        }
    //        return null;
    //    }
}
