/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.managers.settings;

import com.alee.utils.FileUtils;
import com.thoughtworks.xstream.XStream;

import java.awt.*;
import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * User: mgarin Date: 12.10.11 Time: 17:53
 * <p/>
 * This manager allows you to quickly and easily save any serializable settings you want
 */

public class SettingsManager
{
    public static final String SETTINGS_DIR = ".weblaf";
    public static final String SETTINGS_FORMAT = ".xml";
    public static final String DUMP_FORMAT = ".dump";
    public static final String DEFAULT_GROUP_NAME = "default";

    private static Map<String, SettingsGroup> groups = new HashMap<String, SettingsGroup> ();

    private static boolean saveOnChange = true;

    private static boolean initialized = false;

    public static void initializeManager ()
    {
        if ( !initialized )
        {
            initialized = true;

            // Initializing XStream first
            getXStream ();

            // Getting settings folder content
            File settingsDir = new File ( getSettingsDir () );
            if ( !settingsDir.exists () )
            {
                settingsDir.mkdir ();
            }

            // If there are any settings files
            File[] files = settingsDir.listFiles ();
            if ( files != null )
            {
                // Checking for dump files
                // If any dump file presented it will replace actual settings file with same name
                for ( File file : files )
                {
                    // Check if it is file
                    if ( file.isFile () )
                    {
                        // Check if it is a dump file
                        String name = file.getName ();
                        if ( name.endsWith ( SETTINGS_FORMAT + DUMP_FORMAT ) )
                        {
                            // Replacing file with dump
                            FileUtils.copyFile ( file, new File ( file.getParent (),
                                    name.substring ( 0, name.lastIndexOf ( DUMP_FORMAT ) ) ) );

                            // Removing dump file
                            FileUtils.removeFile ( file );
                        }
                    }
                }

                // Pre-reading of groups replaced with runtime groups parsing due to aliases
                //                // Reading all available SettingsGroup
                //                files = settingsDir.listFiles ();
                //                for ( File file : files )
                //                {
                //                    // Check if it is file
                //                    if ( file.isFile () && file.getName ().endsWith ( SETTINGS_FORMAT ) )
                //                    {
                //                        try
                //                        {
                //                            // Read single SettingsGroup
                //                            // SettingsGroup group = ( SettingsGroup ) getXStream ().fromXML ( file );
                //                            groups.put ( FileUtils.getFileNamePart ( file ), null );
                //                        }
                //                        catch ( Throwable e )
                //                        {
                //                            // Delete incorrect settings file
                //                            FileUtils.removeFile ( file );
                //                        }
                //                    }
                //                }
            }

            // Save settings on JVM shutdown
            Runtime.getRuntime ().addShutdownHook ( new Thread ( new Runnable ()
            {
                public void run ()
                {
                    // Saving each group 1 by 1
                    for ( SettingsGroup settingsGroup : groups.values () )
                    {
                        saveSettingsGroup ( settingsGroup );
                    }
                }
            } ) );
        }
    }

    private static void saveSettingsGroup ( SettingsGroup settingsGroup )
    {
        try
        {
            // Settings file
            File file = new File ( getSettingsDir (), settingsGroup.getName () + SETTINGS_FORMAT );

            // Creating settings dump if there are old settings
            File dumpFile = null;
            if ( file.exists () )
            {
                dumpFile = new File ( getSettingsDir (),
                        settingsGroup.getName () + SETTINGS_FORMAT + DUMP_FORMAT );
                FileUtils.copyFile ( file, dumpFile );
            }

            // Saving settings
            getXStream ().toXML ( settingsGroup, new FileOutputStream ( file ) );

            // Clearing dump file
            if ( dumpFile != null )
            {
                FileUtils.removeFile ( dumpFile );
            }
        }
        catch ( Throwable e )
        {
            //
        }
    }

    /**
     * All available groups
     */

    public static List<SettingsGroup> getAllGroups ()
    {
        return new ArrayList<SettingsGroup> ( groups.values () );
    }

    public static Map<String, SettingsGroup> getAllGroupsMap ()
    {
        return groups;
    }

    /**
     * Standart values getters
     */

    public static Boolean getBoolean ( Object key )
    {
        return ( Boolean ) get ( key );
    }

    public static Boolean getBoolean ( Object key, Object defaultValue )
    {
        return ( Boolean ) get ( key, defaultValue );
    }

    public static Boolean getBoolean ( String group, Object key, Object defaultValue )
    {
        return ( Boolean ) get ( group, key, defaultValue );
    }

    public static String getString ( Object key )
    {
        return ( String ) get ( key );
    }

    public static String getString ( Object key, Object defaultValue )
    {
        return ( String ) get ( key, defaultValue );
    }

    public static String getString ( String group, Object key, Object defaultValue )
    {
        return ( String ) get ( group, key, defaultValue );
    }

    public static Integer getInteger ( Object key )
    {
        return ( Integer ) get ( key );
    }

    public static Integer getInteger ( Object key, Object defaultValue )
    {
        return ( Integer ) get ( key, defaultValue );
    }

    public static Integer getInteger ( String group, Object key, Object defaultValue )
    {
        return ( Integer ) get ( group, key, defaultValue );
    }

    public static Float getFloat ( Object key )
    {
        return ( Float ) get ( key );
    }

    public static Float getFloat ( Object key, Object defaultValue )
    {
        return ( Float ) get ( key, defaultValue );
    }

    public static Float getFloat ( String group, Object key, Object defaultValue )
    {
        return ( Float ) get ( group, key, defaultValue );
    }

    public static Dimension getDimension ( Object key )
    {
        return ( Dimension ) get ( key );
    }

    public static Dimension getDimension ( Object key, Object defaultValue )
    {
        return ( Dimension ) get ( key, defaultValue );
    }

    public static Dimension getDimension ( String group, Object key, Object defaultValue )
    {
        return ( Dimension ) get ( group, key, defaultValue );
    }

    public static Point getPoint ( Object key )
    {
        return ( Point ) get ( key );
    }

    public static Point getPoint ( Object key, Object defaultValue )
    {
        return ( Point ) get ( key, defaultValue );
    }

    public static Point getPoint ( String group, Object key, Object defaultValue )
    {
        return ( Point ) get ( group, key, defaultValue );
    }

    public static Color getColor ( Object key )
    {
        return ( Color ) get ( key );
    }

    public static Color getColor ( Object key, Object defaultValue )
    {
        return ( Color ) get ( key, defaultValue );
    }

    public static Color getColor ( String group, Object key, Object defaultValue )
    {
        return ( Color ) get ( group, key, defaultValue );
    }

    /**
     * Base settings getters/setters
     */

    public static Object get ( Object key )
    {
        return get ( DEFAULT_GROUP_NAME, key );
    }

    public static Object get ( String group, Object key )
    {
        return get ( group, key, null );
    }

    public static Object get ( Object key, Object defaultValue )
    {
        return get ( DEFAULT_GROUP_NAME, key, defaultValue );
    }

    public static Object get ( String group, Object key, Object defaultValue )
    {
        // Check manager initialization
        initializeManager ();

        // Get SettingsGroup safely
        SettingsGroup settingsGroup = getSettingsGroup ( group );

        // Get value
        Object value = settingsGroup.get ( key != null ? key.toString () : null );

        // Returning default value if no actual found
        return value == null && defaultValue != null ? defaultValue : value;
    }

    public static Object set ( Object key, Object object )
    {
        return set ( DEFAULT_GROUP_NAME, key, object );
    }

    public static Object set ( String group, Object key, Object object )
    {
        // Check manager initialization
        initializeManager ();

        // Get SettingsGroup safely
        SettingsGroup settingsGroup = getSettingsGroup ( group );

        // Put new value
        Object oldValue = settingsGroup.put ( key != null ? key.toString () : null, object );

        // Save group if needed
        if ( saveOnChange )
        {
            saveSettingsGroup ( settingsGroup );
        }

        return oldValue;
    }

    /**
     * Settings get/set methods
     */

    public static SettingsGroup getSettingsGroup ( String group )
    {
        if ( groups.containsKey ( group ) )
        {
            return groups.get ( group );
        }
        else
        {
            SettingsGroup settingsGroup = null;

            // Try reading SettingsGroup
            File file = new File ( getSettingsDir (), group + SETTINGS_FORMAT );
            if ( file.exists () && file.isFile () )
            {
                try
                {
                    // Read single SettingsGroup
                    settingsGroup = ( SettingsGroup ) getXStream ().fromXML ( file );
                }
                catch ( Throwable e )
                {
                    // Delete incorrect SettingsGroup file
                    FileUtils.removeFile ( file );
                }
            }

            // Create new SettingsGroup
            if ( settingsGroup == null )
            {
                settingsGroup = new SettingsGroup ( group );
            }

            groups.put ( group, settingsGroup );
            return settingsGroup;
        }
    }

    /**
     * Default settings save directory
     */

    public static String getSettingsDir ()
    {
        return FileUtils.getUserHome () + SETTINGS_DIR;
    }

    /**
     * Save SettingsGroup with changes right away
     */

    public static boolean isSaveOnChange ()
    {
        return saveOnChange;
    }

    public static void setSaveOnChange ( boolean saveOnChange )
    {
        SettingsManager.saveOnChange = saveOnChange;
    }

    /**
     * XStream xml serializer/deserializer instance
     */

    private static XStream xStream = null;

    public static XStream getXStream ()
    {
        if ( xStream == null )
        {
            initializeXStream ();
        }
        return xStream;
    }

    private static void initializeXStream ()
    {
        try
        {
            xStream = new XStream ();

            // SettingsManager aliases
            xStream.alias ( "SettingsGroup", SettingsGroup.class );

            // Standart Java-classes aliases
            xStream.alias ( "Point", Point.class );
            xStream.alias ( "Dimension", Dimension.class );
            xStream.alias ( "Font", Font.class );
            xStream.alias ( "Color", Color.class );
        }
        catch ( Throwable e )
        {
            e.printStackTrace ();
        }
    }

    /**
     * Additional aliases adding
     */

    public static void alias ( String name, Class type )
    {
        getXStream ().alias ( name, type );
    }
}
