/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.managers.tooltip;

import com.alee.managers.glasspane.GlasspaneManager;
import com.alee.managers.glasspane.WebGlassPane;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * User: mgarin Date: 05.10.11 Time: 17:59
 * <p/>
 * This manager allows you to set extended tooltips for any Swing component with any possible
 * content (would it be simple text or some JComponent ancestor) or show one-time tooltips at custom
 * location inside any window. Also this manager is integrated with HotkeyManager and provides an
 * opportunity to automatically show components hotkeys on their tooltips set with this class.
 */

public class TooltipManager
{
    // Default settings
    private static int defaultDelay = 500;
    private static boolean allowManyTooltips = true;
    private static boolean showHotkeysInTooltips = true;
    private static boolean showHotkeysInOneTimeTooltips = false;

    // Standart tooltips
    private static Map<Component, List<WebCustomTooltip>> webTooltips =
            new HashMap<Component, List<WebCustomTooltip>> ();
    private static Map<Component, MouseAdapter> adapters = new HashMap<Component, MouseAdapter> ();
    private static Map<Component, Timer> timers = new HashMap<Component, Timer> ();

    // One-time tooltips
    private static List<WebCustomTooltip> oneTimeTooltips = new ArrayList<WebCustomTooltip> ();

    /*
    * TooltipManager initialization
    */

    public static void initializeManager ()
    {
        // Tooltips hide listener
        AWTEventListener listener = new AWTEventListener ()
        {
            public void eventDispatched ( AWTEvent event )
            {
                //                if ( event instanceof MouseEvent )
                //                {
                //                    hideAllTooltips ();
                //                }
                //                else
                //                if ( event instanceof KeyEvent && event.getID () == KeyEvent.KEY_PRESSED )
                //                {
                //                    hideAllTooltips ();
                //                }
                //                else
                if ( event instanceof MouseWheelEvent )
                {
                    hideAllTooltips ();
                }
            }
        };

        Toolkit toolkit = Toolkit.getDefaultToolkit ();
        toolkit.addAWTEventListener ( listener, AWTEvent.MOUSE_WHEEL_EVENT_MASK );
        //        toolkit.addAWTEventListener ( listener, AWTEvent.KEY_EVENT_MASK );
        //        toolkit.addAWTEventListener ( listener, AWTEvent.MOUSE_EVENT_MASK );
    }

    public static void hideAllTooltips ()
    {
        // Hiding standart tooltips
        for ( Component component : webTooltips.keySet () )
        {
            timers.get ( component ).stop ();
            for ( WebCustomTooltip tooltip : webTooltips.get ( component ) )
            {
                tooltip.closeTooltip ();
            }
        }

        // Hiding one-time tooltips
        for ( WebCustomTooltip tooltip : oneTimeTooltips )
        {
            tooltip.closeTooltip ();
        }
    }

    /*
    * Registers standart tooltip
    */

    public static WebCustomTooltip setTooltip ( Component component, String tooltip )
    {
        return setTooltip ( component, tooltip, null );
    }

    public static WebCustomTooltip setTooltip ( Component component, String tooltip,
                                                TooltipWay tooltipWay )
    {
        return setTooltip ( component, tooltip, tooltipWay, defaultDelay );
    }

    public static WebCustomTooltip setTooltip ( Component component, String tooltip,
                                                TooltipWay tooltipWay, int delay )
    {
        return setTooltip ( component, new JLabel ( tooltip )
        {
            {
                setOpaque ( false );
            }
        }, tooltipWay, delay );
    }

    public static WebCustomTooltip setTooltip ( Component component, JComponent tooltip )
    {
        return setTooltip ( component, tooltip, null );
    }

    public static WebCustomTooltip setTooltip ( Component component, JComponent tooltip, int delay )
    {
        return setTooltip ( component, tooltip, null, delay );
    }

    public static WebCustomTooltip setTooltip ( Component component, JComponent tooltip,
                                                TooltipWay tooltipWay )
    {
        return setTooltip ( component, tooltip, tooltipWay, defaultDelay );
    }

    public static WebCustomTooltip setTooltip ( final Component component, final JComponent tooltip,
                                                final TooltipWay tooltipWay, final int delay )
    {
        // Pre-checks
        if ( webTooltips.get ( component ) == null )
        {
            // Create tooltips list
            webTooltips.put ( component, new ArrayList<WebCustomTooltip> () );
        }
        else if ( webTooltips.get ( component ).size () > 0 && !allowManyTooltips )
        {
            // Erase old tooltip if more than one not allowed
            removeTooltips ( component );
        }

        // Creating tooltip component
        final WebCustomTooltip customTooltip =
                new WebCustomTooltip ( component, tooltip, tooltipWay, showHotkeysInTooltips );
        webTooltips.get ( component ).add ( customTooltip );

        // Creating listeners for component if they aren't created yet
        if ( !timers.containsKey ( component ) || !adapters.containsKey ( component ) )
        {
            // Tooltip pop timer
            final Timer showTips = new Timer ( delay, new ActionListener ()
            {
                public void actionPerformed ( ActionEvent e )
                {
                    Window wa = SwingUtilities.getWindowAncestor ( component );
                    if ( wa != null && wa.isActive () )
                    {
                        showTooltips ( component, false );
                    }
                }
            } );
            showTips.setRepeats ( false );
            timers.put ( component, showTips );

            // Show/hide listener
            MouseAdapter mouseAdapter = new MouseAdapter ()
            {
                public void mouseEntered ( MouseEvent e )
                {
                    // Component ancestor window
                    Window window = SwingUtilities.getWindowAncestor ( component );

                    // Starting show timer if needed
                    if ( window.isShowing () && window.isActive () )
                    {
                        showTips.start ();
                    }
                }

                public void mouseExited ( MouseEvent e )
                {
                    // Hiding component tooltips
                    showTips.stop ();
                    hideTooltips ( component );
                }
            };
            component.addMouseListener ( mouseAdapter );
            adapters.put ( component, mouseAdapter );
        }

        return customTooltip;
    }

    public static boolean showTooltips ( Component component, boolean delayed )
    {
        if ( webTooltips.containsKey ( component ) )
        {
            if ( delayed )
            {
                timers.get ( component ).restart ();
            }
            else
            {
                Window window = SwingUtilities.getWindowAncestor ( component );
                if ( window != null && window.isShowing () && window.isActive () )
                {
                    WebGlassPane webGlassPane = GlasspaneManager.getGlassPane ( window );
                    if ( webGlassPane != null )
                    {
                        for ( WebCustomTooltip tooltip : webTooltips.get ( component ) )
                        {
                            webGlassPane.showComponent ( tooltip );
                        }
                    }
                }
            }
            return true;
        }
        else
        {
            return false;
        }
    }

    private static void hideTooltips ( Component component )
    {
        if ( webTooltips.containsKey ( component ) )
        {
            for ( WebCustomTooltip tooltip : webTooltips.get ( component ) )
            {
                tooltip.closeTooltip ();
            }
        }
    }

    public static void removeTooltips ( Component component )
    {
        for ( WebCustomTooltip tooltip : webTooltips.get ( component ) )
        {
            removeTooltip ( component, tooltip );
        }
    }

    public static void removeTooltip ( Component component, WebCustomTooltip tooltip )
    {
        if ( webTooltips.get ( component ) != null &&
                webTooltips.get ( component ).contains ( tooltip ) )
        {
            // Removing all listeners in case its last component tooltip
            if ( webTooltips.get ( component ).size () <= 1 )
            {
                // Removing mouse listeners
                component.removeMouseListener ( adapters.get ( component ) );
                adapters.remove ( component );

                // Clearing timers
                timers.get ( component ).stop ();
                timers.remove ( component );
            }

            // Removing registered tooltip
            webTooltips.get ( component ).remove ( tooltip );

            // Hiding and destroying tooltip
            tooltip.closeTooltip ();
            tooltip.destroyTooltip ();

            // Removing component from list if its last tooltip removed
            if ( webTooltips.get ( component ).size () == 0 )
            {
                webTooltips.remove ( component );
            }
        }
    }

    /*
    * Shows one-time tooltip
    */

    public static WebCustomTooltip showOneTimeTooltip ( Component component, Point point,
                                                        String tooltip )
    {
        return showOneTimeTooltip ( component, point, tooltip, null );
    }

    public static WebCustomTooltip showOneTimeTooltip ( Component component, Point point,
                                                        String tooltip, TooltipWay tooltipWay )
    {
        WebCustomTooltip customTooltip = new WebCustomTooltip ( component, tooltip, tooltipWay,
                showHotkeysInOneTimeTooltips );
        customTooltip.setDisplayLocation ( point );
        return showOneTimeTooltip ( customTooltip );
    }

    public static WebCustomTooltip showOneTimeTooltip ( Component component, Point point,
                                                        JComponent tooltip )
    {
        return showOneTimeTooltip ( component, point, tooltip, null );
    }

    public static WebCustomTooltip showOneTimeTooltip ( Component component, Point point,
                                                        JComponent tooltip, TooltipWay tooltipWay )
    {
        WebCustomTooltip customTooltip = new WebCustomTooltip ( component, tooltip, tooltipWay,
                showHotkeysInOneTimeTooltips );
        customTooltip.setDisplayLocation ( point );
        return showOneTimeTooltip ( customTooltip );
    }

    private static WebCustomTooltip showOneTimeTooltip ( final WebCustomTooltip customTooltip )
    {
        if ( customTooltip.getComponent () == null || !customTooltip.getComponent ().isShowing () )
        {
            return null;
        }
        Window window = SwingUtilities.getWindowAncestor ( customTooltip.getComponent () );
        if ( window == null )
        {
            return null;
        }
        final WebGlassPane webGlassPane = GlasspaneManager.getGlassPane ( window );
        if ( webGlassPane == null )
        {
            return null;
        }

        // Disable default close dehavior
        customTooltip.setDefaultCloseBehavior ( false );

        // Adding relocate listener
        final ComponentAdapter componentAdapter = new ComponentAdapter ()
        {
            public void componentResized ( ComponentEvent e )
            {
                customTooltip.updateLocation ();
            }
        };
        webGlassPane.addComponentListener ( componentAdapter );

        // Global mouse listener
        final AWTEventListener closeListener = new AWTEventListener ()
        {
            public void eventDispatched ( AWTEvent event )
            {
                if ( event instanceof MouseEvent && event.getID () == MouseEvent.MOUSE_PRESSED )
                {
                    customTooltip.closeTooltip ();
                }
            }
        };

        // Adding destroy sequence
        customTooltip.addGuiTooltipListener ( new TooltipAdapter ()
        {
            public void tooltipFullyShown ()
            {
                Toolkit.getDefaultToolkit ()
                        .addAWTEventListener ( closeListener, AWTEvent.MOUSE_EVENT_MASK );
            }

            public void tooltipHidden ()
            {
                oneTimeTooltips.remove ( customTooltip );
                webGlassPane.removeComponentListener ( componentAdapter );
                Toolkit.getDefaultToolkit ().removeAWTEventListener ( closeListener );
                customTooltip.destroyTooltip ();
            }
        } );

        // Registering one-time tooltip
        oneTimeTooltips.add ( customTooltip );

        // Showing one-time tooltip
        webGlassPane.showComponent ( customTooltip );

        return customTooltip;
    }

    /*
    * Default tooltip show delay
    */

    public static int getDefaultDelay ()
    {
        return defaultDelay;
    }

    public static void setDefaultDelay ( int delay )
    {
        defaultDelay = delay;
    }

    /*
    * Allow more than one tooltip per component
    */

    public static boolean isAllowManyTooltips ()
    {
        return allowManyTooltips;
    }

    public static void setAllowManyTooltips ( boolean allowManyTooltips )
    {
        TooltipManager.allowManyTooltips = allowManyTooltips;
    }

    /*
    * Show hotkeys in tooltips by default
    */

    public static boolean isShowHotkeysInTooltips ()
    {
        return showHotkeysInTooltips;
    }

    public static void setShowHotkeysInTooltips ( boolean showHotkeysInTooltips )
    {
        TooltipManager.showHotkeysInTooltips = showHotkeysInTooltips;
    }

    /*
    * Show hotkeys in one-time tooltips by default
    */

    public static boolean isShowHotkeysInOneTimeTooltips ()
    {
        return showHotkeysInOneTimeTooltips;
    }

    public static void setShowHotkeysInOneTimeTooltips ( boolean showHotkeysInOneTimeTooltips )
    {
        TooltipManager.showHotkeysInOneTimeTooltips = showHotkeysInOneTimeTooltips;
    }
}
