/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.managers.tooltip;

import com.alee.laf.StyleConstants;
import com.alee.managers.hotkey.HotkeyManager;
import com.alee.utils.LafUtils;
import com.alee.utils.SwingUtils;
import com.alee.utils.TextUtils;
import info.clearthought.layout.TableLayout;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Area;
import java.awt.geom.GeneralPath;
import java.awt.geom.RoundRectangle2D;
import java.util.ArrayList;
import java.util.List;

/**
 * User: mgarin Date: 10.12.10 Time: 20:47
 */

public class WebCustomTooltip extends JComponent
{
    // Tooltip constants
    private static final Border hotkeyBorder = BorderFactory.createEmptyBorder ( 0, 4, 0, 0 );
    private static final int fadeSpeed = 25;
    private static final int cornerLength = 10;
    private static final int cornerSideX = 7;

    // Tooltip settings
    private String id = null;
    private JComponent tooltip = null;
    private Component component = null;
    private Point displayLocation = null;
    private TooltipWay displayWay = CustomTooltipStyle.displayWay;
    private boolean showHotkey = CustomTooltipStyle.showHotkey;
    private boolean defaultCloseBehavior = CustomTooltipStyle.defaultCloseBehavior;
    private int contentSpacing = CustomTooltipStyle.contentSpacing;
    private int leftRightSpacing = CustomTooltipStyle.leftRightSpacing;
    private int windowSideSpacing = CustomTooltipStyle.windowSideSpacing;
    private int round = CustomTooltipStyle.round;
    private Color borderColor = CustomTooltipStyle.borderColor;
    private Color topBgColor = CustomTooltipStyle.topBgColor;
    private Color bottomBgColor = CustomTooltipStyle.bottomBgColor;
    private Color textColor = CustomTooltipStyle.textColor;
    private float trasparency = CustomTooltipStyle.trasparency;

    // Tooltip listeners
    private List<TooltipListener> listeners = new ArrayList<TooltipListener> ();

    // Tooltip variables
    private JLabel hotkey;
    private int cornerPeak = 0;

    // Animation variables
    private FadeType fadeType;
    private int fade = 0;
    private Timer fadeTimer;

    // Component mouse adapter
    private MouseAdapter mouseAdapter;

    public WebCustomTooltip ( Component component, String tooltip )
    {
        this ( component, new JLabel ( tooltip ) );
    }

    public WebCustomTooltip ( Component component, String tooltip, TooltipWay tooltipWay )
    {
        this ( component, new JLabel ( tooltip ), tooltipWay );
    }

    public WebCustomTooltip ( Component component, String tooltip, boolean showHotkey )
    {
        this ( component, new JLabel ( tooltip ), showHotkey );
    }

    public WebCustomTooltip ( Component component, String tooltip, TooltipWay tooltipWay,
                              boolean showHotkey )
    {
        this ( component, new JLabel ( tooltip ), tooltipWay, showHotkey );
    }

    public WebCustomTooltip ( Component component, JComponent tooltip )
    {
        this ( component, tooltip, CustomTooltipStyle.displayWay );
    }

    public WebCustomTooltip ( Component component, JComponent tooltip, TooltipWay tooltipWay )
    {
        this ( component, tooltip, tooltipWay, CustomTooltipStyle.showHotkey );
    }

    public WebCustomTooltip ( Component component, JComponent tooltip, boolean showHotkey )
    {
        this ( component, tooltip, CustomTooltipStyle.displayWay, showHotkey );
    }

    public WebCustomTooltip ( Component component, JComponent tooltip, TooltipWay tooltipWay,
                              boolean showHotkey )
    {
        super ();

        setOpaque ( false );
        tooltip.setForeground ( textColor );

        // Tooltip unique id
        this.id = TextUtils.generateTooltipId ();

        // Component to which this tooltip attached
        this.component = component;

        // Tooltip component
        this.tooltip = tooltip;

        // Show component hotkey on tooltip
        this.showHotkey = showHotkey;

        // Tooltip hotkey preview component
        this.hotkey = new JLabel ();
        this.hotkey.setVerticalAlignment ( JLabel.CENTER );
        this.hotkey.setForeground ( textColor );
        this.hotkey.setFont ( this.hotkey.getFont ().deriveFont ( Font.BOLD ) );

        // Tooltip display direction
        this.displayWay = tooltipWay;

        // Fade in-out timer
        fadeTimer = new Timer ( 1000 / StyleConstants.animationFps, new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                if ( fadeType.equals ( FadeType.fadeIn ) )
                {
                    if ( fade < 100 )
                    {
                        fade = Math.min ( fade + fadeSpeed, 100 );
                        SwingUtilities.invokeLater ( new Runnable ()
                        {
                            public void run ()
                            {
                                WebCustomTooltip.this.repaint ();
                            }
                        } );
                    }
                    else
                    {
                        fireTooltipFullyShown ();
                        fadeTimer.stop ();
                    }
                }
                else if ( fadeType.equals ( FadeType.fadeOut ) )
                {
                    if ( fade > 0 )
                    {
                        fade = Math.max ( fade - fadeSpeed, 0 );
                        SwingUtilities.invokeLater ( new Runnable ()
                        {
                            public void run ()
                            {
                                WebCustomTooltip.this.repaint ();
                            }
                        } );
                    }
                    else
                    {
                        final JComponent parent = ( JComponent ) WebCustomTooltip.this.getParent ();
                        if ( parent != null )
                        {
                            SwingUtilities.invokeLater ( new Runnable ()
                            {
                                public void run ()
                                {
                                    Rectangle b = WebCustomTooltip.this.getBounds ();
                                    parent.remove ( WebCustomTooltip.this );
                                    parent.repaint ( b );
                                }
                            } );
                        }
                        fadeTimer.stop ();
                    }
                }
            }
        } );
        addAncestorListener ( new AncestorListener ()
        {
            public void ancestorAdded ( AncestorEvent event )
            {
                // Updating tooltip hotkey
                updateHotkey ();

                // Updating tooltip layout
                updateLayout ();
                // Updating tooltip location
                updateLocation ();

                // todo Fix this (need to update 2nd time to proprly position text atm)
                // Updating tooltip layout
                updateLayout ();
                // Updating tooltip location
                updateLocation ();

                // Starting fade-in animation
                fade = 0;
                fadeType = FadeType.fadeIn;
                fadeTimer.start ();

                // Informing listeners that tooltip was shown
                fireTooltipShown ();
            }

            public void ancestorRemoved ( AncestorEvent event )
            {
                // Informing listeners that tooltip was hidden
                fireTooltipHidden ();
            }

            public void ancestorMoved ( AncestorEvent event )
            {
                // Updating location of the tooltip
                updateLocation ();
            }
        } );

        mouseAdapter = new MouseAdapter ()
        {
            public void mousePressed ( MouseEvent e )
            {
                if ( defaultCloseBehavior )
                {
                    // Closing tooltip
                    closeTooltip ();
                }
            }

            public void mouseExited ( MouseEvent e )
            {
                if ( defaultCloseBehavior )
                {
                    // Closing tooltip
                    closeTooltip ();
                }
            }
        };
        component.addMouseListener ( mouseAdapter );
    }

    /*
    * Tooltip hotkey label update
    */

    private void updateHotkey ()
    {
        String hotkeyText = HotkeyManager.getComponentHotkeysString ( this.component );
        if ( showHotkey && !hotkeyText.trim ().equals ( "" ) )
        {
            hotkey.setText ( "(" + hotkeyText + ")" );
            hotkey.setBorder ( hotkeyBorder );
        }
        else
        {
            hotkey.setText ( "" );
            hotkey.setBorder ( null );
        }
    }

    /*
    * Tooltip hide
    */

    public void closeTooltip ()
    {
        if ( getParent () == null )
        {
            return;
        }

        fadeType = FadeType.fadeOut;
        if ( !fadeTimer.isRunning () )
        {
            fadeTimer.start ();
        }
    }

    /*
    * Tooltip destroy
    */

    public void destroyTooltip ()
    {
        component.removeMouseListener ( mouseAdapter );
        component.removeMouseWheelListener ( mouseAdapter );
        fireTooltipDestroyed ();
    }

    /*
    * Calculated display way
    */

    public TooltipWay getActualDisplayWay ()
    {
        if ( displayWay != null )
        {
            return displayWay;
        }
        else
        {
            if ( !component.isShowing () )
            {
                return TooltipWay.down;
            }

            JRootPane rootPane = SwingUtilities.getRootPane ( component );
            Dimension rootSize = rootPane.getSize ();
            Rectangle componentBounds = SwingUtils.getRelativeBounds ( component, rootPane );
            Dimension ps = WebCustomTooltip.this.getPreferredSize ();

            if ( componentBounds.y + getTooltipPoint ( component, TooltipWay.down ).y + ps.height <
                    rootSize.height - windowSideSpacing )
            {
                return TooltipWay.down;
            }
            else if (
                    componentBounds.y + getTooltipPoint ( component, TooltipWay.up ).y - ps.height >
                            windowSideSpacing )
            {
                return TooltipWay.up;
            }
            else if (
                    componentBounds.x + getTooltipPoint ( component, TooltipWay.up ).x + ps.width <
                            rootSize.width - windowSideSpacing )
            {
                return TooltipWay.right;
            }
            else if (
                    componentBounds.x + getTooltipPoint ( component, TooltipWay.up ).x - ps.width >
                            windowSideSpacing )
            {
                return TooltipWay.left;
            }
            else
            {
                return TooltipWay.down;
            }
        }
    }

    private Point getTooltipPoint ( Component component, TooltipWay tooltipWay )
    {
        if ( displayLocation == null )
        {
            if ( tooltipWay.equals ( TooltipWay.down ) )
            {
                return new Point ( component.getWidth () / 2,
                        component.getHeight () - cornerLength / 2 );
            }
            else if ( tooltipWay.equals ( TooltipWay.up ) )
            {
                return new Point ( component.getWidth () / 2, cornerLength / 2 );
            }
            else if ( tooltipWay.equals ( TooltipWay.left ) )
            {
                return new Point ( cornerLength / 2, getHeight () / 2 );
            }
            else
            {
                return new Point ( component.getWidth () - cornerLength / 2, getHeight () / 2 );
            }
        }
        else
        {
            return displayLocation;
        }
    }

    /*
    * Tooltip layout update
    */

    public void updateLayout ()
    {
        TooltipWay displayWay = getActualDisplayWay ();

        int leftSpacing = contentSpacing + leftRightSpacing +
                ( displayWay.equals ( TooltipWay.right ) ? cornerLength : 0 );
        int rightSpacing = contentSpacing + leftRightSpacing +
                ( displayWay.equals ( TooltipWay.left ) ? cornerLength : 0 );
        int topSpacing =
                contentSpacing + ( displayWay.equals ( TooltipWay.down ) ? cornerLength : 0 );
        int bottomSpacing =
                contentSpacing + ( displayWay.equals ( TooltipWay.up ) ? cornerLength : 0 );

        double[] columns =
                { leftSpacing, TableLayout.PREFERRED, TableLayout.PREFERRED, rightSpacing };
        double[] rows = { topSpacing, TableLayout.PREFERRED, bottomSpacing };

        TableLayout layout = new TableLayout ( new double[][]{ columns, rows } );
        layout.setHGap ( 0 );
        layout.setVGap ( 0 );
        setLayout ( layout );

        removeAll ();
        add ( tooltip, "1,1" );
        add ( hotkey, "2,1" );
        revalidate ();
    }

    /*
    * Tooltip spacing between sides and content
    */

    public int getContentSpacing ()
    {
        return contentSpacing;
    }

    public void setContentSpacing ( int contentSpacing )
    {
        this.contentSpacing = contentSpacing;
        updateLayout ();
    }

    /*
    * Additional tooltip content left/right spacing
    */

    public int getLeftRightSpacing ()
    {
        return leftRightSpacing;
    }

    public void setLeftRightSpacing ( int leftRightSpacing )
    {
        this.leftRightSpacing = leftRightSpacing;
    }

    /*
    * Tooltip location on glasspane update
    */

    public void updateLocation ()
    {
        if ( getParent () != null && getParent ().isShowing () && component.isShowing () )
        {
            TooltipWay displayWay = getActualDisplayWay ();

            Point p = getParent ().getLocationOnScreen ();
            Point c = component.getLocationOnScreen ();
            Dimension ps = getPreferredSize ();

            if ( displayWay.equals ( TooltipWay.up ) || displayWay.equals ( TooltipWay.down ) )
            {
                int compMiddle;
                int compTipY;
                if ( displayLocation == null )
                {
                    compMiddle = c.x - p.x + component.getWidth () / 2;
                    compTipY = c.y - p.y + ( displayWay.equals ( TooltipWay.up ) ?
                            ( cornerLength / 2 - ps.height ) :
                            ( component.getHeight () - cornerLength / 2 ) );
                }
                else
                {
                    compMiddle = c.x - p.x + displayLocation.x;
                    compTipY = c.y - p.y + displayLocation.y -
                            ( displayWay.equals ( TooltipWay.up ) ? ps.height : 0 );
                }

                if ( compMiddle - ps.width / 2 < windowSideSpacing )
                {
                    int cw = windowSideSpacing - ( compMiddle - ps.width / 2 );
                    cornerPeak = Math.max ( round + cornerSideX + 1, getWidth () / 2 - cw );
                    setLocation ( windowSideSpacing, compTipY );
                }
                else if ( compMiddle + ps.width / 2 > getParent ().getWidth () - windowSideSpacing )
                {
                    int cw = ( compMiddle + ps.width / 2 ) -
                            ( getParent ().getWidth () - windowSideSpacing );
                    cornerPeak =
                            Math.min ( ps.width - round - cornerSideX - 1, getWidth () / 2 + cw );
                    setLocation ( getParent ().getWidth () - windowSideSpacing - ps.width,
                            compTipY );
                }
                else
                {
                    cornerPeak = getWidth () / 2;
                    setLocation ( compMiddle - ps.width / 2, compTipY );
                }
            }
            else if ( displayWay.equals ( TooltipWay.left ) ||
                    displayWay.equals ( TooltipWay.right ) )
            {
                int compMiddle;
                int compTipX;
                if ( displayLocation == null )
                {
                    compMiddle = c.y - p.y + component.getHeight () / 2;
                    compTipX = c.x - p.x + ( displayWay.equals ( TooltipWay.left ) ?
                            ( cornerLength / 2 - ps.width ) :
                            ( component.getWidth () - cornerLength / 2 ) );
                }
                else
                {
                    compMiddle = c.y - p.y + displayLocation.y;
                    compTipX = c.x - p.x + displayLocation.x -
                            ( displayWay.equals ( TooltipWay.left ) ? ps.width : 0 );
                }

                if ( compMiddle - ps.height / 2 < windowSideSpacing )
                {
                    int cw = windowSideSpacing - ( compMiddle - ps.height / 2 );
                    cornerPeak = Math.max ( round + cornerSideX + 1, getHeight () / 2 - cw );
                    setLocation ( compTipX, windowSideSpacing );
                }
                else if ( compMiddle + ps.height / 2 >
                        getParent ().getHeight () - windowSideSpacing )
                {
                    int cw = ( compMiddle + ps.height / 2 ) -
                            ( getParent ().getHeight () - windowSideSpacing );
                    cornerPeak =
                            Math.min ( ps.height - round - cornerSideX - 1, getHeight () / 2 + cw );
                    setLocation ( compTipX,
                            getParent ().getHeight () - windowSideSpacing - ps.height );
                }
                else
                {
                    cornerPeak = getHeight () / 2;
                    setLocation ( compTipX, compMiddle - ps.height / 2 );
                }
            }
            setSize ( getPreferredSize () );
        }
    }

    /*
    * Custom display location relative to component
    */

    public Point getDisplayLocation ()
    {
        return displayLocation;
    }

    public void setDisplayLocation ( Point displayLocation )
    {
        this.displayLocation = displayLocation;
        updateLocation ();
    }

    /*
    * Minimal spacing between tooltip and window edge
    */

    public int getWindowSideSpacing ()
    {
        return windowSideSpacing;
    }

    public void setWindowSideSpacing ( int windowSideSpacing )
    {
        this.windowSideSpacing = windowSideSpacing;
        updateLocation ();
    }

    /*
    * Tooltip corners rounding
    */

    public int getRound ()
    {
        return round;
    }

    public void setRound ( int round )
    {
        this.round = round;
        updateLocation ();
    }

    /*
    * Component to which this tooltip attached
    */

    public Component getComponent ()
    {
        return component;
    }

    /*
    * Tooltip unique id
    */

    public String getId ()
    {
        return id;
    }

    /*
    * Tooltip component
    */

    public JComponent getTooltip ()
    {
        return tooltip;
    }

    public void setTooltip ( JComponent tooltip )
    {
        this.tooltip = tooltip;
        updateLayout ();
        updateLocation ();
    }

    /*
    * Tooltip display way
    */

    public TooltipWay getDisplayWay ()
    {
        return displayWay;
    }

    public void setDisplayWay ( TooltipWay displayWay )
    {
        this.displayWay = displayWay;
        updateLayout ();
        updateLocation ();
    }

    /*
    * Should display component hotkey on tooltip or not
    */

    public boolean isShowHotkey ()
    {
        return showHotkey;
    }

    public void setShowHotkey ( boolean showHotkey )
    {
        this.showHotkey = showHotkey;
        updateHotkey ();
        updateLocation ();
    }

    /*
    * Default tooltip close behavior
    */

    public boolean isDefaultCloseBehavior ()
    {
        return defaultCloseBehavior;
    }

    public void setDefaultCloseBehavior ( boolean defaultCloseBehavior )
    {
        this.defaultCloseBehavior = defaultCloseBehavior;
    }

    /*
    * Tooltip text color
    */

    public Color getTextColor ()
    {
        return textColor;
    }

    public void setTextColor ( Color textColor )
    {
        this.textColor = textColor;
    }

    /*
    * Tooltip top background color
    */

    public Color getTopBgColor ()
    {
        return topBgColor;
    }

    public void setTopBgColor ( Color topBgColor )
    {
        this.topBgColor = topBgColor;
    }

    /*
    * Tooltip bottom background color
    */

    public Color getBottomBgColor ()
    {
        return bottomBgColor;
    }

    public void setBottomBgColor ( Color bottomBgColor )
    {
        this.bottomBgColor = bottomBgColor;
    }

    /*
    * Tooltip border color
    */

    public Color getBorderColor ()
    {
        return borderColor;
    }

    public void setBorderColor ( Color borderColor )
    {
        this.borderColor = borderColor;
    }

    /*
    * Tooltip background transparency
    */

    public float getTrasparency ()
    {
        return trasparency;
    }

    public void setTrasparency ( float trasparency )
    {
        this.trasparency = trasparency;
    }

    /*
    * Tooltip background painting
    */

    protected void paintComponent ( Graphics g )
    {
        super.paintComponent ( g );

        Graphics2D g2d = ( Graphics2D ) g;
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );

        TooltipWay displayWay = getActualDisplayWay ();

        // Fade animation
        if ( fade != 100 )
        {
            g2d.setComposite (
                    AlphaComposite.getInstance ( AlphaComposite.SRC_OVER, fade / 100f ) );
        }

        Composite oldComposite =
                LafUtils.setupAlphaComposite ( g2d, g2d.getComposite (), trasparency,
                        trasparency < 1f );

        // Tooltip background
        if ( topBgColor != null )
        {
            if ( bottomBgColor != null )
            {
                // Gradient background
                if ( displayWay.equals ( TooltipWay.down ) )
                {
                    g2d.setPaint ( new GradientPaint ( 0, getHeight () * 2 / 3, topBgColor, 0,
                            getHeight (), bottomBgColor ) );
                }
                else
                {
                    g2d.setPaint (
                            new GradientPaint ( 0, getHeight () / 3, topBgColor, 0, getHeight (),
                                    bottomBgColor ) );
                }
            }
            else
            {
                // Single color background
                g2d.setPaint ( topBgColor );
            }
            g2d.fill ( getTooltipShape ( displayWay, true ) );
        }

        // Tooltip border
        if ( borderColor != null )
        {
            g2d.setPaint ( borderColor );
            g2d.draw ( getTooltipShape ( displayWay, false ) );
        }


        LafUtils.restoreComposite ( g2d, oldComposite, oldComposite != null );

        //        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING,
        //                RenderingHints.VALUE_ANTIALIAS_OFF );
        //        g2d.setRenderingHint ( RenderingHints.KEY_TEXT_ANTIALIASING,
        //                RenderingHints.VALUE_TEXT_ANTIALIAS_OFF );
    }

    private Shape getTooltipShape ( TooltipWay displayWay, boolean fill )
    {
        Area borderShape;

        // Fill shape extend
        float fillExtend = fill ? 0.5f : 0;

        // Content area shape
        float widthMinus =
                ( displayWay.equals ( TooltipWay.left ) || displayWay.equals ( TooltipWay.right ) ?
                        cornerLength + 1 : 1 ) - fillExtend;
        float heightMinus =
                ( displayWay.equals ( TooltipWay.up ) || displayWay.equals ( TooltipWay.down ) ?
                        cornerLength + 1 : 1 ) - fillExtend;
        borderShape = new Area ( new RoundRectangle2D.Double (
                displayWay.equals ( TooltipWay.right ) ? cornerLength : 0,
                displayWay.equals ( TooltipWay.down ) ? cornerLength : 0, getWidth () - widthMinus,
                getHeight () - heightMinus, round * 2, round * 2 ) );

        // Corner shape
        GeneralPath gp = new GeneralPath ( GeneralPath.WIND_EVEN_ODD );
        if ( displayWay.equals ( TooltipWay.up ) )
        {
            gp.moveTo ( cornerPeak + fillExtend, getHeight () - 1 );
            gp.lineTo ( cornerPeak - cornerSideX + fillExtend, getHeight () - 1 - cornerLength );
            gp.lineTo ( cornerPeak + cornerSideX + fillExtend, getHeight () - 1 - cornerLength );
        }
        else if ( displayWay.equals ( TooltipWay.down ) )
        {
            gp.moveTo ( cornerPeak, 0 );
            gp.lineTo ( cornerPeak - cornerSideX, cornerLength );
            gp.lineTo ( cornerPeak + cornerSideX, cornerLength );
        }
        else if ( displayWay.equals ( TooltipWay.left ) )
        {
            gp.moveTo ( getWidth () - 1, cornerPeak + fillExtend );
            gp.lineTo ( getWidth () - 1 - cornerLength, cornerPeak + fillExtend - cornerSideX );
            gp.lineTo ( getWidth () - 1 - cornerLength, cornerPeak + fillExtend + cornerSideX );
        }
        else if ( displayWay.equals ( TooltipWay.right ) )
        {
            gp.moveTo ( 0, cornerPeak );
            gp.lineTo ( cornerLength, cornerPeak - cornerSideX );
            gp.lineTo ( cornerLength, cornerPeak + cornerSideX );
        }
        gp.closePath ();
        borderShape.add ( new Area ( gp ) );

        return borderShape;
    }

    /*
    * Tooltip listeners
    */

    private void fireTooltipShown ()
    {
        for ( TooltipListener listener : listeners )
        {
            listener.tooltipShown ();
        }
    }

    private void fireTooltipFullyShown ()
    {
        for ( TooltipListener listener : listeners )
        {
            listener.tooltipFullyShown ();
        }
    }

    private void fireTooltipHidden ()
    {
        for ( TooltipListener listener : listeners )
        {
            listener.tooltipHidden ();
        }
    }

    private void fireTooltipDestroyed ()
    {
        for ( TooltipListener listener : listeners )
        {
            listener.tooltipDestroyed ();
        }
    }

    public void addGuiTooltipListener ( TooltipListener listener )
    {
        listeners.add ( listener );
    }

    public void removeGuiTooltipListener ( TooltipListener listener )
    {
        listeners.remove ( listener );
    }
}
