/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils;

import com.alee.extended.filefilter.DefaultFileFilter;
import com.alee.laf.GlobalConstants;
import com.alee.laf.StyleConstants;
import com.alee.managers.proxy.ProxyManager;
import com.alee.utils.file.Description;
import com.alee.utils.file.FileDownloadListener;

import javax.swing.*;
import javax.swing.filechooser.FileSystemView;
import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.nio.channels.FileChannel;
import java.text.DecimalFormat;
import java.util.*;

/**
 * User: mgarin Date: 05.07.11 Time: 13:04
 */

public class FileUtils
{
    public static final String SEPARATOR = StyleConstants.SEPARATOR;

    private static FileSystemView fsv = FileSystemView.getFileSystemView ();

    /**
     * Returns user home directory
     */

    public static File getUserHomeFile ()
    {
        return new File ( getUserHome () );
    }

    public static String getUserHome ()
    {
        String home = System.getProperty ( "user.home" );
        if ( !home.endsWith ( File.separator ) )
        {
            home += File.separator;
        }
        return home;
    }

    /**
     * Returns FileSystem roots
     */

    public static File[] getRoots ()
    {
        File[] roots = File.listRoots ();
        int proper = 0;
        for ( File root : roots )
        {
            if ( root.isDirectory () )
            {
                proper++;
            }
        }
        File[] properRoots = new File[ proper ];
        int currentIndex = 0;
        for ( File root : roots )
        {
            if ( root.isDirectory () )
            {
                properRoots[ currentIndex ] = root;
                currentIndex++;
            }
        }
        return properRoots;
    }

    /**
     * Are files equal
     */

    public static boolean areEqual ( File file1, File file2 )
    {
        return file1 == null && file2 == null || file1 != null && file2 != null &&
                file1.getAbsolutePath ().equals ( file2.getAbsolutePath () );
    }

    /**
     * Are file lists equal
     */

    public static boolean areEqual ( List<File> files1, List<File> files2 )
    {
        if ( files1.size () != files2.size () )
        {
            return false;
        }
        else if ( files1.size () == files2.size () && files2.size () == 0 )
        {
            return true;
        }
        else
        {
            for ( int i = 0; i < files1.size (); i++ )
            {
                if ( !files1.get ( i ).getAbsolutePath ()
                        .equals ( files2.get ( i ).getAbsolutePath () ) )
                {
                    return false;
                }
            }
            return true;
        }
    }

    /**
     * Is file contained in list
     */

    public static boolean containtsFile ( List files, File file )
    {
        for ( Object f : files )
        {
            if ( f instanceof File )
            {
                if ( ( ( File ) f ).getAbsolutePath ().equals ( file.getAbsolutePath () ) )
                {
                    return true;
                }
            }
            else if ( f instanceof String )
            {
                if ( f.equals ( file.getAbsolutePath () ) )
                {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Is file accepted by any specified filter or not
     */

    public static boolean isFileAccepted ( File file, List<DefaultFileFilter> filters )
    {
        if ( filters == null || filters.size () == 0 )
        {
            return true;
        }
        else
        {
            for ( FileFilter fileFilter : filters )
            {
                if ( fileFilter.accept ( file ) )
                {
                    return true;
                }
            }
            return false;
        }
    }

    /**
     * Valid file name retrieval from any string
     */

    public static final char[] ILLEGAL_CHARACTERS =
            { '/', '\n', '\r', '\t', '\0', '\f', '\"', '`', '!', '?', '*', '\\', '<', '>', '|', ':',
                    ';', '.', ',', '%', '$', '@', '#', '^', '{', '}', '[', ']', ']' };

    public static String getProperFileName ( String fileName )
    {
        StringBuilder sb = new StringBuilder ();
        for ( int i = 0; i < fileName.length (); ++i )
        {
            if ( !isIllegalFileNameChar ( fileName.charAt ( i ) ) )
            {
                sb.append ( fileName.charAt ( i ) );
            }
        }
        return sb.toString ().replaceAll ( " ", "_" );
    }

    public static boolean isIllegalFileNameChar ( char c )
    {
        boolean isIllegal = false;
        for ( char ILLEGAL_CHARACTER : ILLEGAL_CHARACTERS )
        {
            if ( c == ILLEGAL_CHARACTER )
            {
                isIllegal = true;
            }
        }
        return isIllegal;
    }

    /**
     * Checks if specified File is a CD/DVD/Bluray drive
     */

    public static boolean isCdDrive ( File file )
    {
        if ( file.getParent () == null )
        {
            String sysDes = getDisplayFileDescription ( file );
            String description;
            if ( sysDes != null )
            {
                description = sysDes.toLowerCase ();
            }
            else
            {
                description = file.getName ();
            }
            return description.contains ( "cd" ) || description.contains ( "dvd" ) ||
                    description.contains ( "blu-ray" ) || description.contains ( "bluray" );
        }
        else
        {
            return false;
        }
    }

    /**
     * Returns system file or folder description
     */

    public static String getFileTypeDescription ( File file )
    {
        return fsv.getSystemTypeDescription ( file );
    }

    /**
     * Cached isFile property retrieval
     */

    private static Map<String, Boolean> isFileCache = new HashMap<String, Boolean> ();

    public static void clearIsFileCache ()
    {
        isFileCache.clear ();
    }

    public static boolean isFile ( File file )
    {
        if ( file == null )
        {
            return false;
        }
        else if ( isFileCache.containsKey ( file.getAbsolutePath () ) )
        {
            return isFileCache.get ( file.getAbsolutePath () );
        }
        else
        {
            boolean isFile = file.isFile ();
            isFileCache.put ( file.getAbsolutePath (), isFile );
            return isFile;
        }
    }

    /**
     * Cached isHidden property retrieval
     */

    private static Map<String, Boolean> isHiddenCache = new HashMap<String, Boolean> ();

    public static void clearIsHiddenCache ()
    {
        isHiddenCache.clear ();
    }

    public static boolean isHidden ( File file )
    {
        if ( file == null )
        {
            return false;
        }
        else if ( isHiddenCache.containsKey ( file.getAbsolutePath () ) )
        {
            return isHiddenCache.get ( file.getAbsolutePath () );
        }
        else
        {
            boolean isHidden = file.getParentFile () != null && file.isHidden ();
            isHiddenCache.put ( file.getAbsolutePath (), isHidden );
            return isHidden;
        }
    }

    /**
     * File icon retrieval methods
     */

    private static Map<String, ImageIcon> extensionIcons = new HashMap<String, ImageIcon> ();

    public static ImageIcon getFileIcon ( File file )
    {
        return getFileIcon ( file, false );
    }

    public static ImageIcon getFileIcon ( File file, boolean large )
    {
        //        if ( SystemUtils.isWindows () )
        //        {
        //            if ( file == null )
        //            {
        //                return null;
        //            }
        //
        //            ShellFolder sf;
        //            try
        //            {
        //                sf = ShellFolder.getShellFolder ( file );
        //            }
        //            catch ( FileNotFoundException e )
        //            {
        //                return null;
        //            }
        //
        //            Image img = sf.getIcon ( large );
        //            if ( img != null )
        //            {
        //                return new ImageIcon ( img, sf.getFolderType () );
        //            }
        //            else
        //            {
        //                return null;
        //            }
        //        }
        //        else
        //        {
        return getStandartFileIcon ( file, large );
        //        }
    }

    public static ImageIcon getStandartFileIcon ( File file, boolean large )
    {
        if ( file == null )
        {
            return null;
        }
        String extension;
        if ( isFile ( file ) )
        {
            extension = getFileExtPart ( file.getName (), false ).trim ().toLowerCase ();
            if ( extension.trim ().equals ( "" ) )
            {
                extension = file.getAbsolutePath ();
            }
        }
        else
        {
            if ( isCdDrive ( file ) )
            {
                extension = "cd-drive";
            }
            else if ( file.getParent () == null )
            {
                extension = "drive";
            }
            else
            {
                extension = "folder";
            }
        }
        float transparency = isHidden ( file ) ? 0.5f : 1f;
        String key = extension + StyleConstants.SEPARATOR + ( large ? "32" : "16" ) +
                StyleConstants.SEPARATOR + transparency;
        if ( extensionIcons.containsKey ( key ) )
        {
            return extensionIcons.get ( key );
        }
        else
        {
            ImageIcon icon = getIconResource ( FileUtils.class,
                    "icons/extensions/" + ( large ? "32" : "16" ) + "/file_extension_" + extension +
                            ".png", transparency );
            if ( icon == null )
            {
                if ( file.isFile () )
                {
                    //                    if ( file.canExecute () )
                    //                    {
                    //                        icon =  getIconResource ( FileUtils.class,
                    //                                "icons/extensions/" + ( large ? "32" : "16" ) +
                    //                                        "/executable.png", transparency );
                    //                    }
                    //                    else
                    //                    {
                    icon = getIconResource ( FileUtils.class,
                            "icons/extensions/" + ( large ? "32" : "16" ) + "/file.png",
                            transparency );
                    //                    }
                }
                else
                {
                    if ( isCdDrive ( file ) )
                    {
                        icon = getIconResource ( FileUtils.class,
                                "icons/extensions/" + ( large ? "32" : "16" ) + "/cd_drive.png",
                                transparency );
                    }
                    else if ( file.getParent () == null )
                    {
                        icon = getIconResource ( FileUtils.class,
                                "icons/extensions/" + ( large ? "32" : "16" ) + "/drive.png",
                                transparency );
                    }
                    else
                    {
                        icon = getIconResource ( FileUtils.class,
                                "icons/extensions/" + ( large ? "32" : "16" ) + "/folder.png",
                                transparency );
                    }
                }
            }
            extensionIcons.put ( key, icon );
            return icon;
        }
    }

    /**
     * Returns cached image icon
     */

    private static Map<String, ImageIcon> cachedResourceIcons = new HashMap<String, ImageIcon> ();

    public static ImageIcon getIconResource ( Class nearClass, String resource )
    {
        return getIconResource ( nearClass, resource, 1f );
    }

    public static ImageIcon getIconResource ( Class nearClass, String resource, float transparency )
    {
        String key =
                nearClass.getCanonicalName () + SEPARATOR + resource + SEPARATOR + transparency;
        if ( cachedResourceIcons.containsKey ( key ) )
        {
            return cachedResourceIcons.get ( key );
        }
        else
        {
            URL url = nearClass.getResource ( resource );
            ImageIcon icon;
            if ( url != null )
            {
                icon = new ImageIcon ( url );
                if ( transparency < 1f )
                {
                    icon = ImageUtils.createTransparentCopy ( icon, transparency );
                }
            }
            else
            {
                icon = null;
            }
            cachedResourceIcons.put ( key, icon );
            return icon;
        }
    }

    /**
     * Returns file name without extension
     */

    public static String getFileNamePart ( File file )
    {
        return getFileNamePart ( file.getName () );
    }

    public static String getFileNamePart ( String name )
    {
        int i = name.lastIndexOf ( "." );
        return i == -1 ? name : name.substring ( 0, i );
    }

    /**
     * Returns file extension
     */

    public static String getFileExtPart ( String file, boolean withDot )
    {
        int i = file.lastIndexOf ( "." );
        return i == -1 ? "" : withDot ? file.substring ( i ) : file.substring ( i + 1 );
    }

    /**
     * Returns shortened file name
     */

    public static String getShortFileName ( String file )
    {
        String name = getFileNamePart ( file );
        if ( name.length () > 30 )
        {
            // 2 symbols taken by dots
            return name.substring ( 0, 28 ) + "..." + getFileExtPart ( file, false );
        }
        else
        {
            return file;
        }
    }

    /**
     * Sorts files by their names
     */

    public static List sortFiles ( List<File> files )
    {
        if ( files != null )
        {
            Collections.sort ( files, new Comparator<File> ()
            {
                public int compare ( File f1, File f2 )
                {
                    if ( f1.isDirectory () && f2.isFile () )
                    {
                        return -1;
                    }
                    if ( f1.isFile () && f2.isDirectory () )
                    {
                        return 1;
                    }
                    //                return fsv.getSystemDisplayName ( f1 )
                    //                        .compareTo ( fsv.getSystemDisplayName ( f2 ) );
                    return f1.getName ().compareTo ( f2.getName () );
                }
            } );
        }
        return files;
    }

    public static File[] sortFiles ( File[] files )
    {
        if ( files != null )
        {
            Arrays.sort ( files, GlobalConstants.FILE_COMPARATOR );
        }
        return files;
    }

    /**
     * Returns free file name in specified directory
     */

    public static String getFreeName ( String dir, String name )
    {
        return getFreeName ( new File ( dir ), name );
    }

    public static String getFreeName ( File dir, String name )
    {
        // Gathering existing files
        List<String> exist = new ArrayList<String> ();
        File[] files = dir.listFiles ();
        if ( files != null )
        {
            for ( File img : files )
            {
                exist.add ( img.getName () );
            }
        }
        return getFreeName ( exist, name );
    }

    public static String getFreeName ( List<String> exist, String name )
    {
        // If specified name doesn't exist return it
        if ( !exist.contains ( name ) )
        {
            return name;
        }

        // Gathering name parts
        int dot = name.lastIndexOf ( "." );
        String nameStart = dot != -1 ? name.substring ( 0, dot ).trim () : name;
        String nameExt = dot != -1 ? name.substring ( name.lastIndexOf ( "." ) ) : null;
        String nameReal = null;
        Integer index = null;
        int ob = nameStart.lastIndexOf ( "(" );
        int cb = nameStart.lastIndexOf ( ")" );
        if ( ob < cb && cb == nameStart.length () - 1 )
        {
            try
            {
                nameReal = nameStart.substring ( 0, ob );
                index = Integer.parseInt ( nameStart.substring ( ob + 1, cb ) );
                index++;
            }
            catch ( Throwable e )
            {
                //
            }
        }

        // Choosing new name
        int i = 1;
        while ( exist.contains ( name ) )
        {
            if ( nameReal != null && index != null )
            {
                name = nameReal + "(" + index + ")" + nameExt;
                index++;
            }
            else
            {
                name = nameStart + " (" + i + ")" + ( nameExt != null ? nameExt : "" );
                i++;
            }
        }

        return name;
    }

    /**
     * Recursive files removal
     */

    public static void removeFiles ( Object... files )
    {
        for ( Object object : files )
        {
            if ( object instanceof File )
            {
                removeFile ( ( File ) object );
            }
        }
    }

    public static void removeFile ( File file )
    {
        if ( file.exists () )
        {
            if ( file.isFile () )
            {
                if ( !file.delete () )
                {
                    file.deleteOnExit ();
                }
            }
            else if ( file.isDirectory () )
            {
                for ( File child : file.listFiles () )
                {
                    removeFile ( child );
                }
                if ( !file.delete () )
                {
                    file.deleteOnExit ();
                }
            }
        }
    }

    /**
     * Returns full file description
     */

    //    private static SimpleDateFormat sdf = new SimpleDateFormat ( "dd.MM.yyyy HH:mm" );
    private static Map<String, Description> descriptionsCache = new HashMap<String, Description> ();

    public static Description getFileDescription ( File file, String imageSize )
    {
        if ( descriptionsCache.containsKey ( file.getAbsolutePath () ) )
        {
            return descriptionsCache.get ( file.getAbsolutePath () );
        }
        else
        {
            Description description = createFileDescription ( file, imageSize );
            descriptionsCache.put ( file.getAbsolutePath (), description );
            return description;
        }
    }

    public static Description createFileDescription ( File file, String imageSize )
    {
        // File name
        String name = getDisplayFileName ( file );

        // File size / image size
        String size = file.isFile () ?
                getDisplayFileSize ( file ) + ( imageSize != null ? " (" + imageSize + ")" : "" ) :
                null;

        // File type description
        String description = getDisplayFileDescription ( file );

        // Modification date
        //        long modified = file.lastModified ();
        //        String date = modified != 0 ? /*"<br>" +*/ sdf.format ( new Date ( modified ) ) : null;

        return new Description ( name, size, description, null/*date*/ );
    }

    /**
     * Displayable file name
     */

    private static Map<String, String> displayFileNameCache = new HashMap<String, String> ();

    public static String getDisplayFileName ( File file )
    {
        String absolutePath = file.getAbsolutePath ();
        if ( displayFileNameCache.containsKey ( absolutePath ) )
        {
            return displayFileNameCache.get ( absolutePath );
        }
        else
        {
            String name = fsv.getSystemDisplayName ( file );
            if ( name == null || name.trim ().equals ( "" ) )
            {
                name = getDisplayFileDescription ( file );
            }
            displayFileNameCache.put ( absolutePath, name );
            return name;
        }
    }

    /**
     * Displayable file size
     */

    public static String getDisplayFileSize ( File file )
    {
        return getFileSizeString ( file.length () );
    }

    public static String getDisplayFileSize ( File file, int digits )
    {
        return getFileSizeString ( file.length (), digits );
    }

    public static String getFileSizeString ( long size )
    {
        return getFileSizeString ( size, 2 );
    }

    public static String getFileSizeString ( long size, int digits )
    {
        DecimalFormat df = new DecimalFormat ( digits == 0 ? "#" : "#." + getDigits ( digits ) );
        if ( size < 1024 )
        {
            return df.format ( size ) + " " + UIManager.getString ( "FileUtils.file.size.b" );
        }
        else if ( size >= 1024 && size < 1024 * 1024 )
        {
            return df.format ( ( float ) size / 1024 ) + " " +
                    UIManager.getString ( "FileUtils.file.size.kb" );
        }
        else if ( size >= 1024 * 1024 && size < 1024 * 1024 * 1024 )
        {
            return df.format ( ( float ) size / 1024 / 1024 ) + " " +
                    UIManager.getString ( "FileUtils.file.size.mb" );
        }
        else
        {
            return df.format ( ( float ) size / 1024 / 1024 / 1024 ) + " " +
                    UIManager.getString ( "FileUtils.file.size.gb" );
        }
    }

    private static String getDigits ( int number )
    {
        String digits = "";
        for ( int i = 0; i < number; i++ )
        {
            digits += "#";
        }
        return digits;
    }

    /**
     * Displayable file type
     */

    private static Map<String, String> displayDescriptionCache = new HashMap<String, String> ();

    public static String getDisplayFileDescription ( File file )
    {
        if ( file == null )
        {
            return "";
        }
        else if ( displayDescriptionCache.containsKey ( file.getAbsolutePath () ) )
        {
            return displayDescriptionCache.get ( file.getAbsolutePath () );
        }
        else
        {
            String description = fsv.getSystemTypeDescription ( file );
            displayDescriptionCache.put ( file.getAbsolutePath (), description );
            return description;
        }
    }

    /**
     * Copies folder with all included content to any local destination
     */

    public static void copyFolder ( String src, String dst )
    {
        File srcFolder = new File ( src );
        File dstFolder = new File ( dst );
        if ( srcFolder.exists () && srcFolder.isDirectory () )
        {
            // Creating destination folder
            if ( !dstFolder.exists () )
            {
                dstFolder.mkdir ();
            }

            // Copying all subfolders and subfiles
            for ( File file : srcFolder.listFiles () )
            {
                String copied = dstFolder.getAbsolutePath () + File.separator + file.getName ();
                if ( file.isDirectory () )
                {
                    copyFolder ( file.getAbsolutePath (), copied );
                }
                else
                {
                    copyFile ( file.getAbsolutePath (), copied );
                }
            }
        }
    }

    /**
     * Copies file to any local destination
     */

    public static boolean copyFile ( String sourceFile, String dstFile )
    {
        try
        {
            // Creating destination folder if needed
            File dstFolder = new File ( new File ( dstFile ).getParent () );
            if ( !dstFolder.exists () )
            {
                dstFolder.mkdirs ();
            }

            FileChannel source = new FileInputStream ( sourceFile ).getChannel ();
            FileChannel dst = new FileOutputStream ( dstFile ).getChannel ();
            return copyFile ( source, dst );
        }
        catch ( FileNotFoundException e )
        {
            return false;
        }
    }

    public static boolean copyFile ( File sourceFile, File dstFile )
    {
        if ( sourceFile.exists () && sourceFile.isFile () )
        {
            try
            {
                // Creating destination folder if needed
                File dstFolder = new File ( dstFile.getParent () );
                if ( !dstFolder.exists () )
                {
                    dstFolder.mkdirs ();
                }

                FileChannel source = new FileInputStream ( sourceFile ).getChannel ();
                FileChannel dst = new FileOutputStream ( dstFile ).getChannel ();
                return copyFile ( source, dst );
            }
            catch ( FileNotFoundException e )
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }

    public static boolean copyFile ( FileChannel source, FileChannel dst )
    {
        try
        {
            dst.transferFrom ( source, 0, source.size () );

            source.close ();
            dst.close ();

            return true;
        }
        catch ( IOException e )
        {
            return false;
        }
    }

    /**
     * Reads any file into String
     */

    private static final int BUFFER_SIZE = 2048;

    public static String readToString ( Class nearClass, String resource )
    {
        try
        {
            InputStream inputStream = nearClass.getResourceAsStream ( resource );
            if ( inputStream != null )
            {
                InputStreamReader inputStreamReader = new InputStreamReader ( inputStream );
                return readToString ( inputStreamReader );
            }
            else
            {
                return null;
            }
        }
        catch ( Throwable e )
        {
            return null;
        }
    }

    public static String readToString ( File file )
    {
        try
        {
            if ( file != null && file.exists () && file.isFile () )
            {
                return readToString ( new FileReader ( file ) );
            }
            else
            {
                return null;
            }
        }
        catch ( Throwable e )
        {
            return null;
        }
    }

    public static String readToString ( Reader reader )
    {
        try
        {
            if ( reader == null )
            {
                return "";
            }
            try
            {
                int charsRead;
                final char[] copyBuffer = new char[ BUFFER_SIZE ];
                final StringBuilder sb = new StringBuilder ();
                while ( ( charsRead = reader.read ( copyBuffer, 0, BUFFER_SIZE ) ) != -1 )
                {
                    sb.append ( copyBuffer, 0, charsRead );
                }
                return sb.toString ();
            }
            finally
            {
                reader.close ();
            }
        }
        catch ( Throwable e )
        {
            return "";
        }
    }

    /**
     * Returns list of all files in this directory and all its subdirectories
     */

    public static List<String> getSubpaths ( String dir )
    {
        return getSubpaths ( new File ( dir ) );
    }

    public static List<String> getSubpaths ( File dir )
    {
        return getSubpaths ( dir, "" );
    }

    public static List<String> getSubpaths ( File dir, String startPath )
    {
        return getSubpaths ( dir, startPath, new ArrayList<String> () );
    }

    public static List<String> getSubpaths ( File dir, String startPath, List<String> subPaths )
    {
        for ( File file : dir.listFiles () )
        {
            if ( file.isFile () )
            {
                subPaths.add ( startPath + file.getName () );
            }
            else if ( file.isDirectory () )
            {
                getSubpaths ( file, startPath + file.getName () + File.separator, subPaths );
            }
        }
        return subPaths;
    }

    /**
     * Downloads file from specified URL to destination File
     */

    public static File downloadFile ( String url, File dst )
    {
        return downloadFile ( url, dst, null );
    }

    public static File downloadFile ( String url, File dst, FileDownloadListener listener )
    {
        // Downloading file with a short connection timeout
        if ( copyUrl ( url, dst, false, null, GlobalConstants.SHORT_TIMEOUT, listener ) )
        {
            // Download succeed
            return dst;
        }
        else
        {
            // Download failed
            return null;
        }
    }

    /**
     * Copies file from specified URL to destination File
     */

    public static boolean copyUrl ( String sourceUrl, String dstFile, boolean encodeUrl,
                                    String contentType, int timeouts )
    {
        return copyUrl ( sourceUrl, new File ( dstFile ), encodeUrl, contentType, timeouts, null );
    }

    public static boolean copyUrl ( String sourceUrl, String dstFile, boolean encodeUrl,
                                    String contentType, int timeouts,
                                    FileDownloadListener listener )
    {
        return copyUrl ( sourceUrl, new File ( dstFile ), encodeUrl, contentType, timeouts,
                listener );
    }

    public static boolean copyUrl ( String sourceUrl, File dstFile, boolean encodeUrl,
                                    String contentType, int timeouts )
    {
        return copyUrl ( sourceUrl, dstFile, encodeUrl, contentType, timeouts, null );
    }

    public static boolean copyUrl ( String sourceUrl, File dstFile, boolean encodeUrl,
                                    String contentType, int timeouts,
                                    FileDownloadListener listener )
    {
        try
        {
            // Creating URLConnection
            URL url = new URL ( encodeUrl ? WebUtils.encodeUrl ( sourceUrl ) : sourceUrl );
            final URLConnection uc = ProxyManager.getURLConnection ( url );
            if ( timeouts != 0 )
            {
                uc.setConnectTimeout ( timeouts );
                uc.setReadTimeout ( timeouts );
            }

            if ( listener != null && listener.shouldStopDownload () )
            {
                removeFile ( dstFile );
                return false;
            }

            // Content type limitation
            if ( contentType != null )
            {
                String ct = uc.getContentType ();
                if ( !ct.contains ( contentType ) )
                {
                    removeFile ( dstFile );
                    return false;
                }
            }

            // Notifying about file size
            if ( listener != null )
            {
                listener.sizeDetermined ( uc.getContentLength () );
                if ( listener.shouldStopDownload () )
                {
                    removeFile ( dstFile );
                    return false;
                }
            }

            // Opening file stream
            final InputStream in = uc.getInputStream ();
            final FileOutputStream out = new FileOutputStream ( dstFile );
            if ( listener != null && listener.shouldStopDownload () )
            {
                out.flush ();
                out.close ();
                in.close ();
                removeFile ( dstFile );
                return false;
            }

            // Downloading content part by part
            byte[] buf = new byte[ 50 * 1024 ];
            int totalBytesRead = 0;
            int bytesRead;
            while ( ( bytesRead = in.read ( buf ) ) != -1 )
            {
                if ( listener != null )
                {
                    totalBytesRead += bytesRead;
                    listener.partDownloaded ( totalBytesRead );
                    if ( listener.shouldStopDownload () )
                    {
                        out.flush ();
                        out.close ();
                        in.close ();
                        removeFile ( dstFile );
                        return false;
                    }
                }
                out.write ( buf, 0, bytesRead );
            }
            out.flush ();
            out.close ();
            in.close ();

            return listener == null || !listener.shouldStopDownload ();
        }
        catch ( Throwable e )
        {
            return false;
        }
    }

    /**
     * Retrieves file size at the specified url address
     */

    public static int retrieveUrlSize ( String url )
    {
        try
        {
            // Creating URLConnection
            URLConnection uc = ProxyManager.getURLConnection ( new URL ( url ) );

            // Retrieving file size
            return uc.getContentLength ();
        }
        catch ( Throwable e )
        {
            return -1;
        }
    }

    /**
     * Finds all files with specified extensions
     */

    public static List<File> findViewProjectsRecursively ( String dir, List<String> extension )
    {
        return findViewProjectsRecursively ( dir, extension, true );
    }

    private static List<File> findViewProjectsRecursively ( String dir, List<String> extension,
                                                            boolean withDot )
    {
        return findViewProjectsRecursively ( dir, extension, withDot, new ArrayList<File> () );
    }

    private static List<File> findViewProjectsRecursively ( String dir, List<String> extension,
                                                            boolean withDot, List<File> projects )
    {
        File directory = new File ( dir );
        if ( directory.exists () && directory.isDirectory () )
        {
            for ( File file : directory.listFiles () )
            {
                if ( file.isFile () )
                {
                    String ext = getFileExtPart ( file.getName (), withDot );
                    if ( extension.contains ( ext ) )
                    {
                        projects.add ( file );
                    }
                }
                else
                {
                    projects.addAll (
                            findViewProjectsRecursively ( file.getAbsolutePath (), extension ) );
                }
            }
        }
        return projects;
    }
}
