/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils;

import com.alee.laf.GlobalConstants;
import com.alee.laf.StyleConstants;
import com.alee.utils.image.NinePatchInterval;
import com.alee.utils.image.NinePatchIntervalType;
import com.mortennobel.imagescaling.ResampleOp;
import sun.awt.image.ToolkitImage;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.color.ColorSpace;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * User: mgarin Date: 05.07.11 Time: 13:22
 */

public class ImageUtils
{
    public static final String SEPARATOR = StyleConstants.SEPARATOR;

    /**
     * Parses 9-patch image file into usable java Icon
     */

    public static List<NinePatchInterval> parseIntervals ( BufferedImage image,
                                                           NinePatchIntervalType intervalType )
    {
        List<NinePatchInterval> intervals = new ArrayList<NinePatchInterval> ();
        NinePatchInterval interval = null;
        boolean pixelPart;
        for ( int i = 1; i < ( intervalType.equals ( NinePatchIntervalType.horizontalStretch ) ||
                intervalType.equals ( NinePatchIntervalType.horizontalContent ) ?
                image.getWidth () : image.getHeight () ) - 1; i++ )
        {
            int rgb;
            switch ( intervalType )
            {
                case horizontalStretch:
                    rgb = image.getRGB ( i, 0 );
                    break;
                case verticalStretch:
                    rgb = image.getRGB ( 0, i );
                    break;
                case horizontalContent:
                    rgb = image.getRGB ( i, image.getHeight () - 1 );
                    break;
                case verticalContent:
                    rgb = image.getRGB ( image.getWidth () - 1, i );
                    break;
                default:
                    rgb = 0;
                    break;
            }

            pixelPart = rgb != Color.BLACK.getRGB ();
            if ( interval == null )
            {
                // Initial interval
                interval = new NinePatchInterval ( i - 1, i - 1, pixelPart );
            }
            else if ( pixelPart == interval.isPixel () )
            {
                // Enlarge interval
                interval.setEnd ( i - 1 );
            }
            else if ( pixelPart != interval.isPixel () )
            {
                // Add pixel interval only for stretch types and nonpixel for any type
                if ( intervalType.equals ( NinePatchIntervalType.horizontalStretch ) ||
                        intervalType.equals ( NinePatchIntervalType.verticalStretch ) ||
                        !interval.isPixel () )
                {
                    intervals.add ( interval );
                }
                // New interval starts
                interval = new NinePatchInterval ( i - 1, i - 1, pixelPart );
            }
        }
        if ( interval != null )
        {
            // Add pixel interval only for stretch types and nonpixel for any type
            if ( intervalType.equals ( NinePatchIntervalType.horizontalStretch ) ||
                    intervalType.equals ( NinePatchIntervalType.verticalStretch ) ||
                    !interval.isPixel () )
            {
                intervals.add ( interval );
            }
        }
        return intervals;
    }

    public static List<NinePatchInterval> parseStretchIntervals ( boolean[] filled )
    {
        List<NinePatchInterval> intervals = new ArrayList<NinePatchInterval> ();
        NinePatchInterval interval = null;
        boolean pixelPart;
        for ( int i = 0; i < filled.length; i++ )
        {
            pixelPart = !filled[ i ];
            if ( interval == null )
            {
                // Initial interval
                interval = new NinePatchInterval ( i, i, pixelPart );
            }
            else if ( pixelPart == interval.isPixel () )
            {
                // Enlarge interval
                interval.setEnd ( i );
            }
            else if ( pixelPart != interval.isPixel () )
            {
                intervals.add ( interval );

                // New interval starts
                interval = new NinePatchInterval ( i, i, pixelPart );
            }
        }
        if ( interval != null )
        {
            intervals.add ( interval );
        }
        return intervals;
    }

    /**
     * Loads image from specified source file
     */

    public static BufferedImage loadImage ( String src )
    {
        return loadImage ( new File ( src ) );
    }

    public static BufferedImage loadImage ( File file )
    {
        try
        {
            return ImageIO.read ( file );
        }
        catch ( Throwable e )
        {
            return null;
        }
    }

    /**
     * Loads image from specified resource near class
     */

    public static ImageIcon loadImage ( Class nearClass, String src )
    {
        try
        {
            return new ImageIcon ( nearClass.getResource ( src ) );
        }
        catch ( Throwable e )
        {
            return null;
        }
    }

    /**
     * Specified color icon
     */

    public static ImageIcon createColorIcon ( Color color )
    {
        return createColorIcon ( color, 16, 16 );
    }

    public static ImageIcon createColorIcon ( Color color, int width, int height )
    {
        return new ImageIcon ( createColorImage ( color, width, height ) );
    }

    public static BufferedImage createColorImage ( Color color )
    {
        return createColorImage ( color, 16, 16 );
    }

    public static BufferedImage createColorImage ( Color color, int width, int height )
    {
        BufferedImage image = new BufferedImage ( width, height, BufferedImage.TYPE_INT_ARGB );
        Graphics2D g2d = image.createGraphics ();
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
        g2d.setPaint ( Color.GRAY );
        g2d.drawRoundRect ( 0, 0, width - 1, height - 1, StyleConstants.largeRound,
                StyleConstants.round );
        g2d.setPaint ( Color.WHITE );
        g2d.drawRoundRect ( 1, 1, width - 3, height - 3, StyleConstants.round,
                StyleConstants.round );
        g2d.setPaint ( color );
        g2d.fillRoundRect ( 2, 2, width - 4, height - 4, StyleConstants.round,
                StyleConstants.round );
        g2d.dispose ();
        return image;
    }

    /**
     * Darkens specified BufferedImage
     */

    public static void darkenImage ( BufferedImage image, float darken )
    {
        Graphics2D g2d = image.createGraphics ();
        g2d.setComposite ( AlphaComposite.getInstance ( AlphaComposite.SRC_OVER, darken ) );
        g2d.setPaint ( Color.BLACK );
        g2d.fillRect ( 0, 0, image.getWidth (), image.getHeight () );
        g2d.dispose ();
    }

    /**
     * Average image color
     */

    public static Color getImageAverageColor ( ImageIcon icon )
    {
        int red = 0;
        int green = 0;
        int blue = 0;
        BufferedImage bi = getBufferedImage ( icon.getImage () );
        for ( int i = 0; i < icon.getIconWidth (); i++ )
        {
            for ( int j = 0; j < icon.getIconHeight (); j++ )
            {
                int rgb = bi.getRGB ( i, j );
                red += ( rgb >> 16 ) & 0xFF;
                green += ( rgb >> 8 ) & 0xFF;
                blue += ( rgb ) & 0xFF;
            }
        }
        int count = icon.getIconWidth () * icon.getIconHeight ();
        return new Color ( red / count, green / count, blue / count );
    }

    /**
     * Is this image format can be loaded
     */

    public static boolean isImageLoadable ( String name )
    {
        return GlobalConstants.IMAGE_FORMATS
                .contains ( FileUtils.getFileExtPart ( name, false ).toLowerCase () );
    }

    /**
     * Image preview generation
     */

    private static final Map<String, ImageIcon> thumbnailsCache = new HashMap<String, ImageIcon> ();

    public static void clearThumbnailsCache ()
    {
        synchronized ( thumbnailsCache )
        {
            for ( ImageIcon icon : thumbnailsCache.values () )
            {
                if ( icon != null )
                {
                    icon.getImage ().flush ();
                }
            }
            thumbnailsCache.clear ();
        }
    }

    public static void clearThumbnailsCache ( String src )
    {
        clearThumbnailsCache ( "", src );
    }

    public static void clearThumbnailsCache ( String prefix, String src )
    {
        synchronized ( thumbnailsCache )
        {
            if ( thumbnailsCache.size () > 0 )
            {
                List<String> toRemove = new ArrayList<String> ();
                for ( String key : thumbnailsCache.keySet () )
                {
                    if ( key.endsWith ( SEPARATOR + src ) )
                    {
                        toRemove.add ( key );
                    }
                }
                for ( String key : toRemove )
                {
                    ImageIcon icon = thumbnailsCache.remove ( key );
                    if ( icon != null )
                    {
                        icon.getImage ().flush ();
                    }
                }
                toRemove.clear ();
            }
        }
    }

    public static void clearThumbnailsCacheByPrefix ( String prefix )
    {
        synchronized ( thumbnailsCache )
        {
            if ( thumbnailsCache.size () > 0 )
            {
                List<String> toRemove = new ArrayList<String> ();
                for ( String key : thumbnailsCache.keySet () )
                {
                    if ( key.startsWith ( prefix ) )
                    {
                        toRemove.add ( key );
                    }
                }
                for ( String key : toRemove )
                {
                    ImageIcon icon = thumbnailsCache.remove ( key );
                    if ( icon != null && icon.getImage () != null )
                    {
                        icon.getImage ().flush ();
                    }
                }
                toRemove.clear ();
            }
        }
    }

    public static ImageIcon getImageThumbnailIcon ( String src )
    {
        return getImageThumbnailIcon ( src, 50 );
    }

    public static ImageIcon getImageThumbnailIcon ( String src, int size )
    {
        return getImageThumbnailIcon ( "", src, size );
    }

    public static ImageIcon getImageThumbnailIcon ( String prefix, String src, int size )
    {
        synchronized ( thumbnailsCache )
        {
            if ( src != null && !src.trim ().equals ( "" ) )
            {
                if ( thumbnailsCache.containsKey ( prefix + size + SEPARATOR + src ) )
                {
                    ImageIcon imageIcon = thumbnailsCache.get ( prefix + size + SEPARATOR + src );
                    if ( imageIcon != null )
                    {
                        return imageIcon;
                    }
                    else
                    {
                        thumbnailsCache.remove ( prefix + size + SEPARATOR + src );
                        return getImageThumbnailIcon ( prefix, src, size );
                    }
                }
                else
                {
                    boolean wasCached = isImageCached ( src );
                    ImageIcon icon = getImageIcon ( src );
                    if ( icon != null )
                    {
                        // Creating and caching thumbnail
                        ImageIcon imageIcon = createPreviewIcon ( icon.getImage (), size );
                        thumbnailsCache.put ( prefix + size + SEPARATOR + src, imageIcon );

                        if ( imageIcon != null )
                        {
                            // Saving image size
                            imageIcon.setDescription (
                                    icon.getIconWidth () + "x" + icon.getIconHeight () );

                            // Removing forcely loaded images from memory
                            if ( !wasCached )
                            {
                                clearImageCache ( src );
                            }
                        }

                        return imageIcon;
                    }
                    else
                    {
                        // Caching empty image
                        thumbnailsCache.put ( prefix + size + SEPARATOR + src, icon );
                        return icon;
                    }
                }
            }
            else
            {
                return null;
            }
        }
    }

    public static ImageIcon createPreviewIcon ( ImageIcon image, int length )
    {
        return createPreviewIcon ( image.getImage (), length );
    }

    public static ImageIcon createPreviewIcon ( Image image, int length )
    {
        return createPreviewIcon ( getBufferedImage ( image ), length );
    }

    public static ImageIcon createPreviewIcon ( BufferedImage image, int length )
    {
        BufferedImage previewImage = createPreviewImage ( image, length );
        if ( previewImage != null )
        {
            return new ImageIcon ( previewImage );
        }
        else
        {
            return new ImageIcon ();
        }
    }

    public static BufferedImage createPreviewImage ( BufferedImage image, Dimension fitTo )
    {
        return createPreviewImage ( image, fitTo.width, fitTo.height );
    }

    public static BufferedImage createPreviewImage ( BufferedImage image, int width, int height )
    {
        if ( image.getWidth () > width || image.getHeight () > height )
        {
            // Calcuating maximum preview length
            if ( height * ( ( float ) image.getWidth () / image.getHeight () ) <= width )
            {
                return createPreviewImage ( image, Math.max ( height, Math.round (
                        height * ( ( float ) image.getWidth () / image.getHeight () ) ) ) );
            }
            else
            {
                return createPreviewImage ( image, Math.max ( width, Math.round (
                        width * ( ( float ) image.getHeight () / image.getWidth () ) ) ) );
            }
        }
        else
        {
            // Image is smaller than allowed size
            return image;
        }
    }

    public static BufferedImage createPreviewImage ( BufferedImage image, int length )
    {
        if ( image == null )
        {
            return null;
        }

        int width;
        int height;
        if ( image.getWidth () <= length && image.getHeight () <= length )
        {
            return image;
            //            width = image.getWidth ();
            //            height = image.getHeight ();
        }
        else if ( image.getWidth () > image.getHeight () )
        {
            width = length;
            height = Math.round ( ( float ) length * image.getHeight () / image.getWidth () );
        }
        else if ( image.getWidth () < image.getHeight () )
        {
            height = length;
            width = Math.round ( ( float ) length * image.getWidth () / image.getHeight () );
        }
        else
        {
            width = height = length;
        }

        // Creating scaled image (can only scale down)
        // http://code.google.com/p/java-image-scaling/
        BufferedImage rescaledImage;
        if ( width >= 3 && height >= 3 )
        {
            ResampleOp resampleOp = new ResampleOp ( width, height );
            resampleOp.setNumberOfThreads ( 4 );
            rescaledImage = resampleOp.filter ( image, null );
        }
        else
        {
            rescaledImage = new BufferedImage ( Math.max ( 1, width ), Math.max ( 1, height ),
                    BufferedImage.TYPE_INT_ARGB );
            Graphics2D g2d = rescaledImage.createGraphics ();
            g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON );
            g2d.setRenderingHint ( RenderingHints.KEY_RENDERING,
                    RenderingHints.VALUE_RENDER_QUALITY );
            g2d.setRenderingHint ( RenderingHints.KEY_ALPHA_INTERPOLATION,
                    RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY );
            g2d.setRenderingHint ( RenderingHints.KEY_INTERPOLATION,
                    RenderingHints.VALUE_INTERPOLATION_BILINEAR );
            g2d.drawImage ( image, 0, 0, width, height, null );
            g2d.dispose ();
        }

        return rescaledImage;
    }

    /**
     * Image read methods
     */

    private static Map<String, ImageIcon> iconsCache = new HashMap<String, ImageIcon> ();

    public static boolean isImageCached ( String src )
    {
        return iconsCache.containsKey ( src ) && iconsCache.get ( src ) != null;
    }

    public static void setImageCache ( String src, ImageIcon imageIcon )
    {
        iconsCache.put ( src, imageIcon );
    }

    public static void clearImagesCache ()
    {
        iconsCache.clear ();
    }

    public static void clearImageCache ( String src )
    {
        if ( iconsCache.size () > 0 && iconsCache.containsKey ( src ) )
        {
            if ( iconsCache.get ( src ) != null && iconsCache.get ( src ).getImage () != null )
            {
                iconsCache.get ( src ).getImage ().flush ();
            }
            iconsCache.remove ( src );
        }
    }

    public static ImageIcon getImageIcon ( File file )
    {
        return getImageIcon ( file, true );
    }

    public static ImageIcon getImageIcon ( File file, boolean useCache )
    {
        return getImageIcon ( file.getAbsolutePath (), useCache );
    }

    public static ImageIcon getImageIcon ( String src )
    {
        return getImageIcon ( src, true );
    }

    public static ImageIcon getImageIcon ( String src, boolean useCache )
    {
        if ( src != null && !src.trim ().equals ( "" ) )
        {
            ImageIcon imageIcon;
            if ( useCache && iconsCache.containsKey ( src ) )
            {
                imageIcon = iconsCache.get ( src );
                if ( imageIcon != null )
                {
                    return imageIcon;
                }
                else
                {
                    iconsCache.remove ( src );
                    return getImageIcon ( src, useCache );
                }
            }
            else
            {
                imageIcon = createImageIcon ( src );
                if ( useCache )
                {
                    iconsCache.put ( src, imageIcon );
                }
                return imageIcon;
            }
        }
        else
        {
            return null;
        }
    }

    private static ImageIcon createImageIcon ( String src )
    {
        if ( !new File ( src ).exists () )
        {
            return new ImageIcon ();
        }
        else
        {
            try
            {
                return new ImageIcon ( ImageIO.read ( new File ( src ) ) );
            }
            catch ( Throwable e )
            {
                return new ImageIcon ();
            }
        }
    }

    /*
    * Makes a copy of BufferedImage
    */

    public static BufferedImage copy ( BufferedImage bufferedImage )
    {
        BufferedImage newImage =
                new BufferedImage ( bufferedImage.getWidth (), bufferedImage.getHeight (),
                        BufferedImage.TYPE_INT_ARGB );
        Graphics2D g2d = newImage.createGraphics ();
        g2d.drawImage ( bufferedImage, 0, 0, null );
        g2d.dispose ();
        return newImage;
    }

    /*
    * Retrieves BufferedImage from Image
    */

    public static BufferedImage getBufferedImage ( URL url )
    {
        return getBufferedImage ( new ImageIcon ( url ) );
    }

    public static BufferedImage getBufferedImage ( String iconSrc )
    {
        return getBufferedImage ( new ImageIcon ( iconSrc ) );
    }

    public static BufferedImage getBufferedImage ( ImageIcon imageIcon )
    {
        return getBufferedImage ( imageIcon.getImage () );
    }

    public static BufferedImage getBufferedImage ( Image image )
    {
        if ( image == null || image.getWidth ( null ) <= 0 || image.getHeight ( null ) <= 0 )
        {
            return null;
        }
        else if ( image instanceof BufferedImage )
        {
            return ( BufferedImage ) image;
        }
        else if ( image instanceof ToolkitImage &&
                ( ( ToolkitImage ) image ).getBufferedImage () != null )
        {
            return ( ( ToolkitImage ) image ).getBufferedImage ();
        }
        else
        {
            BufferedImage bufferedImage =
                    new BufferedImage ( image.getWidth ( null ), image.getHeight ( null ),
                            BufferedImage.TYPE_INT_ARGB );
            Graphics2D g2d = bufferedImage.createGraphics ();
            g2d.drawImage ( image, 0, 0, null );
            g2d.dispose ();
            return bufferedImage;
        }
    }

    /*
    * Scaled preview creation
    */

    private static final Map<String, ImageIcon> sizedPreviewCache =
            new HashMap<String, ImageIcon> ();

    public static ImageIcon getSizedImagePreview ( String src, int length, boolean drawBorder )
    {
        if ( sizedPreviewCache.containsKey ( length + SEPARATOR + src ) )
        {
            return sizedPreviewCache.get ( length + SEPARATOR + src );
        }
        else
        {
            ImageIcon icon = getImageThumbnailIcon ( src, length );
            ImageIcon sized = createSizedImagePreview ( icon, length, drawBorder );
            sizedPreviewCache.put ( length + SEPARATOR + src, sized );
            return sized;
        }
    }

    public static ImageIcon getSizedImagePreview ( String id, ImageIcon icon, int length,
                                                   boolean drawBorder )
    {
        if ( sizedPreviewCache.containsKey ( id ) )
        {
            return sizedPreviewCache.get ( id );
        }
        else
        {
            ImageIcon sized = createSizedImagePreview ( icon, length, drawBorder );
            sizedPreviewCache.put ( id, sized );
            return sized;
        }
    }

    public static ImageIcon createSizedImagePreview ( ImageIcon icon, int length,
                                                      boolean drawBorder )
    {
        // Addition border spacing
        //        if ( drawBorder )
        //        {
        length += 4;
        //        }

        // Creating standard size image
        BufferedImage bi = new BufferedImage ( length, length, BufferedImage.TYPE_INT_ARGB );
        Graphics2D g2d = bi.createGraphics ();
        if ( icon != null )
        {
            g2d.drawImage ( icon.getImage (), length / 2 - icon.getIconWidth () / 2,
                    length / 2 - icon.getIconHeight () / 2, null );
        }
        if ( drawBorder )
        {
            g2d.setPaint ( Color.LIGHT_GRAY );
            g2d.drawRect ( 0, 0, length - 1, length - 1 );
        }
        g2d.dispose ();

        // Creating ImageIcon
        ImageIcon imageIcon = new ImageIcon ( bi );
        imageIcon.setDescription ( icon != null ? icon.getDescription () : null );
        return imageIcon;
    }

    /*
    * Creating grayscale ImageIcon
    */

    private static Map<String, ImageIcon> grayscaleCache = new HashMap<String, ImageIcon> ();

    public static ImageIcon getGrayscaleCopy ( String id, ImageIcon imageIcon )
    {
        if ( grayscaleCache.containsKey ( id ) )
        {
            return grayscaleCache.get ( id );
        }
        else
        {
            grayscaleCache.put ( id, createGrayscaleCopy ( imageIcon ) );
            return grayscaleCache.get ( id );
        }
    }

    private static final JLabel labelForDisable = new JLabel ();

    public static ImageIcon createGrayscaleCopy ( ImageIcon imageIcon )
    {
        try
        {
            return ( ImageIcon ) UIManager.getLookAndFeel ()
                    .getDisabledIcon ( labelForDisable, imageIcon );
        }
        catch ( Throwable e )
        {
            BufferedImage img = getBufferedImage ( imageIcon.getImage () );

            ColorConvertOp colorConvert =
                    new ColorConvertOp ( ColorSpace.getInstance ( ColorSpace.CS_GRAY ), null );
            colorConvert.filter ( img, img );

            return new ImageIcon ( img );
        }
    }

    /*
    * Creating partially transparent ImageIcon
    */

    private static Map<String, ImageIcon> trasparentCache = new HashMap<String, ImageIcon> ();

    public static ImageIcon getTransparentCopy ( String id, ImageIcon imageIcon, float trasparency )
    {
        if ( trasparentCache.containsKey ( id ) )
        {
            return trasparentCache.get ( id );
        }
        else
        {
            trasparentCache.put ( id, createTransparentCopy ( imageIcon, trasparency ) );
            return trasparentCache.get ( id );
        }
    }

    public static ImageIcon createTransparentCopy ( ImageIcon imageIcon, float trasparency )
    {
        BufferedImage bi =
                new BufferedImage ( imageIcon.getIconWidth (), imageIcon.getIconHeight (),
                        BufferedImage.TYPE_INT_ARGB );

        Graphics2D g2d = bi.createGraphics ();
        g2d.setComposite ( AlphaComposite.getInstance ( AlphaComposite.SRC_OVER, trasparency ) );
        g2d.drawImage ( imageIcon.getImage (), 0, 0, null );
        g2d.dispose ();

        return new ImageIcon ( bi );
    }

    /**
     * Creating bordered pretty image
     */

    public static BufferedImage createPrettyImage ( BufferedImage bufferedImage, int shadeWidth,
                                                    int round )
    {
        int width = bufferedImage.getWidth ();
        int height = bufferedImage.getHeight ();

        BufferedImage bi = new BufferedImage ( width + shadeWidth * 2, height + shadeWidth * 2,
                BufferedImage.TYPE_INT_ARGB );
        Graphics2D g2d = bi.createGraphics ();
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
        g2d.setRenderingHint ( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );

        RoundRectangle2D.Double border =
                new RoundRectangle2D.Double ( shadeWidth, shadeWidth, width, height, round, round );

        Shape old = g2d.getClip ();
        g2d.setClip ( border );
        g2d.drawImage ( bufferedImage, shadeWidth, shadeWidth, null );
        g2d.setClip ( old );

        LafUtils.drawShade ( g2d, border, StyleConstants.shadeColor, shadeWidth );

        g2d.setPaint ( new LinearGradientPaint ( 0, shadeWidth, 0, height - shadeWidth,
                new float[]{ 0f, 0.5f, 1f },
                new Color[]{ new Color ( 125, 125, 125, 48 ), new Color ( 125, 125, 125, 0 ),
                        new Color ( 125, 125, 125, 48 ) } ) );
        g2d.fill ( border );

        g2d.setColor ( Color.GRAY );
        g2d.draw ( border );

        g2d.dispose ();

        return bi;
    }
}
