/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils;

import com.alee.laf.StyleConstants;
import com.alee.utils.laf.FocusType;

import javax.swing.*;
import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;

/**
 * User: mgarin Date: 27.04.11 Time: 17:41
 */

public class LafUtils
{
    /*
    * Draws alpha-background
    */

    public static void drawAlphaBackground ( Graphics2D g2d, Rectangle rectangle )
    {
        drawAlphaBackground ( g2d, rectangle.x, rectangle.y, rectangle.width, rectangle.height );
    }

    public static void drawAlphaBackground ( Graphics2D g2d, int x, int y, int width, int height )
    {
        drawAlphaBackground ( g2d, x, y, width, height, StyleConstants.ALPHA_RECT_SIZE );
    }

    public static void drawAlphaBackground ( Graphics2D g2d, Rectangle rectangle, int size )
    {
        drawAlphaBackground ( g2d, rectangle.x, rectangle.y, rectangle.width, rectangle.height,
                size );
    }

    public static void drawAlphaBackground ( Graphics2D g2d, int x, int y, int width, int height,
                                             int size )
    {
        drawAlphaBackground ( g2d, x, y, width, height, size, StyleConstants.LIGHT_ALPHA,
                StyleConstants.DARK_ALPHA );
    }

    public static void drawAlphaBackground ( Graphics2D g2d, Rectangle rectangle, int size,
                                             Color light, Color dark )
    {
        drawAlphaBackground ( g2d, rectangle.x, rectangle.y, rectangle.width, rectangle.height,
                size, light, dark );
    }

    public static void drawAlphaBackground ( Graphics2D g2d, int x, int y, int width, int height,
                                             int size, Color light, Color dark )
    {
        int xAmount = width / size + 1;
        int yAmount = height / size + 1;
        boolean lightColor;
        for ( int i = 0; i < xAmount; i++ )
        {
            for ( int j = 0; j < yAmount; j++ )
            {
                lightColor = ( i + j ) % 2 == 0;
                Color color = lightColor ? light : dark;
                if ( color != null )
                {
                    g2d.setPaint ( color );
                    int w = ( x + i * size + size > x + width ) ? ( width - i * size ) : size;
                    int h = ( y + j * size + size > y + height ) ? ( height - j * size ) : size;
                    g2d.fillRect ( x + i * size, y + j * size, w, h );
                }
            }
        }
    }

    /*
    * Setting clip Shape by taking old clip Shape into account
    */

    public static Shape intersectClip ( Graphics g, Shape clip )
    {
        return intersectClip ( g, clip, true );
    }

    public static Shape intersectClip ( Graphics g, Shape clip, boolean shouldSetup )
    {
        if ( shouldSetup && clip != null )
        {
            Shape oldClip = g.getClip ();

            Area finalClip = new Area ( clip );
            finalClip.intersect ( new Area ( oldClip ) );
            g.setClip ( finalClip );

            return oldClip;
        }
        else
        {
            return null;
        }
    }

    public static Shape subtractClip ( Graphics g, Shape clip )
    {
        return subtractClip ( g, clip, true );
    }

    public static Shape subtractClip ( Graphics g, Shape clip, boolean shouldSetup )
    {
        if ( shouldSetup && clip != null )
        {
            Shape oldClip = g.getClip ();

            Area finalClip = new Area ( oldClip );
            finalClip.subtract ( new Area ( clip ) );
            g.setClip ( finalClip );

            return oldClip;
        }
        else
        {
            return null;
        }
    }

    public static void restoreClip ( Graphics g, Shape clip )
    {
        restoreClip ( g, clip, true );
    }

    public static void restoreClip ( Graphics g, Shape clip, boolean shouldRestore )
    {
        if ( shouldRestore && clip != null )
        {
            g.setClip ( clip );
        }
    }

    /*
    * Setting new stroke
    */

    public static Stroke setupStroke ( Graphics2D g2d, Stroke stroke )
    {
        Stroke old = g2d.getStroke ();
        g2d.setStroke ( stroke );
        return old;
    }

    public static void restoreStroke ( Graphics2D g2d, Stroke old )
    {
        g2d.setStroke ( old );
    }

    /*
    * Setting image quality on
    */

    public static Object setupImageQuality ( Graphics g )
    {
        return setupImageQuality ( ( Graphics2D ) g );
    }

    public static Object setupImageQuality ( Graphics2D g2d )
    {
        Object old = g2d.getRenderingHint ( RenderingHints.KEY_INTERPOLATION );
        g2d.setRenderingHint ( RenderingHints.KEY_INTERPOLATION,
                RenderingHints.VALUE_INTERPOLATION_BILINEAR );
        return old;
    }

    public static void restoreImageQuality ( Graphics g, Object old )
    {
        restoreImageQuality ( ( Graphics2D ) g, old );
    }

    public static void restoreImageQuality ( Graphics2D g2d, Object old )
    {
        g2d.setRenderingHint ( RenderingHints.KEY_INTERPOLATION,
                old != null ? old : RenderingHints.VALUE_INTERPOLATION_BILINEAR );
    }

    /*
    * Setting antialias on
    */

    public static Object setupAntialias ( Graphics g )
    {
        return setupAntialias ( ( Graphics2D ) g );
    }

    public static Object setupAntialias ( Graphics2D g2d )
    {
        Object old = g2d.getRenderingHint ( RenderingHints.KEY_ANTIALIASING );
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
        return old;
    }

    public static void restoreAntialias ( Graphics g, Object old )
    {
        restoreAntialias ( ( Graphics2D ) g, old );
    }

    public static void restoreAntialias ( Graphics2D g2d, Object old )
    {
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, old );
    }

    /*
    * Setting AlphaComposite by taking old AlphaComposite settings into account
    */

    public static Composite setupAlphaComposite ( Graphics2D g2d, float alpha )
    {
        return setupAlphaComposite ( g2d, alpha, true );
    }

    public static Composite setupAlphaComposite ( Graphics2D g2d, float alpha, boolean shouldSetup )
    {
        return setupAlphaComposite ( g2d, g2d.getComposite (), alpha, shouldSetup );
    }

    public static Composite setupAlphaComposite ( Graphics2D g2d, Composite composeWith,
                                                  float alpha )
    {
        return setupAlphaComposite ( g2d, composeWith, alpha, true );
    }

    public static Composite setupAlphaComposite ( Graphics2D g2d, Composite composeWith,
                                                  float alpha, boolean shouldSetup )
    {
        Composite comp = g2d.getComposite ();
        if ( !shouldSetup )
        {
            return comp;
        }

        // Determining old composite alpha
        float currentComposite = 1f;
        if ( composeWith != null && composeWith instanceof AlphaComposite )
        {
            currentComposite = ( ( AlphaComposite ) composeWith ).getAlpha ();
        }

        // Creating new composite
        AlphaComposite newComposite =
                AlphaComposite.getInstance ( AlphaComposite.SRC_OVER, currentComposite * alpha );
        g2d.setComposite ( newComposite );

        return comp;
    }

    public static void restoreComposite ( Graphics2D g2d, Composite composite )
    {
        g2d.setComposite ( composite );
    }

    public static void restoreComposite ( Graphics2D g2d, Composite composite,
                                          boolean shouldRestore )
    {
        if ( shouldRestore )
        {
            g2d.setComposite ( composite );
        }
    }

    /*
    * Determines real text size
    */

    public static Rectangle getTextBounds ( String text, Graphics g, Font font )
    {
        return getTextBounds ( text, ( Graphics2D ) g, font );
    }

    public static Rectangle getTextBounds ( String text, Graphics2D g2d, Font font )
    {
        FontRenderContext renderContext = g2d.getFontRenderContext ();
        GlyphVector glyphVector = font.createGlyphVector ( renderContext, text );
        return glyphVector.getVisualBounds ().getBounds ();
    }

    /*
    * Paints web styled border within the component with shadow and background if needed
    */

    public static Shape drawWebBorder ( Graphics2D g2d, JComponent component )
    {
        return drawWebBorder ( g2d, component, StyleConstants.shadeColor, StyleConstants.shadeWidth,
                StyleConstants.smallRound );
    }

    public static Shape drawWebBorder ( Graphics2D g2d, JComponent component, Color shadeColor,
                                        int shadeWidth, int round )
    {
        return drawWebBorder ( g2d, component, shadeColor, shadeWidth, round, true );
    }

    public static Shape drawWebBorder ( Graphics2D g2d, JComponent component, Color shadeColor,
                                        int shadeWidth, int round, boolean fillBackground )
    {
        return drawWebBorder ( g2d, component, shadeColor, shadeWidth, round, fillBackground,
                false );
    }

    public static Shape drawWebBorder ( Graphics2D g2d, JComponent component, Color shadeColor,
                                        int shadeWidth, int round, boolean fillBackground,
                                        boolean webColored )
    {
        return drawWebBorder ( g2d, component, shadeColor, shadeWidth, round, fillBackground,
                webColored, Color.GRAY, Color.LIGHT_GRAY );
    }

    public static Shape drawWebBorder ( Graphics2D g2d, JComponent component, Color shadeColor,
                                        int shadeWidth, int round, boolean fillBackground,
                                        boolean webColored, Color border )
    {
        return drawWebBorder ( g2d, component, shadeColor, shadeWidth, round, fillBackground,
                webColored, border, border );
    }

    public static Shape drawWebBorder ( Graphics2D g2d, JComponent component, Color shadeColor,
                                        int shadeWidth, int round, boolean fillBackground,
                                        boolean webColored, Color border, Color disabledBorder )
    {
        return drawWebBorder ( g2d, component, shadeColor, shadeWidth, round, fillBackground,
                webColored, border, disabledBorder, 1f );
    }

    public static Shape drawWebBorder ( Graphics2D g2d, JComponent component, Color shadeColor,
                                        int shadeWidth, int round, boolean fillBackground,
                                        boolean webColored, Color border, Color disabledBorder,
                                        float opacity )
    {
        if ( opacity <= 0f || opacity > 1f )
        {
            return null;
        }

        Object aa = setupAntialias ( g2d );

        // Border shape
        Shape bs = getWebBorderShape ( component, shadeWidth, round );

        // Transparency
        Composite composite = g2d.getComposite ();
        if ( opacity < 1f )
        {
            g2d.setComposite ( AlphaComposite.getInstance ( AlphaComposite.SRC_OVER, opacity ) );
        }

        // Outer shadow
        if ( component.isEnabled () )
        {
            LafUtils.drawShade ( g2d, bs, shadeColor, shadeWidth );
        }

        // Background
        if ( fillBackground )
        {
            if ( webColored )
            {
                g2d.setPaint ( new GradientPaint ( 0, shadeWidth, StyleConstants.topBgColor, 0,
                        component.getHeight () - shadeWidth, StyleConstants.bottomBgColor ) );
                g2d.fill ( bs );
            }
            else
            {
                g2d.setPaint ( component.getBackground () );
                g2d.fill ( bs );
            }
        }

        // Border
        if ( border != null )
        {
            g2d.setPaint ( component.isEnabled () ? border : disabledBorder );
            g2d.draw ( bs );
        }

        if ( opacity < 1f )
        {
            g2d.setComposite ( composite );
        }

        restoreAntialias ( g2d, aa );

        return bs;
    }

    public static Shape getWebBorderShape ( JComponent component )
    {
        return getWebBorderShape ( component, StyleConstants.shadeWidth,
                StyleConstants.smallRound );
    }

    public static Shape getWebBorderShape ( JComponent component, int shadeWidth, int round )
    {
        if ( round > 0 )
        {
            return new RoundRectangle2D.Double ( shadeWidth, shadeWidth,
                    component.getWidth () - shadeWidth * 2 - 1,
                    component.getHeight () - shadeWidth * 2 - 1, round * 2, round * 2 );
        }
        else
        {
            return new Rectangle2D.Double ( shadeWidth, shadeWidth,
                    component.getWidth () - shadeWidth * 2 - 1,
                    component.getHeight () - shadeWidth * 2 - 1 );
        }
    }

    /*
    * Paints custom shaped web styled border within the component with shadow and background if needed
    */

    public static void drawCustomWebBorder ( Graphics2D g2d, JComponent component,
                                             Shape borderShape, Color shadeColor, int shadeWidth,
                                             boolean fillBackground, boolean webColored )
    {
        drawCustomWebBorder ( g2d, component, borderShape, shadeColor, shadeWidth, fillBackground,
                webColored, Color.GRAY, Color.LIGHT_GRAY );
    }

    public static void drawCustomWebBorder ( Graphics2D g2d, JComponent component,
                                             Shape borderShape, Color shadeColor, int shadeWidth,
                                             boolean fillBackground, boolean webColored,
                                             Color border, Color disabledBorder )
    {
        Object aa = setupAntialias ( g2d );

        // Outer shadow
        if ( component.isEnabled () )
        {
            LafUtils.drawShade ( g2d, borderShape, shadeColor, shadeWidth );
        }

        // Background
        if ( fillBackground )
        {
            if ( webColored )
            {
                Rectangle shapeBounds = borderShape.getBounds ();
                g2d.setPaint ( new GradientPaint ( 0, shapeBounds.y, StyleConstants.topBgColor, 0,
                        shapeBounds.y + shapeBounds.height, StyleConstants.bottomBgColor ) );
                g2d.fill ( borderShape );
            }
            else
            {
                g2d.setPaint ( component.getBackground () );
                g2d.fill ( borderShape );
            }
        }

        // Border
        if ( border != null )
        {
            g2d.setPaint ( component.isEnabled () ? border : disabledBorder );
            g2d.draw ( borderShape );
        }

        restoreAntialias ( g2d, aa );
    }

    /*
    * Paints web styled focus within the component
    */

    public static boolean drawWebFocus ( Graphics2D g2d, JComponent component, FocusType focusType,
                                         int shadeWidth, int round )
    {
        return drawWebFocus ( g2d, component, focusType, shadeWidth, round, null );
    }

    public static boolean drawWebFocus ( Graphics2D g2d, JComponent component, FocusType focusType,
                                         int shadeWidth, int round, Boolean mouseover )
    {
        return drawWebFocus ( g2d, component, focusType, shadeWidth, round, mouseover, null );
    }

    public static boolean drawWebFocus ( Graphics2D g2d, JComponent component, FocusType focusType,
                                         int shadeWidth, int round, Boolean mouseover,
                                         Boolean hasFocus )
    {
        return drawWebFocus ( g2d, component, focusType, shadeWidth, round, mouseover, hasFocus,
                focusType.equals ( FocusType.componentFocus ) ? StyleConstants.focusColor :
                        StyleConstants.fieldFocusColor );
    }

    public static boolean drawWebFocus ( Graphics2D g2d, JComponent component, FocusType focusType,
                                         int shadeWidth, int round, Boolean mouseover,
                                         Boolean hasFocus, Color color )
    {
        return drawWebFocus ( g2d, component, focusType, shadeWidth, round, mouseover, hasFocus,
                color, focusType.equals ( FocusType.componentFocus ) ? StyleConstants.focusStroke :
                StyleConstants.fieldFocusStroke );
    }

    public static boolean drawWebFocus ( Graphics2D g2d, JComponent component, FocusType focusType,
                                         int shadeWidth, int round, Boolean mouseover,
                                         Boolean hasFocus, Color color, Stroke stroke )
    {
        hasFocus = hasFocus != null ? hasFocus : component.hasFocus () && component.isEnabled ();
        if ( hasFocus && focusType.equals ( FocusType.componentFocus ) )
        {
            Object aa = setupAntialias ( g2d );
            Stroke os = setupStroke ( g2d, stroke );

            g2d.setPaint ( color );
            g2d.draw ( getWebFocusShape ( component, focusType, shadeWidth, round ) );

            restoreStroke ( g2d, os );
            restoreAntialias ( g2d, aa );

            return true;
        }
        else if ( focusType.equals ( FocusType.fieldFocus ) &&
                ( hasFocus || mouseover != null && mouseover ) )
        {
            Object aa = setupAntialias ( g2d );
            Stroke os = setupStroke ( g2d, stroke );

            //            g2d.setPaint ( hasFocus ? StyleConstants.fieldFocusColor :
            //                    StyleConstants.transparentFieldFocusColor );
            g2d.setPaint ( color );
            g2d.draw ( getWebFocusShape ( component, focusType, shadeWidth, round ) );

            restoreStroke ( g2d, os );
            restoreAntialias ( g2d, aa );

            return true;
        }
        else
        {
            return false;
        }
    }

    public static Shape getWebFocusShape ( JComponent component, FocusType focusType,
                                           int shadeWidth, int round )
    {
        // Focus side spacing
        int spacing = focusType.equals ( FocusType.componentFocus ) ? 2 : 0;

        // Corners rounding
        round = focusType.equals ( FocusType.componentFocus ) ? Math.max ( 0, round - 2 ) : round;

        // Final focus shape
        if ( round > 0 )
        {
            return new RoundRectangle2D.Double ( shadeWidth + spacing, shadeWidth + spacing,
                    component.getWidth () - shadeWidth * 2 - spacing * 2 - 1,
                    component.getHeight () - shadeWidth * 2 - spacing * 2 - 1, round * 2,
                    round * 2 );
        }
        else
        {
            return new Rectangle2D.Double ( shadeWidth + spacing, shadeWidth + spacing,
                    component.getWidth () - shadeWidth * 2 - spacing * 2 - 1,
                    component.getHeight () - shadeWidth * 2 - spacing * 2 - 1 );
        }
    }

    /*
    * Draws custom shaped web styled focus within the component
    */

    public static void drawCustonWebFocus ( Graphics2D g2d, JComponent component,
                                            FocusType focusType, Shape shape )
    {
        drawCustonWebFocus ( g2d, component, focusType, shape, null );
    }

    public static void drawCustonWebFocus ( Graphics2D g2d, JComponent component,
                                            FocusType focusType, Shape shape, Boolean mouseover )
    {
        drawCustonWebFocus ( g2d, component, focusType, shape, mouseover, null );
    }

    public static void drawCustonWebFocus ( Graphics2D g2d, JComponent component,
                                            FocusType focusType, Shape shape, Boolean mouseover,
                                            Boolean hasFocus )
    {
        hasFocus = hasFocus != null ? hasFocus : component.hasFocus () && component.isEnabled ();
        if ( hasFocus && focusType.equals ( FocusType.componentFocus ) )
        {
            Object aa = setupAntialias ( g2d );
            Stroke os = setupStroke ( g2d, StyleConstants.focusStroke );

            g2d.setPaint ( StyleConstants.focusColor );
            g2d.draw ( shape );

            restoreStroke ( g2d, os );
            restoreAntialias ( g2d, aa );
        }
        else if ( focusType.equals ( FocusType.fieldFocus ) &&
                ( hasFocus || mouseover != null && mouseover ) )
        {
            Object aa = setupAntialias ( g2d );
            Stroke os = setupStroke ( g2d, StyleConstants.fieldFocusStroke );

            g2d.setPaint ( hasFocus ? StyleConstants.fieldFocusColor :
                    StyleConstants.transparentFieldFocusColor );
            g2d.draw ( shape );

            restoreStroke ( g2d, os );
            restoreAntialias ( g2d, aa );
        }
    }

    /*
    * Draws web styled shade using specified shape
    */

    public static void drawShade ( Graphics2D g2d, Shape shape, Color shadeColor, int width )
    {
        drawShade ( g2d, shape, shadeColor, width, null, true );
    }

    public static void drawShade ( Graphics2D g2d, Shape shape, Color shadeColor, int width,
                                   Shape clip )
    {
        drawShade ( g2d, shape, shadeColor, width, clip, true );
    }

    public static void drawShade ( Graphics2D g2d, Shape shape, Color shadeColor, int width,
                                   boolean round )
    {
        drawShade ( g2d, shape, shadeColor, width, null, round );
    }

    public static void drawShade ( Graphics2D g2d, Shape shape, Color shadeColor, int width,
                                   Shape clip, boolean round )
    {
        Shape oldClip = intersectClip ( g2d, clip );

        Composite comp = g2d.getComposite ();
        float currentComposite = 1f;
        if ( comp instanceof AlphaComposite )
        {
            currentComposite = ( ( AlphaComposite ) comp ).getAlpha ();
        }

        Stroke old = g2d.getStroke ();
        width = width * 2;
        for ( int i = width; i >= 2; i -= 2 )
        {
            float opacity = ( float ) ( width - i ) / ( width - 1 );
            g2d.setColor ( shadeColor );
            g2d.setComposite ( AlphaComposite
                    .getInstance ( AlphaComposite.SRC_OVER, opacity * currentComposite ) );
            g2d.setStroke (
                    new BasicStroke ( i, round ? BasicStroke.CAP_ROUND : BasicStroke.CAP_BUTT,
                            BasicStroke.JOIN_ROUND ) );
            g2d.draw ( shape );
        }
        g2d.setStroke ( old );
        g2d.setComposite ( comp );

        restoreClip ( g2d, oldClip );
    }

    /**
     * Draws web styled selection using shapes operations. This method is pretty slow and should not
     * be used for multiply selections presentantion
     */

    private static int halfButton = 4;
    private static int halfLine = 1;

    public static void drawWebSelection ( Graphics2D g2d, Color color, int x, int y, int width,
                                          int height, boolean resizableLR, boolean resizableUD,
                                          boolean drawConnectors )
    {
        drawWebSelection ( g2d, color, new Rectangle ( x, y, width, height ), resizableLR,
                resizableUD, drawConnectors );
    }

    public static void drawWebSelection ( Graphics2D g2d, Color color, Rectangle selection,
                                          boolean resizableLR, boolean resizableUD,
                                          boolean drawConnectors )
    {
        selection = GeometryUtils.verifyRect ( selection );

        Object aa = setupAntialias ( g2d );

        Area buttonsShape = new Area ();

        // Top
        if ( resizableUD )
        {
            if ( resizableLR )
            {
                buttonsShape.add ( new Area (
                        new Ellipse2D.Double ( selection.x - halfButton, selection.y - halfButton,
                                halfButton * 2, halfButton * 2 ) ) );
                buttonsShape.add ( new Area (
                        new Ellipse2D.Double ( selection.x + selection.width - halfButton,
                                selection.y - halfButton, halfButton * 2, halfButton * 2 ) ) );
            }
            buttonsShape.add ( new Area (
                    new Ellipse2D.Double ( selection.x + selection.width / 2 - halfButton,
                            selection.y - halfButton, halfButton * 2, halfButton * 2 ) ) );
        }

        // Middle
        if ( resizableLR )
        {
            buttonsShape.add ( new Area ( new Ellipse2D.Double ( selection.x - halfButton,
                    selection.y + selection.height / 2 - halfButton, halfButton * 2,
                    halfButton * 2 ) ) );
            buttonsShape.add ( new Area (
                    new Ellipse2D.Double ( selection.x + selection.width - halfButton,
                            selection.y + selection.height / 2 - halfButton, halfButton * 2,
                            halfButton * 2 ) ) );
        }

        // Bottom
        if ( resizableUD )
        {
            if ( resizableLR )
            {
                buttonsShape.add ( new Area ( new Ellipse2D.Double ( selection.x - halfButton,
                        selection.y + selection.height - halfButton, halfButton * 2,
                        halfButton * 2 ) ) );
                buttonsShape.add ( new Area (
                        new Ellipse2D.Double ( selection.x + selection.width - halfButton,
                                selection.y + selection.height - halfButton, halfButton * 2,
                                halfButton * 2 ) ) );
            }
            buttonsShape.add ( new Area (
                    new Ellipse2D.Double ( selection.x + selection.width / 2 - halfButton,
                            selection.y + selection.height - halfButton, halfButton * 2,
                            halfButton * 2 ) ) );
        }

        // Button connectors
        if ( drawConnectors )
        {
            Area selectionShape = new Area (
                    new RoundRectangle2D.Double ( selection.x - halfLine, selection.y - halfLine,
                            selection.width + ( halfLine ) * 2, selection.height + ( halfLine ) * 2,
                            5, 5 ) );
            selectionShape.subtract ( new Area (
                    new RoundRectangle2D.Double ( selection.x + halfLine, selection.y + halfLine,
                            selection.width - halfLine * 2, selection.height - halfLine * 2, 3,
                            3 ) ) );
            buttonsShape.add ( selectionShape );
        }

        // Shade
        LafUtils.drawShade ( g2d, buttonsShape, Color.GRAY, 3 );

        // Border
        g2d.setPaint ( Color.GRAY );
        g2d.draw ( buttonsShape );

        // Background
        g2d.setPaint ( color );
        g2d.fill ( buttonsShape );

        restoreAntialias ( g2d, aa );
    }

    /**
     * Draws web styled selection using predefined images set. This method is much faster than the
     * one before but has less settings
     */

    private static final ImageIcon top_left_button =
            new ImageIcon ( LafUtils.class.getResource ( "icons/selection/top_left_button.png" ) );
    private static final ImageIcon top_button =
            new ImageIcon ( LafUtils.class.getResource ( "icons/selection/top_button.png" ) );
    private static final ImageIcon top_right_button =
            new ImageIcon ( LafUtils.class.getResource ( "icons/selection/top_right_button.png" ) );

    private static final ImageIcon left_button =
            new ImageIcon ( LafUtils.class.getResource ( "icons/selection/left_button.png" ) );
    private static final ImageIcon right_button =
            new ImageIcon ( LafUtils.class.getResource ( "icons/selection/right_button.png" ) );

    private static final ImageIcon bottom_left_button = new ImageIcon (
            LafUtils.class.getResource ( "icons/selection/bottom_left_button.png" ) );
    private static final ImageIcon bottom_button =
            new ImageIcon ( LafUtils.class.getResource ( "icons/selection/bottom_button.png" ) );
    private static final ImageIcon bottom_right_button = new ImageIcon (
            LafUtils.class.getResource ( "icons/selection/bottom_right_button.png" ) );

    private static final ImageIcon top =
            new ImageIcon ( LafUtils.class.getResource ( "icons/selection/top.png" ) );
    private static final ImageIcon left =
            new ImageIcon ( LafUtils.class.getResource ( "icons/selection/left.png" ) );
    private static final ImageIcon bottom =
            new ImageIcon ( LafUtils.class.getResource ( "icons/selection/bottom.png" ) );
    private static final ImageIcon right =
            new ImageIcon ( LafUtils.class.getResource ( "icons/selection/right.png" ) );

    private static final ImageIcon button =
            new ImageIcon ( LafUtils.class.getResource ( "icons/selection/button.png" ) );

    public static void drawWebIconedSelection ( Graphics2D g2d, Rectangle selection,
                                                boolean resizableLR, boolean resizableUD,
                                                boolean drawConnectors )
    {
        drawWebIconedSelection ( g2d, selection, resizableLR, resizableUD, drawConnectors, 0 );
    }

    public static void drawWebIconedSelection ( Graphics2D g2d, Rectangle selection,
                                                boolean resizableLR, boolean resizableUD,
                                                boolean drawConnectors, int diff )
    {
        selection = GeometryUtils.verifyRect ( selection );

        if ( drawConnectors )
        {
            // Top filler
            g2d.drawImage ( top.getImage (), selection.x,
                    selection.y - top.getIconHeight () / 2 - diff, selection.width,
                    top.getIconHeight (), null );
            // Bottom filler
            g2d.drawImage ( bottom.getImage (), selection.x,
                    selection.y + selection.height - bottom.getIconHeight () / 2 + diff,
                    selection.width, bottom.getIconHeight (), null );
            // Left filler
            g2d.drawImage ( left.getImage (), selection.x - left.getIconWidth () / 2 - diff,
                    selection.y, left.getIconWidth (), selection.height, null );
            // Right filler
            g2d.drawImage ( right.getImage (),
                    selection.x + selection.width - right.getIconWidth () / 2 + diff, selection.y,
                    right.getIconWidth (), selection.height, null );

            // Top buttons
            if ( resizableUD )
            {
                if ( resizableLR )
                {
                    g2d.drawImage ( top_left_button.getImage (),
                            selection.x - top_left_button.getIconWidth () / 2 - diff,
                            selection.y - top_left_button.getIconHeight () / 2 - diff, null );
                    g2d.drawImage ( top_right_button.getImage (),
                            selection.x + selection.width - top_right_button.getIconWidth () / 2 +
                                    diff,
                            selection.y - top_right_button.getIconHeight () / 2 - diff, null );
                }
                g2d.drawImage ( top_button.getImage (),
                        selection.x + selection.width / 2 - top_button.getIconWidth () / 2,
                        selection.y - top_button.getIconHeight () / 2 + 1 - diff, null );
            }
            // Left-right buttons
            if ( resizableLR )
            {
                g2d.drawImage ( left_button.getImage (),
                        selection.x - left_button.getIconWidth () / 2 + 1 - diff,
                        selection.y + selection.height / 2 - left_button.getIconHeight () / 2,
                        null );
                g2d.drawImage ( right_button.getImage (),
                        selection.x + selection.width - right_button.getIconWidth () / 2 - 1 + diff,
                        selection.y + selection.height / 2 - right_button.getIconHeight () / 2,
                        null );
            }
            // Bottom buttons
            if ( resizableUD )
            {
                if ( resizableLR )
                {
                    g2d.drawImage ( bottom_left_button.getImage (),
                            selection.x - bottom_left_button.getIconWidth () / 2 - diff,
                            selection.y + selection.height -
                                    bottom_left_button.getIconHeight () / 2 + diff, null );
                    g2d.drawImage ( bottom_right_button.getImage (), selection.x + selection.width -
                            bottom_right_button.getIconWidth () / 2 + diff,
                            selection.y + selection.height -
                                    bottom_right_button.getIconHeight () / 2 + diff, null );
                }
                g2d.drawImage ( bottom_button.getImage (),
                        selection.x + selection.width / 2 - bottom_button.getIconWidth () / 2,
                        selection.y + selection.height - bottom_button.getIconHeight () / 2 - 1 +
                                diff, null );
            }
        }
        else
        {
            if ( resizableUD )
            {
                if ( resizableLR )
                {
                    g2d.drawImage ( button.getImage (), selection.x - button.getIconWidth () / 2,
                            selection.y - button.getIconHeight () / 2, null );
                    g2d.drawImage ( button.getImage (),
                            selection.x + selection.width - button.getIconWidth () / 2 - 1,
                            selection.y - button.getIconHeight () / 2, null );
                    g2d.drawImage ( button.getImage (), selection.x - button.getIconWidth () / 2,
                            selection.y + selection.height - button.getIconHeight () / 2 - 1,
                            null );
                    g2d.drawImage ( button.getImage (),
                            selection.x + selection.width - button.getIconWidth () / 2 - 1,
                            selection.y + selection.height - button.getIconHeight () / 2 - 1,
                            null );
                }
                g2d.drawImage ( button.getImage (),
                        selection.x + selection.width / 2 - button.getIconWidth () / 2,
                        selection.y - button.getIconHeight () / 2, null );
                g2d.drawImage ( button.getImage (),
                        selection.x + selection.width / 2 - button.getIconWidth () / 2,
                        selection.y + selection.height - button.getIconHeight () / 2 - 1, null );
            }
            if ( resizableLR )
            {
                g2d.drawImage ( button.getImage (), selection.x - button.getIconWidth () / 2,
                        selection.y + selection.height / 2 - button.getIconHeight () / 2, null );
                g2d.drawImage ( button.getImage (),
                        selection.x + selection.width - button.getIconWidth () / 2 - 1,
                        selection.y + selection.height / 2 - button.getIconHeight () / 2, null );
            }
        }
    }

    /**
     * Draws etched shape with specified background colors
     */

    public static void drawEtchedShape ( Graphics2D g2d, BufferedImage topBg,
                                         BufferedImage bottomBg, Shape fullShape, Shape bevelShape )
    {
        Object aa = setupAntialias ( g2d );

        Rectangle bounds = fullShape.getBounds ();

        g2d.setPaint ( new TexturePaint ( topBg, new Rectangle ( bounds.getLocation (),
                new Dimension ( topBg.getWidth (), topBg.getHeight () ) ) ) );
        g2d.fill ( fullShape );

        Shape oldClip = g2d.getClip ();
        Area newClip = new Area ( oldClip );
        newClip.intersect ( new Area ( bevelShape ) );

        g2d.setClip ( newClip );
        g2d.setPaint ( new TexturePaint ( bottomBg, new Rectangle ( bounds.getLocation (),
                new Dimension ( bottomBg.getWidth (), bottomBg.getHeight () ) ) ) );
        g2d.fill ( bevelShape );

        LafUtils.drawShade ( g2d, bevelShape, Color.BLACK, 4 );

        g2d.setClip ( oldClip );

        g2d.setPaint ( Color.DARK_GRAY );
        g2d.draw ( bevelShape );

        restoreAntialias ( g2d, aa );
    }
}
