/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.alee.utils.image;

import com.alee.utils.ImageUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.net.URL;
import java.util.List;

/**
 * User: mgarin Date: 13.12.11 Time: 15:37
 */

public class NinePatchIcon implements Icon
{
    private BufferedImage image;
    private Component component;

    private List<NinePatchInterval> horizontalStretch;
    private List<NinePatchInterval> verticalStretch;

    private Insets margin;

    public NinePatchIcon ( URL url )
    {
        this ( url, null );
    }

    public NinePatchIcon ( URL url, Component component )
    {
        this ( ImageUtils.getBufferedImage ( url ), component );
    }

    public NinePatchIcon ( String iconSrc )
    {
        this ( iconSrc, null );
    }

    public NinePatchIcon ( String iconSrc, Component component )
    {
        this ( ImageUtils.getBufferedImage ( iconSrc ), component );
    }

    public NinePatchIcon ( ImageIcon imageIcon )
    {
        this ( imageIcon, null );
    }

    public NinePatchIcon ( ImageIcon imageIcon, Component component )
    {
        this ( ImageUtils.getBufferedImage ( imageIcon ), component );
    }

    public NinePatchIcon ( Image image )
    {
        this ( image, null );
    }

    public NinePatchIcon ( Image image, Component component )
    {
        this ( ImageUtils.getBufferedImage ( image ), component );
    }

    public NinePatchIcon ( BufferedImage bufferedImage )
    {
        this ( bufferedImage, null );
    }

    public NinePatchIcon ( BufferedImage bufferedImage, Component component )
    {
        super ();

        // Componet for which this icon will be streched
        this.component = component;

        // Incorrect image
        if ( bufferedImage.getWidth () < 3 || bufferedImage.getHeight () < 3 )
        {
            throw new IllegalArgumentException ( "Buffered image must be atleast 3x3 pixels size" );
        }

        // Actual image
        image = bufferedImage.getSubimage ( 1, 1, bufferedImage.getWidth () - 2,
                bufferedImage.getHeight () - 2 );

        // Parsing streatch variables
        horizontalStretch = ImageUtils
                .parseIntervals ( bufferedImage, NinePatchIntervalType.horizontalStretch );
        verticalStretch =
                ImageUtils.parseIntervals ( bufferedImage, NinePatchIntervalType.verticalStretch );

        // Incorrect image
        if ( !( ( horizontalStretch.size () > 1 ||
                horizontalStretch.size () == 1 && !horizontalStretch.get ( 0 ).isPixel () ) &&
                ( verticalStretch.size () > 1 ||
                        verticalStretch.size () == 1 && !verticalStretch.get ( 0 ).isPixel () ) ) )
        {
            throw new IllegalArgumentException (
                    "There must be stretch constraints specified on image" );
        }

        // Parsing content margins
        List<NinePatchInterval> vc =
                ImageUtils.parseIntervals ( bufferedImage, NinePatchIntervalType.verticalContent );
        List<NinePatchInterval> hc = ImageUtils
                .parseIntervals ( bufferedImage, NinePatchIntervalType.horizontalContent );
        int top = vc.size () == 0 ? 0 : vc.get ( 0 ).getStart ();
        int bottom = vc.size () == 0 ? 0 : image.getHeight () - vc.get ( 0 ).getEnd () - 1;
        int left = hc.size () == 0 ? 0 : hc.get ( 0 ).getStart ();
        int right = hc.size () == 0 ? 0 : image.getWidth () - hc.get ( 0 ).getEnd () - 1;
        margin = new Insets ( top, left, bottom, right );
    }

    public BufferedImage getImage ()
    {
        return image;
    }

    public Component getComponent ()
    {
        return component;
    }

    public void setComponent ( Component component )
    {
        this.component = component;
    }

    public List<NinePatchInterval> getHorizontalStretch ()
    {
        return horizontalStretch;
    }

    public List<NinePatchInterval> getVerticalStretch ()
    {
        return verticalStretch;
    }

    public Insets getMargin ()
    {
        return margin;
    }

    public void setHorizontalStretch ( List<NinePatchInterval> horizontalStretch )
    {
        this.horizontalStretch = horizontalStretch;
    }

    public void addHorizontalStretch ( NinePatchInterval interval )
    {
        this.horizontalStretch.add ( interval );
    }

    public void setVerticalStretch ( List<NinePatchInterval> verticalStretch )
    {
        this.verticalStretch = verticalStretch;
    }

    public void addVerticalStretch ( NinePatchInterval interval )
    {
        this.verticalStretch.add ( interval );
    }

    public void setMargin ( Insets margin )
    {
        this.margin = margin;
    }

    public void paintIcon ( Component c, Graphics g, int x, int y )
    {
        Graphics2D g2d = ( Graphics2D ) g;

        final int componentWidth = Math.max ( c.getWidth (), getIconWidth () );
        final int componentHeight = Math.max ( c.getHeight (), getIconHeight () );
        final int fixedPixelsX = getFixedPixelsWidth ( false );
        final int unfixedX = componentWidth - fixedPixelsX;
        final int fixedPixelsY = getFixedPixelsHeight ( false );
        final int unfixedY = componentHeight - fixedPixelsY;

        int currentY = 0;
        for ( int j = 0; j < verticalStretch.size (); j++ )
        {
            NinePatchInterval intervalY = verticalStretch.get ( j );
            NinePatchInterval beforeY = j > 0 ? verticalStretch.get ( j - 1 ) : null;

            // Percent part height
            int intervalHeight = intervalY.getEnd () - intervalY.getStart () + 1;
            //                    ( j == verticalStretch.size () - 1 ? 1 : 0 );
            int finalHeight;
            if ( intervalY.isPixel () )
            {
                finalHeight = intervalHeight;
            }
            else
            {
                float percents = ( float ) intervalHeight / ( image.getHeight () - fixedPixelsY );
                finalHeight = Math.round ( percents * unfixedY );
            }

            int currentX = 0;
            for ( int i = 0; i < horizontalStretch.size (); i++ )
            {
                NinePatchInterval intervalX = horizontalStretch.get ( i );
                NinePatchInterval beforeX = i > 0 ? horizontalStretch.get ( i - 1 ) : null;

                // Percent part width
                int intervalWidth = intervalX.getEnd () - intervalX.getStart () + 1;
                //                        ( i == horizontalStretch.size () - 1 ? 1 : 0 );
                int finalWidth;
                if ( intervalX.isPixel () )
                {
                    finalWidth = intervalWidth;
                }
                else
                {
                    float percents = ( float ) intervalWidth / ( image.getWidth () - fixedPixelsX );
                    finalWidth = Math.round ( percents * unfixedX );
                }
                g2d.drawImage ( image, currentX, currentY, currentX + finalWidth,
                        currentY + finalHeight, intervalX.getStart (), intervalY.getStart (),
                        intervalX.getStart () + intervalWidth,
                        intervalY.getStart () + intervalHeight, null );

                // Icrementing current X
                currentX += finalWidth;
            }

            // Icrementing current Y
            currentY += finalHeight;
        }
    }

    private int getFixedPixelsHeight ( boolean addUnfixedSpaces )
    {
        int fixedPixelsY = image.getHeight ();
        for ( NinePatchInterval interval : verticalStretch )
        {
            if ( !interval.isPixel () )
            {
                fixedPixelsY -= interval.getEnd () - interval.getStart () + 1;
                if ( addUnfixedSpaces )
                {
                    fixedPixelsY += 1;
                }
            }
        }
        return fixedPixelsY;
    }

    private int getFixedPixelsWidth ( boolean addUnfixedSpaces )
    {
        int fixedPixelsX = image.getWidth ();
        for ( NinePatchInterval interval : horizontalStretch )
        {
            if ( !interval.isPixel () )
            {
                fixedPixelsX -= interval.getEnd () - interval.getStart () + 1;
                if ( addUnfixedSpaces )
                {
                    fixedPixelsX += 1;
                }
            }
        }
        return fixedPixelsX;
    }

    public int getIconWidth ()
    {
        return Math.max ( component != null ? component.getWidth () : 0,
                getFixedPixelsWidth ( true ) );
    }

    public int getIconHeight ()
    {
        return Math.max ( component != null ? component.getHeight () : 0,
                getFixedPixelsHeight ( true ) );
    }

    public Dimension getPreferredSize ()
    {
        return new Dimension ( getFixedPixelsWidth ( true ), getFixedPixelsHeight ( true ) );
    }
}
